#ifndef _CLUSAPI_H
#define _CLUSAPI_H

#ifdef __ORANGEC__ 
#pragma once
#endif

/* Microsoft Cluster Server definitions */

#include <winerror.h>

#define CLUSAPI_VERSION_SERVER2008  0x00000600
#define CLUSAPI_VERSION_SERVER2008R2  0x00000700
#define CLUSAPI_VERSION_WINDOWS8  0x00000701
#define CLUSAPI_VERSION_WINDOWSBLUE  0x00000702
#define CLUSAPI_VERSION_WINTHRESHOLD 0x00000703

#ifndef CLUSAPI_VERSION
#if (!defined(NTDDI_VERSION) || (NTDDI_VERSION >= NTDDI_WINTHRESHOLD))
#define CLUSAPI_VERSION  CLUSAPI_VERSION_WINTHRESHOLD
#elif (NTDDI_VERSION >= NTDDI_WINBLUE)
#define CLUSAPI_VERSION  CLUSAPI_VERSION_WINDOWSBLUE
#elif (NTDDI_VERSION >= NTDDI_WIN8)
#define CLUSAPI_VERSION  CLUSAPI_VERSION_WINDOWS8
#elif (NTDDI_VERSION >= NTDDI_WIN7)
#define CLUSAPI_VERSION  CLUSAPI_VERSION_SERVER2008R2
#else
#define CLUSAPI_VERSION CLUSAPI_VERSION_SERVER2008
#endif
#endif /* !CLUSAPI_VERSION */

#define CREATE_CLUSTER_VERSION  CLUSAPI_VERSION_SERVER2008
#define CREATE_CLUSTER_MAJOR_VERSION_MASK  0xFFFFFF00

#ifdef __cplusplus
extern "C" {
#endif


#define MAX_CLUSTERNAME_LENGTH  DNS_MAX_LABEL_LENGTH

#define CLUSTER_VERSION_FLAG_MIXED_MODE  0x00000001
#define CLUSREG_NAME_MIXED_MODE  L"MixedMode"

#define NT4_MAJOR_VERSION  1
#define NT4SP4_MAJOR_VERSION  2
#define NT5_MAJOR_VERSION  3
#define NT51_MAJOR_VERSION  4
#define NT6_MAJOR_VERSION  5
#define NT7_MAJOR_VERSION  6
#define NT8_MAJOR_VERSION  7
#define NT9_MAJOR_VERSION  8
#define NT10_MAJOR_VERSION  9

#define CLUSTER_VERSION_UNKNOWN  0xFFFFFFFF

#define CLUSTER_MAKE_VERSION(major,minor)  (((major)<<16)|(minor))
#define CLUSTER_GET_MAJOR_VERSION(ver)  ((ver)>>16)
#define CLUSTER_GET_MINOR_VERSION(ver)  ((ver)&0xFFFF)
#define CLUSTER_GET_UPGRADE_VERSION(ver)  ((ver) & 0xFFFF)

#define CLUSTER_INSTALLED  0x00000001
#define CLUSTER_CONFIGURED  0x00000002
#define CLUSTER_RUNNING  0x00000010

#define CLUSAPI_READ_ACCESS  0x00000001L
#define CLUSAPI_CHANGE_ACCESS  0x00000002L
#define CLUSAPI_NO_ACCESS  0x00000004L
#define CLUSAPI_ALL_ACCESS (CLUSAPI_READ_ACCESS|CLUSAPI_CHANGE_ACCESS)

#define CLUS_HYBRID_QUORUM  1024
#define CLUS_NODE_MAJORITY_QUORUM  0
#define CLUS_LEGACY_QUORUM  (4 * 1024 * 1024)

#define CLUSCTL_RESOURCE_STATE_CHANGE_REASON_VERSION_1  1

#define  CLUSREG_DATABASE_SYNC_WRITE_TO_ALL_NODES  1
#define  CLUSREG_DATABASE_ISOLATE_READ  2

#define CLUSTER_SET_ACCESS_TYPE_ALLOWED  0
#define CLUSTER_SET_ACCESS_TYPE_DENIED  1
#define CLUSTER_DELETE_ACCESS_CONTROL_ENTRY 2

#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_WINTHRESHOLD)
#define CLUSGROUPSET_STATUS_GROUPS_PENDING  0x0000000000000001
#define CLUSGROUPSET_STATUS_GROUPS_ONLINE  0x0000000000000002
#define CLUSGROUPSET_STATUS_OS_HEARTBEAT  0x0000000000000004
#define CLUSGROUPSET_STATUS_APPLICATION_READY  0x0000000000000008
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_WINTHRESHOLD */

#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8)

#define CLUSAPI_NODE_PAUSE_REMAIN_ON_PAUSED_NODE_ON_MOVE_ERROR  0x00000001

#define CLUSGRP_STATUS_LOCKED_MODE  0x0000000000000001
#define CLUSGRP_STATUS_PREEMPTED  0x0000000000000002
#define CLUSGRP_STATUS_WAITING_IN_QUEUE_FOR_MOVE  0x0000000000000004
#define CLUSGRP_STATUS_PHYSICAL_RESOURCES_LACKING  0x0000000000000008
#define CLUSGRP_STATUS_WAITING_TO_START  0x0000000000000010
#define CLUSGRP_STATUS_EMBEDDED_FAILURE  0x0000000000000020
#define CLUSGRP_STATUS_OFFLINE_DUE_TO_ANTIAFFINITY_CONFLICT  0x0000000000000040
#define CLUSGRP_STATUS_NETWORK_FAILURE  0x0000000000000080
#define CLUSGRP_STATUS_UNMONITORED  0x0000000000000100
#define CLUSGRP_STATUS_OS_HEARTBEAT  0x0000000000000200
#define CLUSGRP_STATUS_APPLICATION_READY  0x0000000000000400

#define CLUSRES_STATUS_LOCKED_MODE  0x0000000000000001
#define CLUSRES_STATUS_EMBEDDED_FAILURE  0x0000000000000002
#define CLUSRES_STATUS_FAILED_DUE_TO_INSUFFICIENT_CPU  0x0000000000000004
#define CLUSRES_STATUS_FAILED_DUE_TO_INSUFFICIENT_MEMORY  0x0000000000000008
#define CLUSRES_STATUS_FAILED_DUE_TO_INSUFFICIENT_GENERIC_RESOURCES  0x0000000000000010
#define CLUSRES_STATUS_NETWORK_FAILURE  0x0000000000000020
#define CLUSRES_STATUS_UNMONITORED  0x0000000000000040
#define CLUSRES_STATUS_OS_HEARTBEAT  0x0000000000000080
#define CLUSRES_STATUS_APPLICATION_READY  0x0000000000000100

#define CLUSAPI_GROUP_ONLINE_IGNORE_RESOURCE_STATUS  0x00000001
#define CLUSAPI_GROUP_ONLINE_SYNCHRONOUS  0x00000002
#define CLUSAPI_GROUP_ONLINE_BEST_POSSIBLE_NODE  0x00000004

#define CLUSAPI_GROUP_OFFLINE_IGNORE_RESOURCE_STATUS 0x00000001

#define CLUSAPI_RESOURCE_ONLINE_IGNORE_RESOURCE_STATUS  0x00000001
#define CLUSAPI_RESOURCE_ONLINE_DO_NOT_UPDATE_PERSISTENT_STATE  0x00000002
#define CLUSAPI_RESOURCE_ONLINE_NECESSARY_FOR_QUORUM  0x00000004
#define CLUSAPI_RESOURCE_ONLINE_BEST_POSSIBLE_NODE  0x00000008

#define CLUSAPI_RESOURCE_OFFLINE_IGNORE_RESOURCE_STATUS  0x00000001
#define CLUSAPI_RESOURCE_OFFLINE_FORCE_WITH_TERMINATION  0x00000002
#define CLUSAPI_RESOURCE_OFFLINE_DO_NOT_UPDATE_PERSISTENT_STATE  0x00000004

#define CLUSAPI_RESOURCE_OFFLINE_REASON_NONE  0x00000000
#define CLUSAPI_RESOURCE_OFFLINE_REASON_UNKNOWN  0x00000001
#define CLUSAPI_RESOURCE_OFFLINE_REASON_MOVING  0x00000002
#define CLUSAPI_RESOURCE_OFFLINE_REASON_USER_REQUESTED  0x00000004
#define CLUSAPI_RESOURCE_OFFLINE_REASON_BEING_DELETED  0x00000008
#define CLUSAPI_RESOURCE_OFFLINE_REASON_BEING_RESTARTED 0x00000010
#define CLUSAPI_RESOURCE_OFFLINE_REASON_PREEMPTED  0x00000020
#define CLUSAPI_RESOURCE_OFFLINE_REASON_SHUTTING_DOWN  0x00000040

#define CLUSAPI_GROUP_MOVE_IGNORE_RESOURCE_STATUS  0x00000001
#define CLUSAPI_GROUP_MOVE_RETURN_TO_SOURCE_NODE_ON_ERROR  0x00000002
#define CLUSAPI_GROUP_MOVE_QUEUE_ENABLED  0x00000004
#define CLUSAPI_GROUP_MOVE_HIGH_PRIORITY_START  0x00000008
#define CLUSAPI_GROUP_MOVE_FAILBACK  0x00000010

#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8 */

#define CLUSPROP_SYNTAX_VALUE(type,format)  ((DWORD)((type<<16)|format))

#define CLUS_ACCESS_ANY  0
#define CLUS_ACCESS_READ  0x01
#define CLUS_ACCESS_WRITE  0x02

#define CLUS_NO_MODIFY  0
#define CLUS_MODIFY  0x01

#define CLUS_NOT_GLOBAL  0
#define CLUS_GLOBAL  0x01

#define CLUSCTL_ACCESS_SHIFT  0
#define CLUSCTL_FUNCTION_SHIFT  2
#define CLCTL_INTERNAL_SHIFT  20
#define CLCTL_USER_SHIFT  21
#define CLCTL_MODIFY_SHIFT  22
#define CLCTL_GLOBAL_SHIFT  23
#define CLUSCTL_OBJECT_SHIFT  24

#define CLCTL_INTERNAL_MASK  (1<<CLCTL_INTERNAL_SHIFT)
#define CLCTL_USER_MASK  (1<<CLCTL_USER_SHIFT)
#define CLCTL_MODIFY_MASK  (1<<CLCTL_MODIFY_SHIFT)
#define CLCTL_GLOBAL_MASK  (1<<CLCTL_GLOBAL_SHIFT)
#define CLUSCTL_CONTROL_CODE_MASK  0x3FFFFF
#define CLUSCTL_OBJECT_MASK  0xFF
#define CLUSCTL_ACCESS_MODE_MASK  0x03

#define CLCTL_CLUSTER_BASE  0
#define CLCTL_USER_BASE  (1<<CLCTL_USER_SHIFT)

#define CLCTL_EXTERNAL_CODE(Function,Access,Modify)  (((Access)<<CLUSCTL_ACCESS_SHIFT)|((CLCTL_CLUSTER_BASE+Function)<<CLUSCTL_FUNCTION_SHIFT)|((Modify)<<CLCTL_MODIFY_SHIFT))
#define CLCTL_INTERNAL_CODE(Function,Access,Modify)  (((Access)<<CLUSCTL_ACCESS_SHIFT)|CLCTL_INTERNAL_MASK|((CLCTL_CLUSTER_BASE+Function)<<CLUSCTL_FUNCTION_SHIFT)|((Modify)<<CLCTL_MODIFY_SHIFT))

#define CLUSCTL_RESOURCE_CODE(Function)  (((CLUS_OBJECT_RESOURCE<<CLUSCTL_OBJECT_SHIFT)|Function))
#define CLUSCTL_RESOURCE_TYPE_CODE(Function)  (((CLUS_OBJECT_RESOURCE_TYPE<<CLUSCTL_OBJECT_SHIFT)|Function))
#define CLUSCTL_GROUP_CODE(Function)  (((CLUS_OBJECT_GROUP<<CLUSCTL_OBJECT_SHIFT)|Function))
#define CLUSCTL_NODE_CODE(Function)  (((CLUS_OBJECT_NODE<<CLUSCTL_OBJECT_SHIFT)|Function))
#define CLUSCTL_NETWORK_CODE(Function)  (((CLUS_OBJECT_NETWORK<<CLUSCTL_OBJECT_SHIFT)|Function))
#define CLUSCTL_NETINTERFACE_CODE(Function)  (((CLUS_OBJECT_NETINTERFACE<<CLUSCTL_OBJECT_SHIFT)|Function))
#define CLUSCTL_CLUSTER_CODE(Function)  (((CLUS_OBJECT_CLUSTER<<CLUSCTL_OBJECT_SHIFT)|Function))
#define CLUSCTL_GROUPSET_CODE(Function)  (((CLUS_OBJECT_GROUPSET << CLUSCTL_OBJECT_SHIFT) | Function))
#define CLUSCTL_USER_CODE(Function,Object)  (((Object)<<CLUSCTL_OBJECT_SHIFT)|(CLCTL_USER_BASE + Function))

#define CLUSCTL_GET_CONTROL_FUNCTION(ControlCode)  ((ControlCode>>CLUSCTL_ACCESS_SHIFT) & CLUSCTL_CONTROL_CODE_MASK)
#define CLUSCTL_GET_ACCESS_MODE(ControlCode)  ((ControlCode>>CLUSCTL_ACCESS_SHIFT) & CLUSCTL_ACCESS_MODE_MASK)
#define CLUSCTL_GET_CONTROL_OBJECT(ControlCode)  ((ControlCode>>CLUSCTL_OBJECT_SHIFT) & CLUSCTL_OBJECT_MASK)
#define CLUSCTL_GET_USER(ControlCode)  ((ControlCode & CLCTL_USER_MASK) >> CLCTL_USER_SHIFT)

#define ALIGN_CLUSPROP(count)  ((count + 3) & ~3)

#define CLUSPROP_BINARY_DECLARE(name,cb) \
    struct { \
        CLUSPROP_SYNTAX Syntax; \
        DWORD cbLength; \
        BYTE rgb[(cb + 3) & ~3]; \
    } name

#define CLUSPROP_SZ_DECLARE(name,cch)  \
    struct { \
        CLUSPROP_SYNTAX Syntax; \
        DWORD cbLength; \
        WCHAR sz[(cch + 1) & ~1]; \
    } name

#define CLUSPROP_PROPERTY_NAME_DECLARE(name,cch)  CLUSPROP_SZ_DECLARE(name,cch)

#ifndef _CLUSTER_API_TYPES_

typedef struct _HCLUSTER *HCLUSTER;
typedef struct _HNODE *HNODE;
typedef struct _HRESOURCE *HRESOURCE;
typedef struct _HGROUP *HGROUP;
typedef struct _HNETWORK *HNETWORK;
typedef struct _HNETINTERFACE *HNETINTERFACE;
typedef struct _HCHANGE *HCHANGE;
typedef struct _HCLUSENUM *HCLUSENUM;
typedef struct _HGROUPENUM *HGROUPENUM;
typedef struct _HRESENUM *HRESENUM;
typedef struct _HNETWORKENUM *HNETWORKENUM;
typedef struct _HNODEENUM *HNODEENUM;
typedef struct _HNETINTERFACEENUM *HNETINTERFACEENUM;
typedef struct _HRESTYPEENUM *HRESTYPEENUM;
typedef struct _HREGBATCH *HREGBATCH;
typedef struct _HREGBATCHPORT *HREGBATCHPORT;
typedef struct _HREGBATCHNOTIFICATION *HREGBATCHNOTIFICATION;
typedef struct _HREGREADBATCH *HREGREADBATCH;
typedef struct _HREGREADBATCHREPLY *HREGREADBATCHREPLY;

#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2)
typedef struct _HNODEENUMEX *HNODEENUMEX;
typedef struct _HCLUSENUMEX *HCLUSENUMEX;
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2 */

#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8)
typedef struct _HGROUPENUMEX *HGROUPENUMEX;
typedef struct _HRESENUMEX *HRESENUMEX;
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8 */

#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_WINTHRESHOLD)
typedef struct _HGROUPSET *HGROUPSET;
typedef struct _HGROUPSETENUM *HGROUPSETENUM;
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_WINTHRESHOLD */

typedef enum CLUSTER_QUORUM_TYPE {
    OperationalQuorum,
    ModifyQuorum
} CLUSTER_QUORUM_TYPE;

typedef struct CLUSTERVERSIONINFO_NT4 {
    DWORD dwVersionInfoSize;
    WORD MajorVersion;
    WORD MinorVersion;
    WORD BuildNumber;
    WCHAR szVendorId[64];
    WCHAR szCSDVersion[64];
} CLUSTERVERSIONINFO_NT4, *PCLUSTERVERSIONINFO_NT4;

typedef struct CLUSTERVERSIONINFO {
    DWORD dwVersionInfoSize;
    WORD MajorVersion;
    WORD MinorVersion;
    WORD BuildNumber;
    WCHAR szVendorId[64];
    WCHAR szCSDVersion[64];
    DWORD dwClusterHighestVersion;
    DWORD dwClusterLowestVersion;
    DWORD dwFlags;
    DWORD dwReserved;
} CLUSTERVERSIONINFO, *LPCLUSTERVERSIONINFO, *PCLUSTERVERSIONINFO;

typedef struct CLUS_STARTING_PARAMS {
    DWORD dwSize;
    BOOL bForm;
    BOOL bFirst;
} CLUS_STARTING_PARAMS, *PCLUS_STARTING_PARAMS;

typedef enum NODE_CLUSTER_STATE {
    ClusterStateNotInstalled = 0x00000000,
    ClusterStateNotConfigured = CLUSTER_INSTALLED,
    ClusterStateNotRunning = CLUSTER_INSTALLED|CLUSTER_CONFIGURED,
    ClusterStateRunning = CLUSTER_INSTALLED|CLUSTER_CONFIGURED|CLUSTER_RUNNING
} NODE_CLUSTER_STATE;

typedef enum CLUSTER_RESOURCE_STATE_CHANGE_REASON {
    eResourceStateChangeReasonUnknown,
    eResourceStateChangeReasonMove,
    eResourceStateChangeReasonFailover,
    eResourceStateChangeReasonFailedMove,
    eResourceStateChangeReasonShutdown,
    eResourceStateChangeReasonRundown
} CLUSTER_RESOURCE_STATE_CHANGE_REASON;

typedef enum _CLUSTER_REG_COMMAND {
    CLUSREG_COMMAND_NONE = 0,
    CLUSREG_SET_VALUE = 1,
    CLUSREG_CREATE_KEY,
    CLUSREG_DELETE_KEY,
    CLUSREG_DELETE_VALUE,
    CLUSREG_SET_KEY_SECURITY,
    CLUSREG_VALUE_DELETED,
    CLUSREG_READ_KEY,
    CLUSREG_READ_VALUE,
    CLUSREG_READ_ERROR,
    CLUSREG_CONTROL_COMMAND,
    CLUSREG_CONDITION_EXISTS,
    CLUSREG_CONDITION_NOT_EXISTS,
    CLUSREG_CONDITION_IS_EQUAL,
    CLUSREG_CONDITION_IS_NOT_EQUAL,
    CLUSREG_CONDITION_IS_GREATER_THAN,
    CLUSREG_CONDITION_IS_LESS_THAN,
    CLUSREG_CONDITION_KEY_EXISTS,
    CLUSREG_CONDITION_KEY_NOT_EXISTS,
    CLUSREG_LAST_COMMAND
} CLUSTER_REG_COMMAND;

typedef struct _CLUSCTL_RESOURCE_STATE_CHANGE_REASON_STRUCT {
    DWORD dwSize;
    DWORD dwVersion;
    CLUSTER_RESOURCE_STATE_CHANGE_REASON eReason;
} CLUSCTL_RESOURCE_STATE_CHANGE_REASON_STRUCT, *PCLUSCTL_RESOURCE_STATE_CHANGE_REASON_STRUCT;

typedef struct _CLUSTER_BATCH_COMMAND {
    CLUSTER_REG_COMMAND Command;
    DWORD dwOptions;
    LPCWSTR wzName;
    BYTE CONST *lpData;
    DWORD cbData;
} CLUSTER_BATCH_COMMAND;

typedef struct _CLUSTER_READ_BATCH_COMMAND {
    CLUSTER_REG_COMMAND Command;
    DWORD dwOptions;
    LPCWSTR wzSubkeyName;
    LPCWSTR wzValueName;
    BYTE CONST *lpData;
    DWORD cbData;
} CLUSTER_READ_BATCH_COMMAND;

#if CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2

#define CLUSTER_ENUM_ITEM_VERSION_1  0x00000001
#define CLUSTER_ENUM_ITEM_VERSION  CLUSTER_ENUM_ITEM_VERSION_1

typedef struct _CLUSTER_ENUM_ITEM {
    DWORD dwVersion;
    DWORD dwType;
    DWORD cbId;
    LPWSTR lpszId;
    DWORD cbName;
    LPWSTR lpszName;
} CLUSTER_ENUM_ITEM, *PCLUSTER_ENUM_ITEM;

#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2 */

typedef enum {
    ClusGroupTypeCoreCluster = 1,
    ClusGroupTypeAvailableStorage = 2,
    ClusGroupTypeTemporary = 3,
    ClusGroupTypeSharedVolume = 4,
    ClusGroupTypeStoragePool = 5,
    ClusGroupTypeFileServer = 100,
    ClusGroupTypePrintServer = 101,
    ClusGroupTypeDhcpServer = 102,
    ClusGroupTypeDtc = 103,
    ClusGroupTypeMsmq = 104,
    ClusGroupTypeWins = 105,
    ClusGroupTypeStandAloneDfs = 106,
    ClusGroupTypeGenericApplication = 107,
    ClusGroupTypeGenericService = 108,
    ClusGroupTypeGenericScript = 109,
    ClusGroupTypeIScsiNameService = 110,
    ClusGroupTypeVirtualMachine = 111,
    ClusGroupTypeTsSessionBroker = 112,
    ClusGroupTypeIScsiTarget = 113,
    ClusGroupTypeScaleoutFileServer = 114,
    ClusGroupTypeVMReplicaBroker = 115,
    ClusGroupTypeTaskScheduler = 116,
    ClusGroupTypeClusterUpdateAgent = 117,
    ClusGroupTypeScaleoutCluster = 118,
    ClusGroupTypeStorageReplica = 119,
    ClusGroupTypeVMReplicaCoordinator = 120,
    ClusGroupTypeCrossClusterOrchestrator = 121,
    ClusGroupTypeUnknown = 9999
} CLUSGROUP_TYPE, *PCLUSGROUP_TYPE;

#if CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8

#define CLUSTER_CREATE_GROUP_INFO_VERSION_1  0x00000001
#define CLUSTER_CREATE_GROUP_INFO_VERSION  CLUSTER_CREATE_GROUP_INFO_VERSION_1

typedef struct _CLUSTER_CREATE_GROUP_INFO {
    DWORD dwVersion;
    CLUSGROUP_TYPE groupType;
}CLUSTER_CREATE_GROUP_INFO, *PCLUSTER_CREATE_GROUP_INFO;

#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8 */

typedef enum {
    CLUSTER_MGMT_POINT_TYPE_NONE = 0,
    CLUSTER_MGMT_POINT_TYPE_CNO,
    CLUSTER_MGMT_POINT_TYPE_DNS_ONLY,
    CLUSTER_MGMT_POINT_TYPE_CNO_ONLY
} CLUSTER_MGMT_POINT_TYPE;

#if CLUSAPI_VERSION >= CLUSAPI_VERSION_WINTHRESHOLD

#define GROUPSET_READY_SETTING_DELAY  0x00000001
#define GROUPSET_READY_SETTING_ONLINE  0x00000002
#define GROUPSET_READY_SETTING_OS_HEARTBEAT  0x00000003
#define GROUPSET_READY_SETTING_APPLICATION_READY  0x00000004

typedef enum {
    CLUS_GROUP_START_ALWAYS = 0,
    CLUS_GROUP_DO_NOT_START = 1,
    CLUS_GROUP_START_ALLOWED = 2
} CLUS_GROUP_START_SETTING;

#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_WINTHRESHOLD */

#endif /* !_CLUSTER_API_TYPES_ */

typedef enum CLUSTER_QUORUM_VALUE {
    CLUSTER_QUORUM_MAINTAINED = 0,
    CLUSTER_QUORUM_LOST = 1,
} CLUSTER_QUORUM_VALUE;

#define MORON_SIZE  1

typedef struct _CLUSTER_VALIDATE_PATH {
    WCHAR szPath[ MORON_SIZE ];
} CLUSTER_VALIDATE_PATH, *PCLUSTER_VALIDATE_PATH;

typedef struct _CLUSTER_VALIDATE_DIRECTORY {
    WCHAR szPath[ MORON_SIZE ];
} CLUSTER_VALIDATE_DIRECTORY, *PCLUSTER_VALIDATE_DIRECTORY;

typedef struct _CLUSTER_VALIDATE_NETNAME {
    WCHAR szNetworkName[ MORON_SIZE ];
} CLUSTER_VALIDATE_NETNAME , *PCLUSTER_VALIDATE_NETNAME;

typedef struct _CLUSTER_VALIDATE_CSV_FILENAME {
    WCHAR szFileName[ MORON_SIZE ];
} CLUSTER_VALIDATE_CSV_FILENAME , *PCLUSTER_VALIDATE_CSV_FILENAME ;

#undef MORON_SIZE

typedef struct CLUSTER_SET_PASSWORD_STATUS {
    DWORD NodeId;
    BOOLEAN SetAttempted;
    DWORD ReturnStatus;
} CLUSTER_SET_PASSWORD_STATUS, *PCLUSTER_SET_PASSWORD_STATUS;

#ifndef _CLUSTER_API_TYPES_

typedef struct _CLUSTER_IP_ENTRY {
    PCWSTR lpszIpAddress;
    DWORD dwPrefixLength;
} CLUSTER_IP_ENTRY, *PCLUSTER_IP_ENTRY;

typedef struct _CREATE_CLUSTER_CONFIG {
    DWORD dwVersion;
    PCWSTR lpszClusterName;
    DWORD cNodes;
    PCWSTR *ppszNodeNames;
    DWORD cIpEntries;
    PCLUSTER_IP_ENTRY pIpEntries;
    BOOLEAN fEmptyCluster;
} CREATE_CLUSTER_CONFIG, *PCREATE_CLUSTER_CONFIG;

typedef struct _CREATE_CLUSTER_NAME_ACCOUNT {
    DWORD dwVersion;
    PCWSTR lpszClusterName;
    DWORD dwFlags;
    PCWSTR pszUserName;
    PCWSTR pszPassword;
    PCWSTR pszDomain;
    CLUSTER_MGMT_POINT_TYPE managementPointType;
} CREATE_CLUSTER_NAME_ACCOUNT, *PCREATE_CLUSTER_NAME_ACCOUNT;

#endif /* !_CLUSTER_API_TYPES_ */

typedef enum _CLUSTER_UPGRADE_PHASE {
    ClusterUpgradePhaseInitialize = 1,
    ClusterUpgradePhaseValidatingUpgrade = 2,
    ClusterUpgradePhaseUpgradingComponents = 3,
    ClusterUpgradePhaseInstallingNewComponents = 4,
    ClusterUpgradePhaseUpgradeComplete = 5
} CLUSTER_UPGRADE_PHASE;

typedef enum CLUSTER_CHANGE {
    CLUSTER_CHANGE_NODE_STATE = 0x00000001,
    CLUSTER_CHANGE_NODE_DELETED = 0x00000002,
    CLUSTER_CHANGE_NODE_ADDED = 0x00000004,
    CLUSTER_CHANGE_NODE_PROPERTY = 0x00000008,
    CLUSTER_CHANGE_REGISTRY_NAME = 0x00000010,
    CLUSTER_CHANGE_REGISTRY_ATTRIBUTES = 0x00000020,
    CLUSTER_CHANGE_REGISTRY_VALUE = 0x00000040,
    CLUSTER_CHANGE_REGISTRY_SUBTREE = 0x00000080,
    CLUSTER_CHANGE_RESOURCE_STATE = 0x00000100,
    CLUSTER_CHANGE_RESOURCE_DELETED = 0x00000200,
    CLUSTER_CHANGE_RESOURCE_ADDED = 0x00000400,
    CLUSTER_CHANGE_RESOURCE_PROPERTY = 0x00000800,
    CLUSTER_CHANGE_GROUP_STATE = 0x00001000,
    CLUSTER_CHANGE_GROUP_DELETED = 0x00002000,
    CLUSTER_CHANGE_GROUP_ADDED = 0x00004000,
    CLUSTER_CHANGE_GROUP_PROPERTY = 0x00008000,
    CLUSTER_CHANGE_RESOURCE_TYPE_DELETED = 0x00010000,
    CLUSTER_CHANGE_RESOURCE_TYPE_ADDED = 0x00020000,
    CLUSTER_CHANGE_RESOURCE_TYPE_PROPERTY = 0x00040000,
    CLUSTER_CHANGE_CLUSTER_RECONNECT = 0x00080000,
    CLUSTER_CHANGE_NETWORK_STATE = 0x00100000,
    CLUSTER_CHANGE_NETWORK_DELETED = 0x00200000,
    CLUSTER_CHANGE_NETWORK_ADDED = 0x00400000,
    CLUSTER_CHANGE_NETWORK_PROPERTY = 0x00800000,
    CLUSTER_CHANGE_NETINTERFACE_STATE = 0x01000000,
    CLUSTER_CHANGE_NETINTERFACE_DELETED = 0x02000000,
    CLUSTER_CHANGE_NETINTERFACE_ADDED = 0x04000000,
    CLUSTER_CHANGE_NETINTERFACE_PROPERTY = 0x08000000,
    CLUSTER_CHANGE_QUORUM_STATE = 0x10000000,
    CLUSTER_CHANGE_CLUSTER_STATE = 0x20000000,
    CLUSTER_CHANGE_CLUSTER_PROPERTY = 0x40000000,
    CLUSTER_CHANGE_HANDLE_CLOSE = 0x80000000,
    CLUSTER_CHANGE_ALL = (CLUSTER_CHANGE_NODE_STATE | CLUSTER_CHANGE_NODE_DELETED | CLUSTER_CHANGE_NODE_ADDED | CLUSTER_CHANGE_NODE_PROPERTY |
                          CLUSTER_CHANGE_REGISTRY_NAME | CLUSTER_CHANGE_REGISTRY_ATTRIBUTES | CLUSTER_CHANGE_REGISTRY_VALUE |
                          CLUSTER_CHANGE_REGISTRY_SUBTREE | CLUSTER_CHANGE_RESOURCE_STATE | CLUSTER_CHANGE_RESOURCE_DELETED |
                          CLUSTER_CHANGE_RESOURCE_ADDED | CLUSTER_CHANGE_RESOURCE_PROPERTY | CLUSTER_CHANGE_GROUP_STATE |
                          CLUSTER_CHANGE_GROUP_DELETED | CLUSTER_CHANGE_GROUP_ADDED | CLUSTER_CHANGE_GROUP_PROPERTY |
                          CLUSTER_CHANGE_RESOURCE_TYPE_DELETED | CLUSTER_CHANGE_RESOURCE_TYPE_ADDED | CLUSTER_CHANGE_RESOURCE_TYPE_PROPERTY |
                          CLUSTER_CHANGE_NETWORK_STATE | CLUSTER_CHANGE_NETWORK_DELETED | CLUSTER_CHANGE_NETWORK_ADDED |
                          CLUSTER_CHANGE_NETWORK_PROPERTY | CLUSTER_CHANGE_NETINTERFACE_STATE | CLUSTER_CHANGE_NETINTERFACE_DELETED |
                          CLUSTER_CHANGE_NETINTERFACE_ADDED | CLUSTER_CHANGE_NETINTERFACE_PROPERTY | CLUSTER_CHANGE_QUORUM_STATE |
                          CLUSTER_CHANGE_CLUSTER_STATE | CLUSTER_CHANGE_CLUSTER_PROPERTY | CLUSTER_CHANGE_CLUSTER_RECONNECT | CLUSTER_CHANGE_HANDLE_CLOSE)
} CLUSTER_CHANGE;

#ifndef _CLUSTER_API_TYPES_

#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8)

typedef enum CLUSTER_NOTIFICATIONS_VERSION {
    CLUSTER_NOTIFICATIONS_V1 = 0x00000001,
    CLUSTER_NOTIFICATIONS_V2 = 0x00000002
} CLUSTER_NOTIFICATIONS_VERSION;

typedef enum CLUSTER_CHANGE_CLUSTER_V2 {
    CLUSTER_CHANGE_CLUSTER_RECONNECT_V2 = 0x00000001,
    CLUSTER_CHANGE_CLUSTER_STATE_V2 = 0x00000002,
    CLUSTER_CHANGE_CLUSTER_GROUP_ADDED_V2 = 0x00000004,
    CLUSTER_CHANGE_CLUSTER_HANDLE_CLOSE_V2 = 0x00000008,
    CLUSTER_CHANGE_CLUSTER_NETWORK_ADDED_V2 = 0x00000010,
    CLUSTER_CHANGE_CLUSTER_NODE_ADDED_V2 = 0x00000020,
    CLUSTER_CHANGE_CLUSTER_RESOURCE_TYPE_ADDED_V2 = 0x00000040,
    CLUSTER_CHANGE_CLUSTER_COMMON_PROPERTY_V2 = 0x00000080,
    CLUSTER_CHANGE_CLUSTER_PRIVATE_PROPERTY_V2 = 0x00000100,
    CLUSTER_CHANGE_CLUSTER_LOST_NOTIFICATIONS_V2 = 0x00000200,
    CLUSTER_CHANGE_CLUSTER_RENAME_V2 = 0x00000400,
    CLUSTER_CHANGE_CLUSTER_MEMBERSHIP_V2 = 0x00000800,
    CLUSTER_CHANGE_CLUSTER_UPGRADED_V2 = 0x00001000,
    CLUSTER_CHANGE_CLUSTER_ALL_V2 = (CLUSTER_CHANGE_CLUSTER_RECONNECT_V2 | CLUSTER_CHANGE_CLUSTER_STATE_V2 | CLUSTER_CHANGE_CLUSTER_GROUP_ADDED_V2 | 
                                     CLUSTER_CHANGE_CLUSTER_HANDLE_CLOSE_V2 | CLUSTER_CHANGE_CLUSTER_NETWORK_ADDED_V2 | CLUSTER_CHANGE_CLUSTER_NODE_ADDED_V2 |
                                     CLUSTER_CHANGE_CLUSTER_RESOURCE_TYPE_ADDED_V2 | CLUSTER_CHANGE_CLUSTER_COMMON_PROPERTY_V2 | 
                                     CLUSTER_CHANGE_CLUSTER_PRIVATE_PROPERTY_V2 | CLUSTER_CHANGE_CLUSTER_LOST_NOTIFICATIONS_V2 |
                                     CLUSTER_CHANGE_CLUSTER_RENAME_V2 | CLUSTER_CHANGE_CLUSTER_MEMBERSHIP_V2 | CLUSTER_CHANGE_CLUSTER_UPGRADED_V2)
} CLUSTER_CHANGE_CLUSTER_V2;

typedef enum CLUSTER_CHANGE_GROUP_V2 {
    CLUSTER_CHANGE_GROUP_DELETED_V2 = 0x00000001,
    CLUSTER_CHANGE_GROUP_COMMON_PROPERTY_V2 = 0x00000002,
    CLUSTER_CHANGE_GROUP_PRIVATE_PROPERTY_V2 = 0x00000004,
    CLUSTER_CHANGE_GROUP_STATE_V2 = 0x00000008,
    CLUSTER_CHANGE_GROUP_OWNER_NODE_V2 = 0x00000010,
    CLUSTER_CHANGE_GROUP_PREFERRED_OWNERS_V2 = 0x00000020,
    CLUSTER_CHANGE_GROUP_RESOURCE_ADDED_V2 = 0x00000040,
    CLUSTER_CHANGE_GROUP_RESOURCE_GAINED_V2 = 0x00000080,
    CLUSTER_CHANGE_GROUP_RESOURCE_LOST_V2 = 0x00000100,
    CLUSTER_CHANGE_GROUP_HANDLE_CLOSE_V2 = 0x00000200,
    CLUSTER_CHANGE_GROUP_ALL_V2 = (CLUSTER_CHANGE_GROUP_DELETED_V2 | CLUSTER_CHANGE_GROUP_COMMON_PROPERTY_V2 | CLUSTER_CHANGE_GROUP_PRIVATE_PROPERTY_V2 |
                                   CLUSTER_CHANGE_GROUP_STATE_V2 | CLUSTER_CHANGE_GROUP_OWNER_NODE_V2 | CLUSTER_CHANGE_GROUP_PREFERRED_OWNERS_V2 |
                                   CLUSTER_CHANGE_GROUP_RESOURCE_ADDED_V2 | CLUSTER_CHANGE_GROUP_RESOURCE_GAINED_V2 | CLUSTER_CHANGE_GROUP_RESOURCE_LOST_V2 |
                                   CLUSTER_CHANGE_GROUP_HANDLE_CLOSE_V2)
} CLUSTER_CHANGE_GROUP_V2;

typedef enum CLUSTER_CHANGE_GROUPSET_V2 {
    CLUSTER_CHANGE_GROUPSET_DELETED_v2 = 0x00000001,
    CLUSTER_CHANGE_GROUPSET_COMMON_PROPERTY_V2 = 0x00000002,
    CLUSTER_CHANGE_GROUPSET_PRIVATE_PROPERTY_V2 = 0x00000004,
    CLUSTER_CHANGE_GROUPSET_STATE_V2 = 0x00000008,
    CLUSTER_CHANGE_GROUPSET_GROUP_ADDED = 0x00000010,
    CLUSTER_CHANGE_GROUPSET_GROUP_REMOVED = 0x00000020,
    CLUSTER_CHANGE_GROUPSET_DEPENDENCIES_V2 = 0x00000040,
    CLUSTER_CHANGE_GROUPSET_DEPENDENTS_V2 = 0x00000080,
    CLUSTER_CHANGE_GROUPSET_HANDLE_CLOSE_v2 = 0x00000100,
    CLUSTER_CHANGE_GROUPSET_ALL_V2 = (CLUSTER_CHANGE_GROUPSET_DELETED_v2 | CLUSTER_CHANGE_GROUPSET_COMMON_PROPERTY_V2 |
                                      CLUSTER_CHANGE_GROUPSET_PRIVATE_PROPERTY_V2 | CLUSTER_CHANGE_GROUPSET_STATE_V2 |
                                      CLUSTER_CHANGE_GROUPSET_GROUP_ADDED | CLUSTER_CHANGE_GROUPSET_GROUP_REMOVED |
                                      CLUSTER_CHANGE_GROUPSET_DEPENDENCIES_V2 | CLUSTER_CHANGE_GROUPSET_DEPENDENTS_V2 |
                                      CLUSTER_CHANGE_GROUPSET_HANDLE_CLOSE_v2)
} CLUSTER_CHANGE_GROUPSET_V2;

typedef enum CLUSTER_CHANGE_RESOURCE_V2 {
    CLUSTER_CHANGE_RESOURCE_COMMON_PROPERTY_V2 = 0x00000001,
    CLUSTER_CHANGE_RESOURCE_PRIVATE_PROPERTY_V2 = 0x00000002,
    CLUSTER_CHANGE_RESOURCE_STATE_V2 = 0x00000004,
    CLUSTER_CHANGE_RESOURCE_OWNER_GROUP_V2 = 0x00000008,
    CLUSTER_CHANGE_RESOURCE_DEPENDENCIES_V2 = 0x00000010,
    CLUSTER_CHANGE_RESOURCE_DEPENDENTS_V2 = 0x00000020,
    CLUSTER_CHANGE_RESOURCE_POSSIBLE_OWNERS_V2 = 0x00000040,
    CLUSTER_CHANGE_RESOURCE_DELETED_V2 = 0x00000080,
    CLUSTER_CHANGE_RESOURCE_DLL_UPGRADED_V2 = 0x00000100,
    CLUSTER_CHANGE_RESOURCE_HANDLE_CLOSE_V2 = 0x00000200,
    CLUSTER_CHANGE_RESOURCE_TERMINAL_STATE_V2 = 0X00000400,
    CLUSTER_CHANGE_RESOURCE_ALL_V2 = (CLUSTER_CHANGE_RESOURCE_COMMON_PROPERTY_V2 | CLUSTER_CHANGE_RESOURCE_PRIVATE_PROPERTY_V2 |
                                      CLUSTER_CHANGE_RESOURCE_STATE_V2 | CLUSTER_CHANGE_RESOURCE_OWNER_GROUP_V2 |
                                      CLUSTER_CHANGE_RESOURCE_DEPENDENCIES_V2 | CLUSTER_CHANGE_RESOURCE_DEPENDENTS_V2 |
                                      CLUSTER_CHANGE_RESOURCE_POSSIBLE_OWNERS_V2 | CLUSTER_CHANGE_RESOURCE_DELETED_V2 |
                                      CLUSTER_CHANGE_RESOURCE_DLL_UPGRADED_V2 | CLUSTER_CHANGE_RESOURCE_HANDLE_CLOSE_V2 |
                                      CLUSTER_CHANGE_RESOURCE_TERMINAL_STATE_V2)
} CLUSTER_CHANGE_RESOURCE_V2;

typedef enum CLUSTER_CHANGE_RESOURCE_TYPE_V2 {
    CLUSTER_CHANGE_RESOURCE_TYPE_DELETED_V2 = 0x00000001,
    CLUSTER_CHANGE_RESOURCE_TYPE_COMMON_PROPERTY_V2 = 0x00000002,
    CLUSTER_CHANGE_RESOURCE_TYPE_PRIVATE_PROPERTY_V2 = 0x00000004,
    CLUSTER_CHANGE_RESOURCE_TYPE_POSSIBLE_OWNERS_V2 = 0x00000008,
    CLUSTER_CHANGE_RESOURCE_TYPE_DLL_UPGRADED_V2 = 0x00000010,
    CLUSTER_RESOURCE_TYPE_SPECIFIC_V2 = 0x00000020,
    CLUSTER_CHANGE_RESOURCE_TYPE_ALL_V2 = (CLUSTER_CHANGE_RESOURCE_TYPE_DELETED_V2 | CLUSTER_CHANGE_RESOURCE_TYPE_COMMON_PROPERTY_V2 |
                                           CLUSTER_CHANGE_RESOURCE_TYPE_PRIVATE_PROPERTY_V2 | CLUSTER_CHANGE_RESOURCE_TYPE_POSSIBLE_OWNERS_V2 |
                                           CLUSTER_CHANGE_RESOURCE_TYPE_DLL_UPGRADED_V2 | CLUSTER_RESOURCE_TYPE_SPECIFIC_V2)
} CLUSTER_CHANGE_RESOURCE_TYPE_V2;

typedef enum CLUSTER_CHANGE_NETINTERFACE_V2 {
    CLUSTER_CHANGE_NETINTERFACE_DELETED_V2 = 0x00000001,
    CLUSTER_CHANGE_NETINTERFACE_COMMON_PROPERTY_V2 = 0x00000002,
    CLUSTER_CHANGE_NETINTERFACE_PRIVATE_PROPERTY_V2 = 0x00000004,
    CLUSTER_CHANGE_NETINTERFACE_STATE_V2 = 0x00000008,
    CLUSTER_CHANGE_NETINTERFACE_HANDLE_CLOSE_V2 = 0x00000010,
    CLUSTER_CHANGE_NETINTERFACE_ALL_V2 = (CLUSTER_CHANGE_NETINTERFACE_DELETED_V2 | CLUSTER_CHANGE_NETINTERFACE_COMMON_PROPERTY_V2 |
                                          CLUSTER_CHANGE_NETINTERFACE_PRIVATE_PROPERTY_V2 | CLUSTER_CHANGE_NETINTERFACE_STATE_V2 |
                                          CLUSTER_CHANGE_NETINTERFACE_HANDLE_CLOSE_V2)
} CLUSTER_CHANGE_NETINTERFACE_V2;

typedef enum CLUSTER_CHANGE_NETWORK_V2 {
    CLUSTER_CHANGE_NETWORK_DELETED_V2 = 0x00000001,
    CLUSTER_CHANGE_NETWORK_COMMON_PROPERTY_V2 = 0x00000002,
    CLUSTER_CHANGE_NETWORK_PRIVATE_PROPERTY_V2 = 0x00000004,
    CLUSTER_CHANGE_NETWORK_STATE_V2 = 0x00000008,
    CLUSTER_CHANGE_NETWORK_HANDLE_CLOSE_V2 = 0x00000010,
    CLUSTER_CHANGE_NETWORK_ALL_V2 = (CLUSTER_CHANGE_NETWORK_DELETED_V2 | CLUSTER_CHANGE_NETWORK_COMMON_PROPERTY_V2 |
                                     CLUSTER_CHANGE_NETWORK_PRIVATE_PROPERTY_V2 | CLUSTER_CHANGE_NETWORK_STATE_V2 |
                                     CLUSTER_CHANGE_NETWORK_HANDLE_CLOSE_V2)
} CLUSTER_CHANGE_NETWORK_V2;

typedef enum CLUSTER_CHANGE_NODE_V2 {
    CLUSTER_CHANGE_NODE_NETINTERFACE_ADDED_V2 = 0x00000001,
    CLUSTER_CHANGE_NODE_DELETED_V2 = 0x00000002,
    CLUSTER_CHANGE_NODE_COMMON_PROPERTY_V2 = 0x00000004,
    CLUSTER_CHANGE_NODE_PRIVATE_PROPERTY_V2 = 0x00000008,
    CLUSTER_CHANGE_NODE_STATE_V2 = 0x00000010,
    CLUSTER_CHANGE_NODE_GROUP_GAINED_V2 = 0x00000020,
    CLUSTER_CHANGE_NODE_GROUP_LOST_V2 = 0x00000040,
    CLUSTER_CHANGE_NODE_HANDLE_CLOSE_V2 = 0x00000080,
    CLUSTER_CHANGE_NODE_ALL_V2 = (CLUSTER_CHANGE_NODE_NETINTERFACE_ADDED_V2 | CLUSTER_CHANGE_NODE_DELETED_V2 |
                                  CLUSTER_CHANGE_NODE_COMMON_PROPERTY_V2 | CLUSTER_CHANGE_NODE_PRIVATE_PROPERTY_V2 |
                                  CLUSTER_CHANGE_NODE_STATE_V2 | CLUSTER_CHANGE_NODE_GROUP_GAINED_V2 |
                                  CLUSTER_CHANGE_NODE_GROUP_LOST_V2 | CLUSTER_CHANGE_NODE_HANDLE_CLOSE_V2)
} CLUSTER_CHANGE_NODE_V2;

typedef enum CLUSTER_CHANGE_REGISTRY_V2 {
    CLUSTER_CHANGE_REGISTRY_ATTRIBUTES_V2 = 0x00000001,
    CLUSTER_CHANGE_REGISTRY_NAME_V2 = 0x00000002,
    CLUSTER_CHANGE_REGISTRY_SUBTREE_V2 = 0x00000004,
    CLUSTER_CHANGE_REGISTRY_VALUE_V2 = 0x00000008,
    CLUSTER_CHANGE_REGISTRY_HANDLE_CLOSE_V2 = 0x00000010,
    CLUSTER_CHANGE_REGISTRY_ALL_V2 = (CLUSTER_CHANGE_REGISTRY_ATTRIBUTES_V2 | CLUSTER_CHANGE_REGISTRY_NAME_V2 |
                                      CLUSTER_CHANGE_REGISTRY_SUBTREE_V2 | CLUSTER_CHANGE_REGISTRY_VALUE_V2 |
                                      CLUSTER_CHANGE_REGISTRY_HANDLE_CLOSE_V2)
} CLUSTER_CHANGE_REGISTRY_V2;

typedef enum CLUSTER_CHANGE_QUORUM_V2 {
    CLUSTER_CHANGE_QUORUM_STATE_V2 = 0x00000001,
    CLUSTER_CHANGE_QUORUM_ALL_V2 = (CLUSTER_CHANGE_QUORUM_STATE_V2)
} CLUSTER_CHANGE_QUORUM_V2;

typedef enum CLUSTER_CHANGE_SHARED_VOLUME_V2 {
    CLUSTER_CHANGE_SHARED_VOLUME_STATE_V2 = 0x00000001,
    CLUSTER_CHANGE_SHARED_VOLUME_ADDED_V2 = 0x00000002,
    CLUSTER_CHANGE_SHARED_VOLUME_REMOVED_V2 = 0x00000004,
    CLUSTER_CHANGE_SHARED_VOLUME_ALL_V2 = (CLUSTER_CHANGE_SHARED_VOLUME_STATE_V2 | CLUSTER_CHANGE_SHARED_VOLUME_ADDED_V2 | CLUSTER_CHANGE_SHARED_VOLUME_REMOVED_V2)
} CLUSTER_CHANGE_SHARED_VOLUME_V2;

typedef enum CLUSTER_CHANGE_SPACEPORT_V2 {
    CLUSTER_CHANGE_SPACEPORT_CUSTOM_PNP_V2 = 0x00000001,
} CLUSTER_CHANGE_SPACEPORT_V2;

typedef enum CLUSTER_CHANGE_NODE_UPGRADE_PHASE_V2 {
    CLUSTER_CHANGE_UPGRADE_NODE_PREPARE = 0x00000001,
    CLUSTER_CHANGE_UPGRADE_NODE_COMMIT = 0x00000002,
    CLUSTER_CHANGE_UPGRADE_NODE_POSTCOMMIT = 0x00000004,
    CLUSTER_CHANGE_UPGRADE_ALL = (CLUSTER_CHANGE_UPGRADE_NODE_PREPARE | CLUSTER_CHANGE_UPGRADE_NODE_COMMIT | CLUSTER_CHANGE_UPGRADE_NODE_POSTCOMMIT)
} CLUSTER_CHANGE_NODE_UPGRADE_PHASE_V2;

typedef enum CLUSTER_OBJECT_TYPE {
    CLUSTER_OBJECT_TYPE_NONE = 0x00000000,
    CLUSTER_OBJECT_TYPE_CLUSTER = 0x00000001,
    CLUSTER_OBJECT_TYPE_GROUP = 0x00000002,
    CLUSTER_OBJECT_TYPE_RESOURCE = 0x00000003,
    CLUSTER_OBJECT_TYPE_RESOURCE_TYPE = 0x00000004,
    CLUSTER_OBJECT_TYPE_NETWORK_INTERFACE = 0x00000005,
    CLUSTER_OBJECT_TYPE_NETWORK = 0x00000006,
    CLUSTER_OBJECT_TYPE_NODE = 0x00000007,
    CLUSTER_OBJECT_TYPE_REGISTRY = 0x00000008,
    CLUSTER_OBJECT_TYPE_QUORUM = 0x00000009,
    CLUSTER_OBJECT_TYPE_SHARED_VOLUME = 0x0000000a,
    CLUSTER_OBJECT_TYPE_GROUPSET = 0x0000000d,
} CLUSTER_OBJECT_TYPE;

typedef struct _NOTIFY_FILTER_AND_TYPE {
    DWORD dwObjectType;
    LONGLONG FilterFlags;
} NOTIFY_FILTER_AND_TYPE, *PNOTIFY_FILTER_AND_TYPE;

typedef struct _CLUSTER_MEMBERSHIP_INFO {
    BOOL  HasQuorum;
    DWORD UpnodesSize;
    BYTE  Upnodes[1];
} CLUSTER_MEMBERSHIP_INFO, *PCLUSTER_MEMBERSHIP_INFO;

#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8 */

#endif /* !_CLUSTER_API_TYPES_ */

#ifndef _CLUSTER_API_TYPES_

typedef enum CLUSTER_ENUM {
    CLUSTER_ENUM_NODE = 0x00000001,
    CLUSTER_ENUM_RESTYPE = 0x00000002,
    CLUSTER_ENUM_RESOURCE = 0x00000004,
    CLUSTER_ENUM_GROUP = 0x00000008,
    CLUSTER_ENUM_NETWORK = 0x00000010,
    CLUSTER_ENUM_NETINTERFACE = 0x00000020,
    CLUSTER_ENUM_INTERNAL_NETWORK = 0x80000000,
    CLUSTER_ENUM_ALL = (CLUSTER_ENUM_NODE | CLUSTER_ENUM_RESTYPE | CLUSTER_ENUM_RESOURCE | CLUSTER_ENUM_GROUP | CLUSTER_ENUM_NETWORK | CLUSTER_ENUM_NETINTERFACE)
} CLUSTER_ENUM;

#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2)
typedef enum CLUSTER_NODE_ENUM {
    CLUSTER_NODE_ENUM_NETINTERFACES = 0x00000001,
    CLUSTER_NODE_ENUM_GROUPS = 0x00000002,
    CLUSTER_NODE_ENUM_PREFERRED_GROUPS = 0x00000004,
    CLUSTER_NODE_ENUM_ALL = (CLUSTER_NODE_ENUM_NETINTERFACES | CLUSTER_NODE_ENUM_GROUPS)
} CLUSTER_NODE_ENUM;
#else /* CLUSAPI_VERSION < CLUSAPI_VERSION_SERVER2008R2 */
typedef enum CLUSTER_NODE_ENUM {
    CLUSTER_NODE_ENUM_NETINTERFACES = 0x00000001,
    CLUSTER_NODE_ENUM_ALL = CLUSTER_NODE_ENUM_NETINTERFACES
} CLUSTER_NODE_ENUM;
#endif /* CLUSAPI_VERSION < CLUSAPI_VERSION_SERVER2008R2 */

typedef enum CLUSTER_NODE_STATE {
    ClusterNodeStateUnknown = -1,
    ClusterNodeUp,
    ClusterNodeDown,
    ClusterNodePaused,
    ClusterNodeJoining
} CLUSTER_NODE_STATE;

#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8)
typedef enum CLUSTER_NODE_DRAIN_STATUS {
    NodeDrainStatusNotInitiated = 0,
    NodeDrainStatusInProgress,
    NodeDrainStatusCompleted,
    NodeDrainStatusFailed,
    ClusterNodeDrainStatusCount
} CLUSTER_NODE_DRAIN_STATUS;
#endif  /* CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8 */

#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_WINTHRESHOLD)
typedef enum CLUSTER_NODE_STATUS {
    NodeStatusNormal = 0x0,
    NodeStatusIsolated = 0x1,
    NodeStatusQuarantined = 0x2,
    NodeStatusDrainInProgress = 0x4,
    NodeStatusDrainCompleted = 0x8,
    NodeStatusDrainFailed = 0x10,
    NodeStatusMax = (NodeStatusIsolated | NodeStatusQuarantined | NodeStatusDrainFailed)
} CLUSTER_NODE_STATUS;
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_WINTHRESHOLD */

typedef enum CLUSTER_GROUP_ENUM {
    CLUSTER_GROUP_ENUM_CONTAINS = 0x00000001,
    CLUSTER_GROUP_ENUM_NODES = 0x00000002,
    CLUSTER_GROUP_ENUM_ALL = (CLUSTER_GROUP_ENUM_CONTAINS|CLUSTER_GROUP_ENUM_NODES)
} CLUSTER_GROUP_ENUM;

typedef enum CLUSTER_GROUP_STATE {
    ClusterGroupStateUnknown = -1,
    ClusterGroupOnline,
    ClusterGroupOffline,
    ClusterGroupFailed,
    ClusterGroupPartialOnline,
    ClusterGroupPending
} CLUSTER_GROUP_STATE;

typedef enum CLUSTER_GROUP_PRIORITY {
    PriorityDisabled = 0,
    PriorityLow = 1000,
    PriorityMedium = 2000,
    PriorityHigh  = 3000,
} CLUSTER_GROUP_PRIORITY;

typedef enum CLUSTER_GROUP_AUTOFAILBACK_TYPE {
    ClusterGroupPreventFailback = 0,
    ClusterGroupAllowFailback,
    ClusterGroupFailbackTypeCount
} CLUSTER_GROUP_AUTOFAILBACK_TYPE, CGAFT;

#if CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8

#define CLUSTER_GROUP_ENUM_ITEM_VERSION_1  0x00000001
#define CLUSTER_GROUP_ENUM_ITEM_VERSION  CLUSTER_GROUP_ENUM_ITEM_VERSION_1

typedef struct _CLUSTER_GROUP_ENUM_ITEM {
    DWORD dwVersion;
    DWORD cbId;
    LPWSTR lpszId;
    DWORD cbName;
    LPWSTR lpszName;
    CLUSTER_GROUP_STATE state;
    DWORD cbOwnerNode;
    LPWSTR lpszOwnerNode;
    DWORD dwFlags;
    DWORD cbProperties;
    PVOID pProperties;
    DWORD cbRoProperties;
    PVOID pRoProperties;
} CLUSTER_GROUP_ENUM_ITEM, *PCLUSTER_GROUP_ENUM_ITEM;

#define CLUSTER_RESOURCE_ENUM_ITEM_VERSION_1  0x00000001
#define CLUSTER_RESOURCE_ENUM_ITEM_VERSION  CLUSTER_RESOURCE_ENUM_ITEM_VERSION_1

typedef struct _CLUSTER_RESOURCE_ENUM_ITEM {
    DWORD dwVersion;
    DWORD cbId;
    LPWSTR lpszId;
    DWORD cbName;
    LPWSTR lpszName;
    DWORD cbOwnerGroupName;
    LPWSTR lpszOwnerGroupName;
    DWORD cbOwnerGroupId;
    LPWSTR lpszOwnerGroupId;
    DWORD cbProperties;
    PVOID pProperties;
    DWORD cbRoProperties;
    PVOID pRoProperties;
} CLUSTER_RESOURCE_ENUM_ITEM, *PCLUSTER_RESOURCE_ENUM_ITEM;

#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8 */

typedef enum CLUSTER_RESOURCE_STATE {
    ClusterResourceStateUnknown = -1,
    ClusterResourceInherited,
    ClusterResourceInitializing,
    ClusterResourceOnline,
    ClusterResourceOffline,
    ClusterResourceFailed,
    ClusterResourcePending = 128,
    ClusterResourceOnlinePending,
    ClusterResourceOfflinePending
} CLUSTER_RESOURCE_STATE;

typedef enum CLUSTER_RESOURCE_RESTART_ACTION {
    ClusterResourceDontRestart = 0,
    ClusterResourceRestartNoNotify,
    ClusterResourceRestartNotify,
    ClusterResourceRestartActionCount
} CLUSTER_RESOURCE_RESTART_ACTION, CRRA;

typedef enum CLUSTER_RESOURCE_EMBEDDED_FAILURE_ACTION {
    ClusterResourceEmbeddedFailureActionNone = 0,
    ClusterResourceEmbeddedFailureActionLogOnly = 1,
    ClusterResourceEmbeddedFailureActionRecover
} CLUSTER_RESOURCE_EMBEDDED_FAILURE_ACTION;

typedef enum CLUSTER_RESOURCE_CREATE_FLAGS {
    CLUSTER_RESOURCE_DEFAULT_MONITOR = 0,
    CLUSTER_RESOURCE_SEPARATE_MONITOR = 1,
    CLUSTER_RESOURCE_VALID_FLAGS = CLUSTER_RESOURCE_SEPARATE_MONITOR
} CLUSTER_RESOURCE_CREATE_FLAGS;

typedef enum CLUSTER_SHARED_VOLUME_SNAPSHOT_STATE {
    ClusterSharedVolumeSnapshotStateUnknown,
    ClusterSharedVolumePrepareForHWSnapshot,
    ClusterSharedVolumeHWSnapshotCompleted,
    ClusterSharedVolumePrepareForFreeze
} CLUSTER_SHARED_VOLUME_SNAPSHOT_STATE;

typedef enum CLUSTER_PROPERTY_TYPE {
    CLUSPROP_TYPE_UNKNOWN = -1,
    CLUSPROP_TYPE_ENDMARK = 0,
    CLUSPROP_TYPE_LIST_VALUE,
    CLUSPROP_TYPE_RESCLASS,
    CLUSPROP_TYPE_RESERVED1,
    CLUSPROP_TYPE_NAME,
    CLUSPROP_TYPE_SIGNATURE,
    CLUSPROP_TYPE_SCSI_ADDRESS,
    CLUSPROP_TYPE_DISK_NUMBER,
    CLUSPROP_TYPE_PARTITION_INFO,
    CLUSPROP_TYPE_FTSET_INFO,
    CLUSPROP_TYPE_DISK_SERIALNUMBER,
    CLUSPROP_TYPE_DISK_GUID,
    CLUSPROP_TYPE_DISK_SIZE,
    CLUSPROP_TYPE_PARTITION_INFO_EX,
    CLUSPROP_TYPE_PARTITION_INFO_EX2,
    CLUSPROP_TYPE_STORAGE_DEVICE_ID_DESCRIPTOR,
    CLUSPROP_TYPE_USER = 32768
} CLUSTER_PROPERTY_TYPE;

typedef enum CLUSTER_PROPERTY_FORMAT {
    CLUSPROP_FORMAT_UNKNOWN = 0,
    CLUSPROP_FORMAT_BINARY,
    CLUSPROP_FORMAT_DWORD,
    CLUSPROP_FORMAT_SZ,
    CLUSPROP_FORMAT_EXPAND_SZ,
    CLUSPROP_FORMAT_MULTI_SZ,
    CLUSPROP_FORMAT_ULARGE_INTEGER,
    CLUSPROP_FORMAT_LONG,
    CLUSPROP_FORMAT_EXPANDED_SZ,
    CLUSPROP_FORMAT_SECURITY_DESCRIPTOR,
    CLUSPROP_FORMAT_LARGE_INTEGER,
    CLUSPROP_FORMAT_WORD,
    CLUSPROP_FORMAT_FILETIME,
    CLUSPROP_FORMAT_VALUE_LIST,
    CLUSPROP_FORMAT_PROPERTY_LIST,
    CLUSPROP_FORMAT_USER = 32768
} CLUSTER_PROPERTY_FORMAT;

typedef enum CLUSTER_PROPERTY_SYNTAX {
    CLUSPROP_SYNTAX_ENDMARK = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_ENDMARK, CLUSPROP_FORMAT_UNKNOWN),
    CLUSPROP_SYNTAX_NAME = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_NAME, CLUSPROP_FORMAT_SZ),
    CLUSPROP_SYNTAX_RESCLASS = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_RESCLASS, CLUSPROP_FORMAT_DWORD),

    CLUSPROP_SYNTAX_LIST_VALUE_SZ = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_LIST_VALUE, CLUSPROP_FORMAT_SZ),
    CLUSPROP_SYNTAX_LIST_VALUE_EXPAND_SZ = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_LIST_VALUE, CLUSPROP_FORMAT_EXPAND_SZ),
    CLUSPROP_SYNTAX_LIST_VALUE_DWORD = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_LIST_VALUE, CLUSPROP_FORMAT_DWORD),
    CLUSPROP_SYNTAX_LIST_VALUE_BINARY = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_LIST_VALUE, CLUSPROP_FORMAT_BINARY),
    CLUSPROP_SYNTAX_LIST_VALUE_MULTI_SZ = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_LIST_VALUE, CLUSPROP_FORMAT_MULTI_SZ),
    CLUSPROP_SYNTAX_LIST_VALUE_LONG = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_LIST_VALUE, CLUSPROP_FORMAT_LONG),
    CLUSPROP_SYNTAX_LIST_VALUE_EXPANDED_SZ = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_LIST_VALUE, CLUSPROP_FORMAT_EXPANDED_SZ),
    CLUSPROP_SYNTAX_LIST_VALUE_SECURITY_DESCRIPTOR = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_LIST_VALUE, CLUSPROP_FORMAT_SECURITY_DESCRIPTOR),
    CLUSPROP_SYNTAX_LIST_VALUE_LARGE_INTEGER = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_LIST_VALUE, CLUSPROP_FORMAT_LARGE_INTEGER),
    CLUSPROP_SYNTAX_LIST_VALUE_ULARGE_INTEGER = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_LIST_VALUE, CLUSPROP_FORMAT_ULARGE_INTEGER),
    CLUSPROP_SYNTAX_LIST_VALUE_WORD = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_LIST_VALUE, CLUSPROP_FORMAT_WORD),
    CLUSPROP_SYNTAX_LIST_VALUE_PROPERTY_LIST = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_LIST_VALUE, CLUSPROP_FORMAT_PROPERTY_LIST),
    CLUSPROP_SYNTAX_LIST_VALUE_FILETIME = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_LIST_VALUE, CLUSPROP_FORMAT_FILETIME),


    CLUSPROP_SYNTAX_DISK_SIGNATURE = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_SIGNATURE, CLUSPROP_FORMAT_DWORD),
    CLUSPROP_SYNTAX_SCSI_ADDRESS = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_SCSI_ADDRESS, CLUSPROP_FORMAT_DWORD),
    CLUSPROP_SYNTAX_DISK_NUMBER = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_DISK_NUMBER, CLUSPROP_FORMAT_DWORD),
    CLUSPROP_SYNTAX_PARTITION_INFO = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_PARTITION_INFO, CLUSPROP_FORMAT_BINARY),
    CLUSPROP_SYNTAX_FTSET_INFO = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_FTSET_INFO, CLUSPROP_FORMAT_BINARY),
    CLUSPROP_SYNTAX_DISK_SERIALNUMBER = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_DISK_SERIALNUMBER, CLUSPROP_FORMAT_SZ),
    CLUSPROP_SYNTAX_DISK_GUID = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_DISK_GUID, CLUSPROP_FORMAT_SZ),
    CLUSPROP_SYNTAX_DISK_SIZE = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_DISK_SIZE, CLUSPROP_FORMAT_ULARGE_INTEGER),
    CLUSPROP_SYNTAX_PARTITION_INFO_EX = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_PARTITION_INFO_EX, CLUSPROP_FORMAT_BINARY),
    CLUSPROP_SYNTAX_PARTITION_INFO_EX2 = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_PARTITION_INFO_EX2, CLUSPROP_FORMAT_BINARY),
    CLUSPROP_SYNTAX_STORAGE_DEVICE_ID_DESCRIPTOR = CLUSPROP_SYNTAX_VALUE(CLUSPROP_TYPE_STORAGE_DEVICE_ID_DESCRIPTOR, CLUSPROP_FORMAT_BINARY),
} CLUSTER_PROPERTY_SYNTAX;

#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8)

#define GROUP_FAILURE_INFO_VERSION_1  0x1

typedef struct GROUP_FAILURE_INFO {
    DWORD dwFailoverAttemptsRemaining;
    DWORD dwFailoverPeriodRemaining;
} GROUP_FAILURE_INFO, *PGROUP_FAILURE_INFO;

typedef struct GROUP_FAILURE_INFO_BUFFER {
    DWORD dwVersion;
    GROUP_FAILURE_INFO Info;
} GROUP_FAILURE_INFO_BUFFER, *PGROUP_FAILURE_INFO_BUFFER;

#define RESOURCE_FAILURE_INFO_VERSION_1  0x1

typedef struct RESOURCE_FAILURE_INFO {
    DWORD dwRestartAttemptsRemaining;
    DWORD dwRestartPeriodRemaining; // in sec
} RESOURCE_FAILURE_INFO, *PRESOURCE_FAILURE_INFO;

typedef struct RESOURCE_FAILURE_INFO_BUFFER {
    DWORD dwVersion;
    RESOURCE_FAILURE_INFO Info;
} RESOURCE_FAILURE_INFO_BUFFER, *PRESOURCE_FAILURE_INFO_BUFFER;

typedef struct RESOURCE_TERMINAL_FAILURE_INFO_BUFFER {
    BOOL isTerminalFailure;
    DWORD restartPeriodRemaining;
} RESOURCE_TERMINAL_FAILURE_INFO_BUFFER, *PRESOURCE_TERMINAL_FAILURE_INFO_BUFFER;

#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8 */

typedef enum CLUSTER_CONTROL_OBJECT {
    CLUS_OBJECT_INVALID = 0,
    CLUS_OBJECT_RESOURCE,
    CLUS_OBJECT_RESOURCE_TYPE,
    CLUS_OBJECT_GROUP,
    CLUS_OBJECT_NODE,
    CLUS_OBJECT_NETWORK,
    CLUS_OBJECT_NETINTERFACE,
    CLUS_OBJECT_CLUSTER,
    CLUS_OBJECT_GROUPSET,
    CLUS_OBJECT_USER = 128
} CLUSTER_CONTROL_OBJECT;

typedef enum CLCTL_CODES {
    CLCTL_UNKNOWN = CLCTL_EXTERNAL_CODE(0, CLUS_ACCESS_ANY, CLUS_NO_MODIFY),
    CLCTL_GET_CHARACTERISTICS = CLCTL_EXTERNAL_CODE(1, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GET_FLAGS = CLCTL_EXTERNAL_CODE(2, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GET_CLASS_INFO = CLCTL_EXTERNAL_CODE(3, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GET_REQUIRED_DEPENDENCIES = CLCTL_EXTERNAL_CODE(4, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GET_ARB_TIMEOUT = CLCTL_EXTERNAL_CODE(5, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GET_FAILURE_INFO = CLCTL_EXTERNAL_CODE(6, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    CLCTL_GET_NAME = CLCTL_EXTERNAL_CODE(10, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GET_RESOURCE_TYPE = CLCTL_EXTERNAL_CODE(11, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GET_NODE = CLCTL_EXTERNAL_CODE(12, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GET_NETWORK = CLCTL_EXTERNAL_CODE(13, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GET_ID = CLCTL_EXTERNAL_CODE(14, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GET_FQDN = CLCTL_EXTERNAL_CODE(15, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GET_CLUSTER_SERVICE_ACCOUNT_NAME = CLCTL_EXTERNAL_CODE(16, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_CHECK_VOTER_EVICT = CLCTL_EXTERNAL_CODE(17, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_CHECK_VOTER_DOWN = CLCTL_EXTERNAL_CODE(18, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_SHUTDOWN = CLCTL_EXTERNAL_CODE(19, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    CLCTL_ENUM_COMMON_PROPERTIES = CLCTL_EXTERNAL_CODE(20, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GET_RO_COMMON_PROPERTIES = CLCTL_EXTERNAL_CODE(21, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GET_COMMON_PROPERTIES = CLCTL_EXTERNAL_CODE(22, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_SET_COMMON_PROPERTIES = CLCTL_EXTERNAL_CODE(23, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_VALIDATE_COMMON_PROPERTIES = CLCTL_EXTERNAL_CODE(24, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GET_COMMON_PROPERTY_FMTS = CLCTL_EXTERNAL_CODE(25, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GET_COMMON_RESOURCE_PROPERTY_FMTS = CLCTL_EXTERNAL_CODE(26, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    CLCTL_ENUM_PRIVATE_PROPERTIES = CLCTL_EXTERNAL_CODE(30, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GET_RO_PRIVATE_PROPERTIES = CLCTL_EXTERNAL_CODE(31, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GET_PRIVATE_PROPERTIES = CLCTL_EXTERNAL_CODE(32, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_SET_PRIVATE_PROPERTIES = CLCTL_EXTERNAL_CODE(33, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_VALIDATE_PRIVATE_PROPERTIES = CLCTL_EXTERNAL_CODE(34, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GET_PRIVATE_PROPERTY_FMTS = CLCTL_EXTERNAL_CODE(35, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GET_PRIVATE_RESOURCE_PROPERTY_FMTS = CLCTL_EXTERNAL_CODE(36, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    CLCTL_ADD_REGISTRY_CHECKPOINT = CLCTL_EXTERNAL_CODE(40, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_DELETE_REGISTRY_CHECKPOINT = CLCTL_EXTERNAL_CODE(41, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_GET_REGISTRY_CHECKPOINTS = CLCTL_EXTERNAL_CODE(42, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    CLCTL_ADD_CRYPTO_CHECKPOINT = CLCTL_EXTERNAL_CODE(43, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_DELETE_CRYPTO_CHECKPOINT = CLCTL_EXTERNAL_CODE(44, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_GET_CRYPTO_CHECKPOINTS = CLCTL_EXTERNAL_CODE(45, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    CLCTL_RESOURCE_UPGRADE_DLL = CLCTL_EXTERNAL_CODE(46, CLUS_ACCESS_WRITE, CLUS_MODIFY),

    CLCTL_ADD_REGISTRY_CHECKPOINT_64BIT = CLCTL_EXTERNAL_CODE(47, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_ADD_REGISTRY_CHECKPOINT_32BIT = CLCTL_EXTERNAL_CODE(48, CLUS_ACCESS_WRITE, CLUS_MODIFY),

    CLCTL_GET_LOADBAL_PROCESS_LIST = CLCTL_EXTERNAL_CODE(50, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    CLCTL_SET_ACCOUNT_ACCESS = CLCTL_EXTERNAL_CODE(60, CLUS_ACCESS_WRITE, CLUS_MODIFY),

    CLCTL_GET_NETWORK_NAME = CLCTL_EXTERNAL_CODE(90, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_NETNAME_GET_VIRTUAL_SERVER_TOKEN = CLCTL_EXTERNAL_CODE(91, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_NETNAME_REGISTER_DNS_RECORDS = CLCTL_EXTERNAL_CODE(92, CLUS_ACCESS_WRITE, CLUS_NO_MODIFY),
    CLCTL_GET_DNS_NAME = CLCTL_EXTERNAL_CODE(93, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_NETNAME_SET_PWD_INFO = CLCTL_EXTERNAL_CODE(94, CLUS_ACCESS_WRITE, CLUS_NO_MODIFY),
    CLCTL_NETNAME_DELETE_CO = CLCTL_EXTERNAL_CODE(95, CLUS_ACCESS_WRITE, CLUS_NO_MODIFY),
    CLCTL_NETNAME_VALIDATE_VCO = CLCTL_EXTERNAL_CODE(96, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_NETNAME_RESET_VCO = CLCTL_EXTERNAL_CODE(97, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_NETNAME_REPAIR_VCO = CLCTL_EXTERNAL_CODE(99, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    CLCTL_STORAGE_GET_DISK_INFO = CLCTL_EXTERNAL_CODE(100, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_STORAGE_GET_AVAILABLE_DISKS = CLCTL_EXTERNAL_CODE(101, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_STORAGE_IS_PATH_VALID = CLCTL_EXTERNAL_CODE(102, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_STORAGE_SYNC_CLUSDISK_DB = CLCTL_EXTERNAL_CODE(103, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_STORAGE_GET_DISK_NUMBER_INFO = CLCTL_EXTERNAL_CODE(104, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    CLCTL_QUERY_DELETE = CLCTL_EXTERNAL_CODE(110, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    CLCTL_IPADDRESS_RENEW_LEASE = CLCTL_EXTERNAL_CODE(111, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_IPADDRESS_RELEASE_LEASE = CLCTL_EXTERNAL_CODE(112, CLUS_ACCESS_WRITE, CLUS_MODIFY),

    CLCTL_QUERY_MAINTENANCE_MODE = CLCTL_EXTERNAL_CODE(120, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_SET_MAINTENANCE_MODE = CLCTL_EXTERNAL_CODE(121, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_STORAGE_SET_DRIVELETTER = CLCTL_EXTERNAL_CODE(122, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_STORAGE_GET_DRIVELETTERS = CLCTL_EXTERNAL_CODE(123, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_STORAGE_GET_DISK_INFO_EX = CLCTL_EXTERNAL_CODE(124, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_STORAGE_GET_AVAILABLE_DISKS_EX = CLCTL_EXTERNAL_CODE(125, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_STORAGE_GET_DISK_INFO_EX2 = CLCTL_EXTERNAL_CODE(126, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_STORAGE_GET_CLUSPORT_DISK_COUNT = CLCTL_EXTERNAL_CODE(127, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    CLCTL_STORAGE_REMAP_DRIVELETTER = CLCTL_EXTERNAL_CODE(128, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_STORAGE_GET_DISKID = CLCTL_EXTERNAL_CODE(129, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_STORAGE_IS_CLUSTERABLE = CLCTL_EXTERNAL_CODE(130, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_STORAGE_REMOVE_VM_OWNERSHIP = CLCTL_EXTERNAL_CODE(131, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_STORAGE_GET_MOUNTPOINTS = CLCTL_EXTERNAL_CODE(132, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_STORAGE_GET_DIRTY = CLCTL_EXTERNAL_CODE(134, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_STORAGE_GET_SHARED_VOLUME_INFO = CLCTL_EXTERNAL_CODE(137, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_STORAGE_IS_CSV_FILE = CLCTL_EXTERNAL_CODE(138, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_STORAGE_GET_RESOURCEID = CLCTL_EXTERNAL_CODE(139, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    CLCTL_VALIDATE_PATH = CLCTL_EXTERNAL_CODE(140, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_VALIDATE_NETNAME = CLCTL_EXTERNAL_CODE(141, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_VALIDATE_DIRECTORY = CLCTL_EXTERNAL_CODE(142, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_BATCH_BLOCK_KEY = CLCTL_EXTERNAL_CODE(143, CLUS_ACCESS_WRITE, CLUS_NO_MODIFY),
    CLCTL_BATCH_UNBLOCK_KEY = CLCTL_EXTERNAL_CODE(144, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    CLCTL_FILESERVER_SHARE_ADD = CLCTL_EXTERNAL_CODE(145, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_FILESERVER_SHARE_DEL = CLCTL_EXTERNAL_CODE(146, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_FILESERVER_SHARE_MODIFY = CLCTL_EXTERNAL_CODE(147, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_FILESERVER_SHARE_REPORT = CLCTL_EXTERNAL_CODE(148, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    CLCTL_NETNAME_GET_OU_FOR_VCO = CLCTL_EXTERNAL_CODE(155, CLUS_ACCESS_WRITE, CLUS_MODIFY),

    CLCTL_ENABLE_SHARED_VOLUME_DIRECTIO = CLCTL_EXTERNAL_CODE(162, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_DISABLE_SHARED_VOLUME_DIRECTIO = CLCTL_EXTERNAL_CODE(163, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_GET_SHARED_VOLUME_ID = CLCTL_EXTERNAL_CODE(164, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_SET_CSV_MAINTENANCE_MODE = CLCTL_EXTERNAL_CODE(165, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_SET_SHARED_VOLUME_BACKUP_MODE = CLCTL_EXTERNAL_CODE(166, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_STORAGE_GET_SHARED_VOLUME_PARTITION_NAMES = CLCTL_EXTERNAL_CODE(167, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    CLCTL_STORAGE_GET_SHARED_VOLUME_STATES = CLCTL_EXTERNAL_CODE(168, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_STORAGE_IS_SHARED_VOLUME = CLCTL_EXTERNAL_CODE(169, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    CLCTL_GET_CLUSDB_TIMESTAMP = CLCTL_EXTERNAL_CODE(170, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    CLCTL_RW_MODIFY_NOOP = CLCTL_EXTERNAL_CODE(171, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_IS_QUORUM_BLOCKED = CLCTL_EXTERNAL_CODE(172, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_POOL_GET_DRIVE_INFO = CLCTL_EXTERNAL_CODE(173, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    CLCTL_GET_GUM_LOCK_OWNER = CLCTL_EXTERNAL_CODE(174, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GET_STUCK_NODES = CLCTL_EXTERNAL_CODE(175, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_INJECT_GEM_FAULT = CLCTL_EXTERNAL_CODE(176, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_INTRODUCE_GEM_REPAIR_DELAY = CLCTL_EXTERNAL_CODE(177, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_SEND_DUMMY_GEM_MESSAGES = CLCTL_EXTERNAL_CODE(178, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_BLOCK_GEM_SEND_RECV = CLCTL_EXTERNAL_CODE(179, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GET_GEMID_VECTOR = CLCTL_EXTERNAL_CODE(180, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    CLCTL_ADD_CRYPTO_CHECKPOINT_EX = CLCTL_EXTERNAL_CODE(181, CLUS_ACCESS_WRITE, CLUS_MODIFY),

    CLCTL_GROUP_GET_LAST_MOVE_TIME = CLCTL_EXTERNAL_CODE(182, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    CLCTL_SET_STORAGE_CONFIGURATION = CLCTL_EXTERNAL_CODE(184, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_GET_STORAGE_CONFIGURATION = CLCTL_EXTERNAL_CODE(185, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GET_STORAGE_CONFIG_ATTRIBUTES = CLCTL_EXTERNAL_CODE(186, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_REMOVE_NODE = CLCTL_EXTERNAL_CODE(187, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_IS_FEATURE_INSTALLED = CLCTL_EXTERNAL_CODE(188, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_IS_S2D_FEATURE_SUPPORTED = CLCTL_EXTERNAL_CODE(189, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_STORAGE_GET_PHYSICAL_DISK_INFO = CLCTL_EXTERNAL_CODE(190, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_STORAGE_GET_CLUSBFLT_PATHS = CLCTL_EXTERNAL_CODE(191, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_STORAGE_GET_CLUSBFLT_PATHINFO = CLCTL_EXTERNAL_CODE(192, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_CLEAR_NODE_CONNECTION_INFO = CLCTL_EXTERNAL_CODE(193, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_SET_DNS_DOMAIN = CLCTL_EXTERNAL_CODE(194, CLUS_ACCESS_WRITE, CLUS_MODIFY),

    CTCTL_GET_ROUTESTATUS_BASIC = CLCTL_EXTERNAL_CODE(195, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CTCTL_GET_ROUTESTATUS_EXTENDED = CLCTL_EXTERNAL_CODE(196, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    /* Control codes 2000 to 2999 are reserved */

    CLCTL_STORAGE_GET_AVAILABLE_DISKS_EX2_INT = CLCTL_EXTERNAL_CODE(2040, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    CLCTL_CLOUD_WITNESS_RESOURCE_TYPE_VALIDATE_CREDENTIALS = CLCTL_EXTERNAL_CODE(2104, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_CLOUD_WITNESS_RESOURCE_UPDATE_TOKEN = CLCTL_EXTERNAL_CODE(2105, CLUS_ACCESS_WRITE, CLUS_MODIFY),

    CLCTL_RESOURCE_PREPARE_UPGRADE = CLCTL_EXTERNAL_CODE(2106, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_RESOURCE_UPGRADE_COMPLETED = CLCTL_EXTERNAL_CODE(2107, CLUS_ACCESS_WRITE, CLUS_MODIFY),

    CLCTL_CLOUD_WITNESS_RESOURCE_TYPE_VALIDATE_CREDENTIALS_WITH_KEY = CLCTL_EXTERNAL_CODE(2108, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_CLOUD_WITNESS_RESOURCE_UPDATE_KEY = CLCTL_EXTERNAL_CODE(2109, CLUS_ACCESS_WRITE, CLUS_MODIFY),

    CLCTL_REPLICATION_GET_LOG_INFO = CLCTL_EXTERNAL_CODE(2129, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_REPLICATION_GET_ELIGIBLE_LOGDISKS = CLCTL_EXTERNAL_CODE(2130, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_REPLICATION_GET_ELIGIBLE_TARGET_DATADISKS = CLCTL_EXTERNAL_CODE(2131, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_REPLICATION_GET_ELIGIBLE_SOURCE_DATADISKS = CLCTL_EXTERNAL_CODE(2132, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_REPLICATION_GET_REPLICATED_DISKS = CLCTL_EXTERNAL_CODE(2133, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_REPLICATION_GET_REPLICA_VOLUMES = CLCTL_EXTERNAL_CODE(2134, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_REPLICATION_GET_LOG_VOLUME = CLCTL_EXTERNAL_CODE(2135, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_REPLICATION_GET_RESOURCE_GROUP = CLCTL_EXTERNAL_CODE(2136, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_REPLICATION_GET_REPLICATED_PARTITION_INFO = CLCTL_EXTERNAL_CODE(2137, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    CLCTL_GET_STATE_CHANGE_TIME = CLCTL_EXTERNAL_CODE(2903, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_WINTHRESHOLD)
    CLCTL_SET_CLUSTER_S2D_ENABLED = CLCTL_EXTERNAL_CODE(2904, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_SET_CLUSTER_S2D_CACHE_METADATA_RESERVE_BYTES = CLCTL_EXTERNAL_CODE(2907, CLUS_ACCESS_WRITE, CLUS_MODIFY),
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_WINTHRESHOLD */
    CLCTL_GROUPSET_GET_GROUPS = CLCTL_EXTERNAL_CODE(2908, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GROUPSET_GET_PROVIDER_GROUPS = CLCTL_EXTERNAL_CODE(2909, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GROUPSET_GET_PROVIDER_GROUPSETS = CLCTL_EXTERNAL_CODE(2910, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GROUP_GET_PROVIDER_GROUPS = CLCTL_EXTERNAL_CODE(2911, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_GROUP_GET_PROVIDER_GROUPSETS = CLCTL_EXTERNAL_CODE(2912, CLUS_ACCESS_READ, CLUS_NO_MODIFY),

    /* Internal shit */
    CLCTL_DELETE = CLCTL_INTERNAL_CODE(1, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_INSTALL_NODE = CLCTL_INTERNAL_CODE(2, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_EVICT_NODE = CLCTL_INTERNAL_CODE(3, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_ADD_DEPENDENCY = CLCTL_INTERNAL_CODE(4, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_REMOVE_DEPENDENCY = CLCTL_INTERNAL_CODE(5, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_ADD_OWNER = CLCTL_INTERNAL_CODE(6, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_REMOVE_OWNER = CLCTL_INTERNAL_CODE(7, CLUS_ACCESS_WRITE, CLUS_MODIFY),

    CLCTL_SET_NAME = CLCTL_INTERNAL_CODE(9, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_CLUSTER_NAME_CHANGED = CLCTL_INTERNAL_CODE(10, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_CLUSTER_VERSION_CHANGED = CLCTL_INTERNAL_CODE(11, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_FIXUP_ON_UPGRADE = CLCTL_INTERNAL_CODE(12, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_STARTING_PHASE1 = CLCTL_INTERNAL_CODE(13, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_STARTING_PHASE2 = CLCTL_INTERNAL_CODE(14, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_HOLD_IO = CLCTL_INTERNAL_CODE(15, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_RESUME_IO = CLCTL_INTERNAL_CODE(16, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_FORCE_QUORUM = CLCTL_INTERNAL_CODE(17, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_INITIALIZE = CLCTL_INTERNAL_CODE(18, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_STATE_CHANGE_REASON = CLCTL_INTERNAL_CODE(19, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_PROVIDER_STATE_CHANGE = CLCTL_INTERNAL_CODE(20, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_LEAVING_GROUP = CLCTL_INTERNAL_CODE(21, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_JOINING_GROUP = CLCTL_INTERNAL_CODE(22, CLUS_ACCESS_WRITE, CLUS_MODIFY),

    CLCTL_FSWITNESS_GET_EPOCH_INFO = CLCTL_INTERNAL_CODE(23, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_FSWITNESS_SET_EPOCH_INFO = CLCTL_INTERNAL_CODE(24, CLUS_ACCESS_WRITE, CLUS_MODIFY),
    CLCTL_FSWITNESS_RELEASE_LOCK = CLCTL_INTERNAL_CODE(25, CLUS_ACCESS_WRITE, CLUS_MODIFY),

    CLCTL_NETNAME_CREDS_NOTIFYCAM = CLCTL_INTERNAL_CODE(26, CLUS_ACCESS_WRITE, CLUS_MODIFY),

    CLCTL_NOTIFY_QUORUM_STATUS = CLCTL_INTERNAL_CODE(31, CLUS_ACCESS_WRITE, CLUS_MODIFY),

    CLCTL_GET_OPERATION_CONTEXT = CLCTL_INTERNAL_CODE(2106, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
    CLCTL_NOTIFY_OWNER_CHANGE = CLCTL_INTERNAL_CODE(2120, CLUS_ACCESS_WRITE, CLUS_MODIFY),
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_WINTHRESHOLD)
    CLCTL_VALIDATE_CHANGE_GROUP = CLCTL_INTERNAL_CODE(2121, CLUS_ACCESS_READ, CLUS_NO_MODIFY),
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_WINTHRESHOLD */
} CLCTL_CODES;

typedef enum CLUSCTL_RESOURCE_CODES {
    CLUSCTL_RESOURCE_UNKNOWN = CLUSCTL_RESOURCE_CODE(CLCTL_UNKNOWN),
    CLUSCTL_RESOURCE_GET_CHARACTERISTICS = CLUSCTL_RESOURCE_CODE(CLCTL_GET_CHARACTERISTICS),
    CLUSCTL_RESOURCE_GET_FLAGS = CLUSCTL_RESOURCE_CODE(CLCTL_GET_FLAGS),
    CLUSCTL_RESOURCE_GET_CLASS_INFO = CLUSCTL_RESOURCE_CODE(CLCTL_GET_CLASS_INFO),
    CLUSCTL_RESOURCE_GET_REQUIRED_DEPENDENCIES = CLUSCTL_RESOURCE_CODE(CLCTL_GET_REQUIRED_DEPENDENCIES),
    CLUSCTL_RESOURCE_GET_NAME = CLUSCTL_RESOURCE_CODE(CLCTL_GET_NAME),
    CLUSCTL_RESOURCE_GET_ID = CLUSCTL_RESOURCE_CODE(CLCTL_GET_ID),
    CLUSCTL_RESOURCE_GET_RESOURCE_TYPE = CLUSCTL_RESOURCE_CODE(CLCTL_GET_RESOURCE_TYPE),
    CLUSCTL_RESOURCE_ENUM_COMMON_PROPERTIES = CLUSCTL_RESOURCE_CODE(CLCTL_ENUM_COMMON_PROPERTIES),
    CLUSCTL_RESOURCE_GET_RO_COMMON_PROPERTIES = CLUSCTL_RESOURCE_CODE(CLCTL_GET_RO_COMMON_PROPERTIES),
    CLUSCTL_RESOURCE_GET_COMMON_PROPERTIES = CLUSCTL_RESOURCE_CODE(CLCTL_GET_COMMON_PROPERTIES),
    CLUSCTL_RESOURCE_SET_COMMON_PROPERTIES = CLUSCTL_RESOURCE_CODE(CLCTL_SET_COMMON_PROPERTIES),
    CLUSCTL_RESOURCE_VALIDATE_COMMON_PROPERTIES = CLUSCTL_RESOURCE_CODE(CLCTL_VALIDATE_COMMON_PROPERTIES),
    CLUSCTL_RESOURCE_GET_COMMON_PROPERTY_FMTS = CLUSCTL_RESOURCE_CODE(CLCTL_GET_COMMON_PROPERTY_FMTS),
    CLUSCTL_RESOURCE_ENUM_PRIVATE_PROPERTIES = CLUSCTL_RESOURCE_CODE(CLCTL_ENUM_PRIVATE_PROPERTIES),
    CLUSCTL_RESOURCE_GET_RO_PRIVATE_PROPERTIES = CLUSCTL_RESOURCE_CODE(CLCTL_GET_RO_PRIVATE_PROPERTIES),
    CLUSCTL_RESOURCE_GET_PRIVATE_PROPERTIES = CLUSCTL_RESOURCE_CODE(CLCTL_GET_PRIVATE_PROPERTIES),
    CLUSCTL_RESOURCE_SET_PRIVATE_PROPERTIES = CLUSCTL_RESOURCE_CODE(CLCTL_SET_PRIVATE_PROPERTIES),
    CLUSCTL_RESOURCE_VALIDATE_PRIVATE_PROPERTIES = CLUSCTL_RESOURCE_CODE(CLCTL_VALIDATE_PRIVATE_PROPERTIES),
    CLUSCTL_RESOURCE_GET_PRIVATE_PROPERTY_FMTS = CLUSCTL_RESOURCE_CODE(CLCTL_GET_PRIVATE_PROPERTY_FMTS),
    CLUSCTL_RESOURCE_ADD_REGISTRY_CHECKPOINT = CLUSCTL_RESOURCE_CODE(CLCTL_ADD_REGISTRY_CHECKPOINT),
    CLUSCTL_RESOURCE_DELETE_REGISTRY_CHECKPOINT = CLUSCTL_RESOURCE_CODE(CLCTL_DELETE_REGISTRY_CHECKPOINT),
    CLUSCTL_RESOURCE_GET_REGISTRY_CHECKPOINTS = CLUSCTL_RESOURCE_CODE(CLCTL_GET_REGISTRY_CHECKPOINTS),
    CLUSCTL_RESOURCE_ADD_CRYPTO_CHECKPOINT = CLUSCTL_RESOURCE_CODE(CLCTL_ADD_CRYPTO_CHECKPOINT),
    CLUSCTL_RESOURCE_DELETE_CRYPTO_CHECKPOINT = CLUSCTL_RESOURCE_CODE(CLCTL_DELETE_CRYPTO_CHECKPOINT),
    CLUSCTL_RESOURCE_ADD_CRYPTO_CHECKPOINT_EX = CLUSCTL_RESOURCE_CODE(CLCTL_ADD_CRYPTO_CHECKPOINT_EX),
    CLUSCTL_RESOURCE_GET_CRYPTO_CHECKPOINTS = CLUSCTL_RESOURCE_CODE(CLCTL_GET_CRYPTO_CHECKPOINTS),
    CLUSCTL_RESOURCE_GET_LOADBAL_PROCESS_LIST = CLUSCTL_RESOURCE_CODE(CLCTL_GET_LOADBAL_PROCESS_LIST),
    CLUSCTL_RESOURCE_GET_NETWORK_NAME = CLUSCTL_RESOURCE_CODE(CLCTL_GET_NETWORK_NAME),
    CLUSCTL_RESOURCE_NETNAME_GET_VIRTUAL_SERVER_TOKEN = CLUSCTL_RESOURCE_CODE(CLCTL_NETNAME_GET_VIRTUAL_SERVER_TOKEN),
    CLUSCTL_RESOURCE_NETNAME_SET_PWD_INFO = CLUSCTL_RESOURCE_CODE(CLCTL_NETNAME_SET_PWD_INFO),
    CLUSCTL_RESOURCE_NETNAME_DELETE_CO = CLUSCTL_RESOURCE_CODE(CLCTL_NETNAME_DELETE_CO),
    CLUSCTL_RESOURCE_NETNAME_VALIDATE_VCO = CLUSCTL_RESOURCE_CODE(CLCTL_NETNAME_VALIDATE_VCO),
    CLUSCTL_RESOURCE_NETNAME_RESET_VCO = CLUSCTL_RESOURCE_CODE(CLCTL_NETNAME_RESET_VCO),
    CLUSCTL_RESOURCE_NETNAME_REPAIR_VCO = CLUSCTL_RESOURCE_CODE(CLCTL_NETNAME_REPAIR_VCO),
    CLUSCTL_RESOURCE_NETNAME_REGISTER_DNS_RECORDS = CLUSCTL_RESOURCE_CODE(CLCTL_NETNAME_REGISTER_DNS_RECORDS),
    CLUSCTL_RESOURCE_GET_DNS_NAME = CLUSCTL_RESOURCE_CODE(CLCTL_GET_DNS_NAME),
    CLUSCTL_RESOURCE_STORAGE_GET_DISK_INFO = CLUSCTL_RESOURCE_CODE(CLCTL_STORAGE_GET_DISK_INFO),
    CLUSCTL_RESOURCE_STORAGE_GET_DISK_NUMBER_INFO = CLUSCTL_RESOURCE_CODE(CLCTL_STORAGE_GET_DISK_NUMBER_INFO),
    CLUSCTL_RESOURCE_STORAGE_IS_PATH_VALID = CLUSCTL_RESOURCE_CODE(CLCTL_STORAGE_IS_PATH_VALID),
    CLUSCTL_RESOURCE_QUERY_DELETE = CLUSCTL_RESOURCE_CODE(CLCTL_QUERY_DELETE),
    CLUSCTL_RESOURCE_UPGRADE_DLL = CLUSCTL_RESOURCE_CODE(CLCTL_RESOURCE_UPGRADE_DLL),
    CLUSCTL_RESOURCE_IPADDRESS_RENEW_LEASE = CLUSCTL_RESOURCE_CODE(CLCTL_IPADDRESS_RENEW_LEASE),
    CLUSCTL_RESOURCE_IPADDRESS_RELEASE_LEASE = CLUSCTL_RESOURCE_CODE(CLCTL_IPADDRESS_RELEASE_LEASE),
    CLUSCTL_RESOURCE_ADD_REGISTRY_CHECKPOINT_64BIT = CLUSCTL_RESOURCE_CODE(CLCTL_ADD_REGISTRY_CHECKPOINT_64BIT),
    CLUSCTL_RESOURCE_ADD_REGISTRY_CHECKPOINT_32BIT = CLUSCTL_RESOURCE_CODE(CLCTL_ADD_REGISTRY_CHECKPOINT_32BIT),
    CLUSCTL_RESOURCE_QUERY_MAINTENANCE_MODE = CLUSCTL_RESOURCE_CODE(CLCTL_QUERY_MAINTENANCE_MODE),
    CLUSCTL_RESOURCE_SET_MAINTENANCE_MODE = CLUSCTL_RESOURCE_CODE(CLCTL_SET_MAINTENANCE_MODE),
    CLUSCTL_RESOURCE_STORAGE_SET_DRIVELETTER = CLUSCTL_RESOURCE_CODE(CLCTL_STORAGE_SET_DRIVELETTER),
    CLUSCTL_RESOURCE_STORAGE_GET_DISK_INFO_EX = CLUSCTL_RESOURCE_CODE(CLCTL_STORAGE_GET_DISK_INFO_EX),
    CLUSCTL_RESOURCE_STORAGE_GET_DISK_INFO_EX2 = CLUSCTL_RESOURCE_CODE(CLCTL_STORAGE_GET_DISK_INFO_EX2),
    CLUSCTL_RESOURCE_STORAGE_GET_MOUNTPOINTS = CLUSCTL_RESOURCE_CODE(CLCTL_STORAGE_GET_MOUNTPOINTS),
    CLUSCTL_RESOURCE_STORAGE_GET_DIRTY = CLUSCTL_RESOURCE_CODE(CLCTL_STORAGE_GET_DIRTY),
    CLUSCTL_RESOURCE_STORAGE_GET_SHARED_VOLUME_INFO = CLUSCTL_RESOURCE_CODE(CLCTL_STORAGE_GET_SHARED_VOLUME_INFO),
    CLUSCTL_RESOURCE_SET_CSV_MAINTENANCE_MODE = CLUSCTL_RESOURCE_CODE(CLCTL_SET_CSV_MAINTENANCE_MODE),
    CLUSCTL_RESOURCE_ENABLE_SHARED_VOLUME_DIRECTIO = CLUSCTL_RESOURCE_CODE(CLCTL_ENABLE_SHARED_VOLUME_DIRECTIO),
    CLUSCTL_RESOURCE_DISABLE_SHARED_VOLUME_DIRECTIO = CLUSCTL_RESOURCE_CODE(CLCTL_DISABLE_SHARED_VOLUME_DIRECTIO),
    CLUSCTL_RESOURCE_SET_SHARED_VOLUME_BACKUP_MODE = CLUSCTL_RESOURCE_CODE(CLCTL_SET_SHARED_VOLUME_BACKUP_MODE),
    CLUSCTL_RESOURCE_STORAGE_GET_SHARED_VOLUME_PARTITION_NAMES = CLUSCTL_RESOURCE_CODE(CLCTL_STORAGE_GET_SHARED_VOLUME_PARTITION_NAMES),
    CLUSCTL_RESOURCE_GET_FAILURE_INFO = CLUSCTL_RESOURCE_CODE(CLCTL_GET_FAILURE_INFO),
    CLUSCTL_RESOURCE_STORAGE_GET_DISKID = CLUSCTL_RESOURCE_CODE(CLCTL_STORAGE_GET_DISKID),
    CLUSCTL_RESOURCE_STORAGE_GET_SHARED_VOLUME_STATES = CLUSCTL_RESOURCE_CODE(CLCTL_STORAGE_GET_SHARED_VOLUME_STATES),
    CLUSCTL_RESOURCE_STORAGE_IS_SHARED_VOLUME = CLUSCTL_RESOURCE_CODE(CLCTL_STORAGE_IS_SHARED_VOLUME),
    CLUSCTL_RESOURCE_IS_QUORUM_BLOCKED = CLUSCTL_RESOURCE_CODE(CLCTL_IS_QUORUM_BLOCKED),
    CLUSCTL_RESOURCE_POOL_GET_DRIVE_INFO = CLUSCTL_RESOURCE_CODE(CLCTL_POOL_GET_DRIVE_INFO),
    CLUSCTL_RESOURCE_RLUA_GET_VIRTUAL_SERVER_TOKEN = CLUSCTL_RESOURCE_CODE(CLCTL_NETNAME_GET_VIRTUAL_SERVER_TOKEN),
    CLUSCTL_RESOURCE_RLUA_SET_PWD_INFO = CLUSCTL_RESOURCE_CODE(CLCTL_NETNAME_SET_PWD_INFO),
    /**/
    CLUSCTL_RESOURCE_DELETE = CLUSCTL_RESOURCE_CODE(CLCTL_DELETE),
    CLUSCTL_RESOURCE_INSTALL_NODE = CLUSCTL_RESOURCE_CODE(CLCTL_INSTALL_NODE),
    CLUSCTL_RESOURCE_EVICT_NODE = CLUSCTL_RESOURCE_CODE(CLCTL_EVICT_NODE),
    CLUSCTL_RESOURCE_ADD_DEPENDENCY = CLUSCTL_RESOURCE_CODE(CLCTL_ADD_DEPENDENCY),
    CLUSCTL_RESOURCE_REMOVE_DEPENDENCY = CLUSCTL_RESOURCE_CODE(CLCTL_REMOVE_DEPENDENCY),
    CLUSCTL_RESOURCE_ADD_OWNER = CLUSCTL_RESOURCE_CODE(CLCTL_ADD_OWNER),
    CLUSCTL_RESOURCE_REMOVE_OWNER = CLUSCTL_RESOURCE_CODE(CLCTL_REMOVE_OWNER),
    CLUSCTL_RESOURCE_SET_NAME = CLUSCTL_RESOURCE_CODE(CLCTL_SET_NAME),
    CLUSCTL_RESOURCE_CLUSTER_NAME_CHANGED = CLUSCTL_RESOURCE_CODE(CLCTL_CLUSTER_NAME_CHANGED),
    CLUSCTL_RESOURCE_CLUSTER_VERSION_CHANGED = CLUSCTL_RESOURCE_CODE(CLCTL_CLUSTER_VERSION_CHANGED),
    CLUSCTL_RESOURCE_FORCE_QUORUM = CLUSCTL_RESOURCE_CODE(CLCTL_FORCE_QUORUM),
    CLUSCTL_RESOURCE_INITIALIZE = CLUSCTL_RESOURCE_CODE(CLCTL_INITIALIZE),
    CLUSCTL_RESOURCE_STATE_CHANGE_REASON = CLUSCTL_RESOURCE_CODE(CLCTL_STATE_CHANGE_REASON),
    CLUSCTL_RESOURCE_PROVIDER_STATE_CHANGE = CLUSCTL_RESOURCE_CODE(CLCTL_PROVIDER_STATE_CHANGE),
    CLUSCTL_RESOURCE_LEAVING_GROUP = CLUSCTL_RESOURCE_CODE(CLCTL_LEAVING_GROUP),
    CLUSCTL_RESOURCE_JOINING_GROUP = CLUSCTL_RESOURCE_CODE(CLCTL_JOINING_GROUP),
    CLUSCTL_RESOURCE_FSWITNESS_GET_EPOCH_INFO = CLUSCTL_RESOURCE_CODE(CLCTL_FSWITNESS_GET_EPOCH_INFO),
    CLUSCTL_RESOURCE_FSWITNESS_SET_EPOCH_INFO = CLUSCTL_RESOURCE_CODE(CLCTL_FSWITNESS_SET_EPOCH_INFO),
    CLUSCTL_RESOURCE_FSWITNESS_RELEASE_LOCK = CLUSCTL_RESOURCE_CODE(CLCTL_FSWITNESS_RELEASE_LOCK),
    CLUSCTL_RESOURCE_NETNAME_CREDS_NOTIFYCAM = CLUSCTL_RESOURCE_CODE(CLCTL_NETNAME_CREDS_NOTIFYCAM),
    CLUSCTL_RESOURCE_GET_OPERATION_CONTEXT = CLUSCTL_RESOURCE_CODE(CLCTL_GET_OPERATION_CONTEXT),
    CLUSCTL_RESOURCE_RW_MODIFY_NOOP = CLUSCTL_RESOURCE_CODE(CLCTL_RW_MODIFY_NOOP),
    CLUSCTL_RESOURCE_NOTIFY_QUORUM_STATUS = CLUSCTL_RESOURCE_CODE(CLCTL_NOTIFY_QUORUM_STATUS),
    CLUSCTL_RESOURCE_NOTIFY_OWNER_CHANGE = CLUSCTL_RESOURCE_CODE(CLCTL_NOTIFY_OWNER_CHANGE),
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_WINTHRESHOLD)
    CLUSCTL_RESOURCE_VALIDATE_CHANGE_GROUP = CLUSCTL_RESOURCE_CODE(CLCTL_VALIDATE_CHANGE_GROUP),
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_WINTHRESHOLD */
    CLUSCTL_CLOUD_WITNESS_RESOURCE_UPDATE_TOKEN = CLUSCTL_RESOURCE_CODE(CLCTL_CLOUD_WITNESS_RESOURCE_UPDATE_TOKEN),
    CLUSCTL_CLOUD_WITNESS_RESOURCE_UPDATE_KEY = CLUSCTL_RESOURCE_CODE(CLCTL_CLOUD_WITNESS_RESOURCE_UPDATE_KEY),
    CLUSCTL_RESOURCE_PREPARE_UPGRADE = CLUSCTL_RESOURCE_CODE(CLCTL_RESOURCE_PREPARE_UPGRADE),
    CLUSCTL_RESOURCE_UPGRADE_COMPLETED = CLUSCTL_RESOURCE_CODE(CLCTL_RESOURCE_UPGRADE_COMPLETED),
    CLUSCTL_RESOURCE_GET_STATE_CHANGE_TIME = CLUSCTL_RESOURCE_CODE(CLCTL_GET_STATE_CHANGE_TIME)
} CLUSCTL_RESOURCE_CODES;

typedef enum CLUSCTL_RESOURCE_TYPE_CODES {
    CLUSCTL_RESOURCE_TYPE_UNKNOWN = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_UNKNOWN),
    CLUSCTL_RESOURCE_TYPE_GET_CHARACTERISTICS = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_GET_CHARACTERISTICS),
    CLUSCTL_RESOURCE_TYPE_GET_FLAGS = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_GET_FLAGS),
    CLUSCTL_RESOURCE_TYPE_GET_CLASS_INFO = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_GET_CLASS_INFO),
    CLUSCTL_RESOURCE_TYPE_GET_REQUIRED_DEPENDENCIES = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_GET_REQUIRED_DEPENDENCIES),
    CLUSCTL_RESOURCE_TYPE_GET_ARB_TIMEOUT = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_GET_ARB_TIMEOUT),
    CLUSCTL_RESOURCE_TYPE_ENUM_COMMON_PROPERTIES = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_ENUM_COMMON_PROPERTIES),
    CLUSCTL_RESOURCE_TYPE_GET_RO_COMMON_PROPERTIES = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_GET_RO_COMMON_PROPERTIES),
    CLUSCTL_RESOURCE_TYPE_GET_COMMON_PROPERTIES = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_GET_COMMON_PROPERTIES),
    CLUSCTL_RESOURCE_TYPE_VALIDATE_COMMON_PROPERTIES = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_VALIDATE_COMMON_PROPERTIES),
    CLUSCTL_RESOURCE_TYPE_SET_COMMON_PROPERTIES = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_SET_COMMON_PROPERTIES),
    CLUSCTL_RESOURCE_TYPE_GET_COMMON_PROPERTY_FMTS = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_GET_COMMON_PROPERTY_FMTS),
    CLUSCTL_RESOURCE_TYPE_GET_COMMON_RESOURCE_PROPERTY_FMTS = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_GET_COMMON_RESOURCE_PROPERTY_FMTS),
    CLUSCTL_RESOURCE_TYPE_ENUM_PRIVATE_PROPERTIES = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_ENUM_PRIVATE_PROPERTIES),
    CLUSCTL_RESOURCE_TYPE_GET_RO_PRIVATE_PROPERTIES = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_GET_RO_PRIVATE_PROPERTIES),
    CLUSCTL_RESOURCE_TYPE_GET_PRIVATE_PROPERTIES = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_GET_PRIVATE_PROPERTIES),
    CLUSCTL_RESOURCE_TYPE_SET_PRIVATE_PROPERTIES = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_SET_PRIVATE_PROPERTIES),
    CLUSCTL_RESOURCE_TYPE_VALIDATE_PRIVATE_PROPERTIES = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_VALIDATE_PRIVATE_PROPERTIES),
    CLUSCTL_RESOURCE_TYPE_GET_PRIVATE_PROPERTY_FMTS = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_GET_PRIVATE_PROPERTY_FMTS),
    CLUSCTL_RESOURCE_TYPE_GET_PRIVATE_RESOURCE_PROPERTY_FMTS = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_GET_PRIVATE_RESOURCE_PROPERTY_FMTS),
    CLUSCTL_RESOURCE_TYPE_GET_REGISTRY_CHECKPOINTS = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_GET_REGISTRY_CHECKPOINTS),
    CLUSCTL_RESOURCE_TYPE_GET_CRYPTO_CHECKPOINTS = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_GET_CRYPTO_CHECKPOINTS),
    CLUSCTL_RESOURCE_TYPE_STORAGE_GET_AVAILABLE_DISKS = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_STORAGE_GET_AVAILABLE_DISKS),
    CLUSCTL_RESOURCE_TYPE_STORAGE_SYNC_CLUSDISK_DB = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_STORAGE_SYNC_CLUSDISK_DB),
    CLUSCTL_RESOURCE_TYPE_NETNAME_VALIDATE_NETNAME = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_VALIDATE_NETNAME),
    CLUSCTL_RESOURCE_TYPE_NETNAME_GET_OU_FOR_VCO = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_NETNAME_GET_OU_FOR_VCO),
    CLUSCTL_RESOURCE_TYPE_GEN_APP_VALIDATE_PATH = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_VALIDATE_PATH),
    CLUSCTL_RESOURCE_TYPE_GEN_APP_VALIDATE_DIRECTORY = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_VALIDATE_DIRECTORY),
    CLUSCTL_RESOURCE_TYPE_GEN_SCRIPT_VALIDATE_PATH = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_VALIDATE_PATH),
    CLUSCTL_RESOURCE_TYPE_QUERY_DELETE = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_QUERY_DELETE),
    CLUSCTL_RESOURCE_TYPE_STORAGE_GET_DRIVELETTERS = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_STORAGE_GET_DRIVELETTERS),
    CLUSCTL_RESOURCE_TYPE_STORAGE_GET_AVAILABLE_DISKS_EX = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_STORAGE_GET_AVAILABLE_DISKS_EX),
    CLUSCTL_RESOURCE_TYPE_STORAGE_REMAP_DRIVELETTER = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_STORAGE_REMAP_DRIVELETTER),
    CLUSCTL_RESOURCE_TYPE_STORAGE_GET_DISKID = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_STORAGE_GET_DISKID),
    CLUSCTL_RESOURCE_TYPE_STORAGE_GET_RESOURCEID = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_STORAGE_GET_RESOURCEID),
    CLUSCTL_RESOURCE_TYPE_STORAGE_IS_CLUSTERABLE = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_STORAGE_IS_CLUSTERABLE),
    CLUSCTL_RESOURCE_TYPE_STORAGE_REMOVE_VM_OWNERSHIP = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_STORAGE_REMOVE_VM_OWNERSHIP),
    CLUSCTL_RESOURCE_TYPE_STORAGE_IS_CSV_FILE = CLUSCTL_RESOURCE_CODE(CLCTL_STORAGE_IS_CSV_FILE),
    CLUSCTL_RESOURCE_TYPE_WITNESS_VALIDATE_PATH = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_VALIDATE_PATH),
    /**/
    CLUSCTL_RESOURCE_TYPE_INSTALL_NODE = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_INSTALL_NODE),
    CLUSCTL_RESOURCE_TYPE_EVICT_NODE = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_EVICT_NODE),
    CLUSCTL_RESOURCE_TYPE_CLUSTER_VERSION_CHANGED = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_CLUSTER_VERSION_CHANGED),
    CLUSCTL_RESOURCE_TYPE_FIXUP_ON_UPGRADE = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_FIXUP_ON_UPGRADE),
    CLUSCTL_RESOURCE_TYPE_STARTING_PHASE1 = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_STARTING_PHASE1),
    CLUSCTL_RESOURCE_TYPE_STARTING_PHASE2 = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_STARTING_PHASE2),
    CLUSCTL_RESOURCE_TYPE_HOLD_IO = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_HOLD_IO),
    CLUSCTL_RESOURCE_TYPE_RESUME_IO = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_RESUME_IO),
    CLUSCTL_RESOURCE_TYPE_STORAGE_GET_AVAILABLE_DISKS_EX2_INT = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_STORAGE_GET_AVAILABLE_DISKS_EX2_INT),
    CLUSCTL_RESOURCE_TYPE_REPLICATION_GET_ELIGIBLE_LOGDISKS = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_REPLICATION_GET_ELIGIBLE_LOGDISKS),
    CLUSCTL_RESOURCE_TYPE_REPLICATION_GET_ELIGIBLE_TARGET_DATADISKS = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_REPLICATION_GET_ELIGIBLE_TARGET_DATADISKS),
    CLUSCTL_RESOURCE_TYPE_REPLICATION_GET_ELIGIBLE_SOURCE_DATADISKS = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_REPLICATION_GET_ELIGIBLE_SOURCE_DATADISKS),
    CLUSCTL_RESOURCE_TYPE_REPLICATION_GET_REPLICATED_DISKS = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_REPLICATION_GET_REPLICATED_DISKS),
    CLUSCTL_RESOURCE_TYPE_REPLICATION_GET_REPLICA_VOLUMES = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_REPLICATION_GET_REPLICA_VOLUMES),
    CLUSCTL_RESOURCE_TYPE_REPLICATION_GET_LOG_VOLUME = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_REPLICATION_GET_LOG_VOLUME),
    CLUSCTL_RESOURCE_TYPE_REPLICATION_GET_RESOURCE_GROUP = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_REPLICATION_GET_RESOURCE_GROUP),
    CLUSCTL_RESOURCE_TYPE_REPLICATION_GET_REPLICATED_PARTITION_INFO = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_REPLICATION_GET_REPLICATED_PARTITION_INFO),
    CLUSCTL_RESOURCE_TYPE_REPLICATION_GET_LOG_INFO = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_REPLICATION_GET_LOG_INFO),
    CLUSCTL_CLOUD_WITNESS_RESOURCE_TYPE_VALIDATE_CREDENTIALS = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_CLOUD_WITNESS_RESOURCE_TYPE_VALIDATE_CREDENTIALS),
    CLUSCTL_CLOUD_WITNESS_RESOURCE_TYPE_VALIDATE_CREDENTIALS_WITH_KEY = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_CLOUD_WITNESS_RESOURCE_TYPE_VALIDATE_CREDENTIALS_WITH_KEY),
    CLUSCTL_RESOURCE_TYPE_PREPARE_UPGRADE = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_RESOURCE_PREPARE_UPGRADE),
    CLUSCTL_RESOURCE_TYPE_UPGRADE_COMPLETED = CLUSCTL_RESOURCE_TYPE_CODE(CLCTL_RESOURCE_UPGRADE_COMPLETED),
} CLUSCTL_RESOURCE_TYPE_CODES;

typedef enum CLUSCTL_GROUP_CODES {
    CLUSCTL_GROUP_UNKNOWN = CLUSCTL_GROUP_CODE(CLCTL_UNKNOWN),
    CLUSCTL_GROUP_GET_CHARACTERISTICS = CLUSCTL_GROUP_CODE(CLCTL_GET_CHARACTERISTICS),
    CLUSCTL_GROUP_GET_FLAGS = CLUSCTL_GROUP_CODE(CLCTL_GET_FLAGS),
    CLUSCTL_GROUP_GET_NAME = CLUSCTL_GROUP_CODE(CLCTL_GET_NAME),
    CLUSCTL_GROUP_GET_ID = CLUSCTL_GROUP_CODE(CLCTL_GET_ID),
    CLUSCTL_GROUP_ENUM_COMMON_PROPERTIES = CLUSCTL_GROUP_CODE(CLCTL_ENUM_COMMON_PROPERTIES),
    CLUSCTL_GROUP_GET_RO_COMMON_PROPERTIES = CLUSCTL_GROUP_CODE(CLCTL_GET_RO_COMMON_PROPERTIES),
    CLUSCTL_GROUP_GET_COMMON_PROPERTIES = CLUSCTL_GROUP_CODE(CLCTL_GET_COMMON_PROPERTIES),
    CLUSCTL_GROUP_SET_COMMON_PROPERTIES = CLUSCTL_GROUP_CODE(CLCTL_SET_COMMON_PROPERTIES),
    CLUSCTL_GROUP_VALIDATE_COMMON_PROPERTIES = CLUSCTL_GROUP_CODE(CLCTL_VALIDATE_COMMON_PROPERTIES),
    CLUSCTL_GROUP_ENUM_PRIVATE_PROPERTIES = CLUSCTL_GROUP_CODE(CLCTL_ENUM_PRIVATE_PROPERTIES),
    CLUSCTL_GROUP_GET_RO_PRIVATE_PROPERTIES = CLUSCTL_GROUP_CODE(CLCTL_GET_RO_PRIVATE_PROPERTIES),
    CLUSCTL_GROUP_GET_PRIVATE_PROPERTIES = CLUSCTL_GROUP_CODE(CLCTL_GET_PRIVATE_PROPERTIES),
    CLUSCTL_GROUP_SET_PRIVATE_PROPERTIES = CLUSCTL_GROUP_CODE(CLCTL_SET_PRIVATE_PROPERTIES),
    CLUSCTL_GROUP_VALIDATE_PRIVATE_PROPERTIES = CLUSCTL_GROUP_CODE(CLCTL_VALIDATE_PRIVATE_PROPERTIES),
    CLUSCTL_GROUP_QUERY_DELETE = CLUSCTL_GROUP_CODE(CLCTL_QUERY_DELETE),
    CLUSCTL_GROUP_GET_COMMON_PROPERTY_FMTS = CLUSCTL_GROUP_CODE(CLCTL_GET_COMMON_PROPERTY_FMTS),
    CLUSCTL_GROUP_GET_PRIVATE_PROPERTY_FMTS = CLUSCTL_GROUP_CODE(CLCTL_GET_PRIVATE_PROPERTY_FMTS),
    CLUSCTL_GROUP_GET_FAILURE_INFO = CLUSCTL_GROUP_CODE(CLCTL_GET_FAILURE_INFO),
    CLUSCTL_GROUP_GET_LAST_MOVE_TIME = CLUSCTL_GROUP_CODE(CLCTL_GROUP_GET_LAST_MOVE_TIME),
} CLUSCTL_GROUP_CODES;

typedef enum CLUSCTL_NODE_CODES {
    CLUSCTL_NODE_UNKNOWN = CLUSCTL_NODE_CODE(CLCTL_UNKNOWN),
    CLUSCTL_NODE_GET_CHARACTERISTICS = CLUSCTL_NODE_CODE(CLCTL_GET_CHARACTERISTICS),
    CLUSCTL_NODE_GET_FLAGS = CLUSCTL_NODE_CODE(CLCTL_GET_FLAGS),
    CLUSCTL_NODE_GET_NAME = CLUSCTL_NODE_CODE(CLCTL_GET_NAME),
    CLUSCTL_NODE_GET_ID = CLUSCTL_NODE_CODE(CLCTL_GET_ID),
    CLUSCTL_NODE_ENUM_COMMON_PROPERTIES = CLUSCTL_NODE_CODE(CLCTL_ENUM_COMMON_PROPERTIES),
    CLUSCTL_NODE_GET_RO_COMMON_PROPERTIES = CLUSCTL_NODE_CODE(CLCTL_GET_RO_COMMON_PROPERTIES),
    CLUSCTL_NODE_GET_COMMON_PROPERTIES = CLUSCTL_NODE_CODE(CLCTL_GET_COMMON_PROPERTIES),
    CLUSCTL_NODE_SET_COMMON_PROPERTIES = CLUSCTL_NODE_CODE(CLCTL_SET_COMMON_PROPERTIES),
    CLUSCTL_NODE_VALIDATE_COMMON_PROPERTIES = CLUSCTL_NODE_CODE(CLCTL_VALIDATE_COMMON_PROPERTIES),
    CLUSCTL_NODE_ENUM_PRIVATE_PROPERTIES = CLUSCTL_NODE_CODE(CLCTL_ENUM_PRIVATE_PROPERTIES),
    CLUSCTL_NODE_GET_RO_PRIVATE_PROPERTIES = CLUSCTL_NODE_CODE(CLCTL_GET_RO_PRIVATE_PROPERTIES),
    CLUSCTL_NODE_GET_PRIVATE_PROPERTIES = CLUSCTL_NODE_CODE(CLCTL_GET_PRIVATE_PROPERTIES),
    CLUSCTL_NODE_SET_PRIVATE_PROPERTIES = CLUSCTL_NODE_CODE(CLCTL_SET_PRIVATE_PROPERTIES),
    CLUSCTL_NODE_VALIDATE_PRIVATE_PROPERTIES = CLUSCTL_NODE_CODE(CLCTL_VALIDATE_PRIVATE_PROPERTIES),
    CLUSCTL_NODE_GET_COMMON_PROPERTY_FMTS = CLUSCTL_NODE_CODE(CLCTL_GET_COMMON_PROPERTY_FMTS),
    CLUSCTL_NODE_GET_PRIVATE_PROPERTY_FMTS = CLUSCTL_NODE_CODE(CLCTL_GET_PRIVATE_PROPERTY_FMTS),
    CLUSCTL_NODE_GET_CLUSTER_SERVICE_ACCOUNT_NAME = CLUSCTL_NODE_CODE(CLCTL_GET_CLUSTER_SERVICE_ACCOUNT_NAME),
    CLUSCTL_NODE_GET_STUCK_NODES = CLUSCTL_NODE_CODE(CLCTL_GET_STUCK_NODES),
    CLUSCTL_NODE_INJECT_GEM_FAULT = CLUSCTL_NODE_CODE(CLCTL_INJECT_GEM_FAULT),
    CLUSCTL_NODE_INTRODUCE_GEM_REPAIR_DELAY = CLUSCTL_NODE_CODE(CLCTL_INTRODUCE_GEM_REPAIR_DELAY),
    CLUSCTL_NODE_SEND_DUMMY_GEM_MESSAGES = CLUSCTL_NODE_CODE(CLCTL_SEND_DUMMY_GEM_MESSAGES),
    CLUSCTL_NODE_BLOCK_GEM_SEND_RECV = CLUSCTL_NODE_CODE(CLCTL_BLOCK_GEM_SEND_RECV),
    CLUSCTL_NODE_GET_GEMID_VECTOR = CLUSCTL_NODE_CODE(CLCTL_GET_GEMID_VECTOR),
} CLUSCTL_NODE_CODES;

typedef enum CLUSCTL_NETWORK_CODES {
    CLUSCTL_NETWORK_UNKNOWN = CLUSCTL_NETWORK_CODE(CLCTL_UNKNOWN),
    CLUSCTL_NETWORK_GET_CHARACTERISTICS = CLUSCTL_NETWORK_CODE(CLCTL_GET_CHARACTERISTICS),
    CLUSCTL_NETWORK_GET_FLAGS = CLUSCTL_NETWORK_CODE(CLCTL_GET_FLAGS),
    CLUSCTL_NETWORK_GET_NAME = CLUSCTL_NETWORK_CODE(CLCTL_GET_NAME),
    CLUSCTL_NETWORK_GET_ID = CLUSCTL_NETWORK_CODE(CLCTL_GET_ID),
    CLUSCTL_NETWORK_ENUM_COMMON_PROPERTIES = CLUSCTL_NETWORK_CODE(CLCTL_ENUM_COMMON_PROPERTIES),
    CLUSCTL_NETWORK_GET_RO_COMMON_PROPERTIES = CLUSCTL_NETWORK_CODE(CLCTL_GET_RO_COMMON_PROPERTIES),
    CLUSCTL_NETWORK_GET_COMMON_PROPERTIES = CLUSCTL_NETWORK_CODE(CLCTL_GET_COMMON_PROPERTIES),
    CLUSCTL_NETWORK_SET_COMMON_PROPERTIES = CLUSCTL_NETWORK_CODE(CLCTL_SET_COMMON_PROPERTIES),
    CLUSCTL_NETWORK_VALIDATE_COMMON_PROPERTIES = CLUSCTL_NETWORK_CODE(CLCTL_VALIDATE_COMMON_PROPERTIES),
    CLUSCTL_NETWORK_ENUM_PRIVATE_PROPERTIES = CLUSCTL_NETWORK_CODE(CLCTL_ENUM_PRIVATE_PROPERTIES),
    CLUSCTL_NETWORK_GET_RO_PRIVATE_PROPERTIES = CLUSCTL_NETWORK_CODE(CLCTL_GET_RO_PRIVATE_PROPERTIES),
    CLUSCTL_NETWORK_GET_PRIVATE_PROPERTIES = CLUSCTL_NETWORK_CODE(CLCTL_GET_PRIVATE_PROPERTIES),
    CLUSCTL_NETWORK_SET_PRIVATE_PROPERTIES = CLUSCTL_NETWORK_CODE(CLCTL_SET_PRIVATE_PROPERTIES),
    CLUSCTL_NETWORK_VALIDATE_PRIVATE_PROPERTIES = CLUSCTL_NETWORK_CODE(CLCTL_VALIDATE_PRIVATE_PROPERTIES),
    CLUSCTL_NETWORK_GET_COMMON_PROPERTY_FMTS = CLUSCTL_NETWORK_CODE(CLCTL_GET_COMMON_PROPERTY_FMTS),
    CLUSCTL_NETWORK_GET_PRIVATE_PROPERTY_FMTS = CLUSCTL_NETWORK_CODE(CLCTL_GET_PRIVATE_PROPERTY_FMTS),
} CLUSCTL_NETWORK_CODES;

typedef enum CLUSCTL_NETINTERFACE_CODES {
    CLUSCTL_NETINTERFACE_UNKNOWN = CLUSCTL_NETINTERFACE_CODE(CLCTL_UNKNOWN),
    CLUSCTL_NETINTERFACE_GET_CHARACTERISTICS = CLUSCTL_NETINTERFACE_CODE(CLCTL_GET_CHARACTERISTICS),
    CLUSCTL_NETINTERFACE_GET_FLAGS = CLUSCTL_NETINTERFACE_CODE(CLCTL_GET_FLAGS),
    CLUSCTL_NETINTERFACE_GET_NAME = CLUSCTL_NETINTERFACE_CODE(CLCTL_GET_NAME),
    CLUSCTL_NETINTERFACE_GET_ID = CLUSCTL_NETINTERFACE_CODE(CLCTL_GET_ID),
    CLUSCTL_NETINTERFACE_GET_NODE = CLUSCTL_NETINTERFACE_CODE(CLCTL_GET_NODE),
    CLUSCTL_NETINTERFACE_GET_NETWORK = CLUSCTL_NETINTERFACE_CODE(CLCTL_GET_NETWORK),
    CLUSCTL_NETINTERFACE_ENUM_COMMON_PROPERTIES = CLUSCTL_NETINTERFACE_CODE(CLCTL_ENUM_COMMON_PROPERTIES),
    CLUSCTL_NETINTERFACE_GET_RO_COMMON_PROPERTIES = CLUSCTL_NETINTERFACE_CODE(CLCTL_GET_RO_COMMON_PROPERTIES),
    CLUSCTL_NETINTERFACE_GET_COMMON_PROPERTIES = CLUSCTL_NETINTERFACE_CODE(CLCTL_GET_COMMON_PROPERTIES),
    CLUSCTL_NETINTERFACE_SET_COMMON_PROPERTIES = CLUSCTL_NETINTERFACE_CODE(CLCTL_SET_COMMON_PROPERTIES),
    CLUSCTL_NETINTERFACE_VALIDATE_COMMON_PROPERTIES = CLUSCTL_NETINTERFACE_CODE(CLCTL_VALIDATE_COMMON_PROPERTIES),
    CLUSCTL_NETINTERFACE_ENUM_PRIVATE_PROPERTIES = CLUSCTL_NETINTERFACE_CODE(CLCTL_ENUM_PRIVATE_PROPERTIES),
    CLUSCTL_NETINTERFACE_GET_RO_PRIVATE_PROPERTIES = CLUSCTL_NETINTERFACE_CODE(CLCTL_GET_RO_PRIVATE_PROPERTIES),
    CLUSCTL_NETINTERFACE_GET_PRIVATE_PROPERTIES = CLUSCTL_NETINTERFACE_CODE(CLCTL_GET_PRIVATE_PROPERTIES),
    CLUSCTL_NETINTERFACE_SET_PRIVATE_PROPERTIES = CLUSCTL_NETINTERFACE_CODE(CLCTL_SET_PRIVATE_PROPERTIES),
    CLUSCTL_NETINTERFACE_VALIDATE_PRIVATE_PROPERTIES = CLUSCTL_NETINTERFACE_CODE(CLCTL_VALIDATE_PRIVATE_PROPERTIES),
    CLUSCTL_NETINTERFACE_GET_COMMON_PROPERTY_FMTS = CLUSCTL_NETINTERFACE_CODE(CLCTL_GET_COMMON_PROPERTY_FMTS),
    CLUSCTL_NETINTERFACE_GET_PRIVATE_PROPERTY_FMTS = CLUSCTL_NETINTERFACE_CODE(CLCTL_GET_PRIVATE_PROPERTY_FMTS),
} CLUSCTL_NETINTERFACE_CODES;

typedef enum CLUSCTL_CLUSTER_CODES {
    CLUSCTL_CLUSTER_UNKNOWN = CLUSCTL_CLUSTER_CODE(CLCTL_UNKNOWN),
    CLUSCTL_CLUSTER_GET_FQDN = CLUSCTL_CLUSTER_CODE(CLCTL_GET_FQDN),
    CLUSCTL_CLUSTER_SET_STORAGE_CONFIGURATION = CLUSCTL_CLUSTER_CODE(CLCTL_SET_STORAGE_CONFIGURATION),
    CLUSCTL_CLUSTER_GET_STORAGE_CONFIGURATION = CLUSCTL_CLUSTER_CODE(CLCTL_GET_STORAGE_CONFIGURATION),
    CLUSCTL_CLUSTER_GET_STORAGE_CONFIG_ATTRIBUTES = CLUSCTL_CLUSTER_CODE(CLCTL_GET_STORAGE_CONFIG_ATTRIBUTES),
    CLUSCTL_CLUSTER_ENUM_COMMON_PROPERTIES = CLUSCTL_CLUSTER_CODE(CLCTL_ENUM_COMMON_PROPERTIES),
    CLUSCTL_CLUSTER_GET_RO_COMMON_PROPERTIES = CLUSCTL_CLUSTER_CODE(CLCTL_GET_RO_COMMON_PROPERTIES),
    CLUSCTL_CLUSTER_GET_COMMON_PROPERTIES = CLUSCTL_CLUSTER_CODE(CLCTL_GET_COMMON_PROPERTIES),
    CLUSCTL_CLUSTER_SET_COMMON_PROPERTIES = CLUSCTL_CLUSTER_CODE(CLCTL_SET_COMMON_PROPERTIES),
    CLUSCTL_CLUSTER_VALIDATE_COMMON_PROPERTIES = CLUSCTL_CLUSTER_CODE(CLCTL_VALIDATE_COMMON_PROPERTIES),
    CLUSCTL_CLUSTER_ENUM_PRIVATE_PROPERTIES = CLUSCTL_CLUSTER_CODE(CLCTL_ENUM_PRIVATE_PROPERTIES),
    CLUSCTL_CLUSTER_GET_RO_PRIVATE_PROPERTIES = CLUSCTL_CLUSTER_CODE(CLCTL_GET_RO_PRIVATE_PROPERTIES),
    CLUSCTL_CLUSTER_GET_PRIVATE_PROPERTIES = CLUSCTL_CLUSTER_CODE(CLCTL_GET_PRIVATE_PROPERTIES),
    CLUSCTL_CLUSTER_SET_PRIVATE_PROPERTIES = CLUSCTL_CLUSTER_CODE(CLCTL_SET_PRIVATE_PROPERTIES),
    CLUSCTL_CLUSTER_VALIDATE_PRIVATE_PROPERTIES = CLUSCTL_CLUSTER_CODE(CLCTL_VALIDATE_PRIVATE_PROPERTIES),
    CLUSCTL_CLUSTER_GET_COMMON_PROPERTY_FMTS = CLUSCTL_CLUSTER_CODE(CLCTL_GET_COMMON_PROPERTY_FMTS),
    CLUSCTL_CLUSTER_GET_PRIVATE_PROPERTY_FMTS = CLUSCTL_CLUSTER_CODE(CLCTL_GET_PRIVATE_PROPERTY_FMTS),
    CLUSCTL_CLUSTER_CHECK_VOTER_EVICT = CLUSCTL_CLUSTER_CODE(CLCTL_CHECK_VOTER_EVICT),
    CLUSCTL_CLUSTER_CHECK_VOTER_DOWN = CLUSCTL_CLUSTER_CODE(CLCTL_CHECK_VOTER_DOWN),
    CLUSCTL_CLUSTER_SHUTDOWN = CLUSCTL_CLUSTER_CODE(CLCTL_SHUTDOWN),
    CLUSCTL_CLUSTER_BATCH_BLOCK_KEY = CLUSCTL_CLUSTER_CODE(CLCTL_BATCH_BLOCK_KEY),
    CLUSCTL_CLUSTER_BATCH_UNBLOCK_KEY = CLUSCTL_CLUSTER_CODE(CLCTL_BATCH_UNBLOCK_KEY),
    CLUSCTL_CLUSTER_GET_SHARED_VOLUME_ID = CLUSCTL_CLUSTER_CODE(CLCTL_GET_SHARED_VOLUME_ID),
    CLUSCTL_CLUSTER_GET_CLUSDB_TIMESTAMP = CLUSCTL_CLUSTER_CODE(CLCTL_GET_CLUSDB_TIMESTAMP),
    CLUSCTL_CLUSTER_GET_GUM_LOCK_OWNER = CLUSCTL_CLUSTER_CODE(CLCTL_GET_GUM_LOCK_OWNER),
    CLUSCTL_CLUSTER_REMOVE_NODE = CLUSCTL_CLUSTER_CODE(CLCTL_REMOVE_NODE),
    CLUSCTL_CLUSTER_SET_ACCOUNT_ACCESS = CLUSCTL_CLUSTER_CODE(CLCTL_SET_ACCOUNT_ACCESS),
    CLUSCTL_CLUSTER_CLEAR_NODE_CONNECTION_INFO = CLUSCTL_CLUSTER_CODE(CLCTL_CLEAR_NODE_CONNECTION_INFO),
    CLUSCTL_CLUSTER_SET_DNS_DOMAIN = CLUSCTL_CLUSTER_CODE(CLCTL_SET_DNS_DOMAIN),
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_WINTHRESHOLD)
    CLUSCTL_CLUSTER_SET_CLUSTER_S2D_ENABLED = CLUSCTL_CLUSTER_CODE(CLCTL_SET_CLUSTER_S2D_ENABLED),
    CLUSCTL_CLUSTER_SET_CLUSTER_S2D_CACHE_METADATA_RESERVE_BYTES = CLUSCTL_CLUSTER_CODE(CLCTL_SET_CLUSTER_S2D_CACHE_METADATA_RESERVE_BYTES),
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_WINTHRESHOLD */
} CLUSCTL_CLUSTER_CODES;

typedef enum CLUSCTL_GROUPSET_CODES {
    CLUSCTL_GROUPSET_GET_COMMON_PROPERTIES = CLUSCTL_GROUPSET_CODE(CLCTL_GET_COMMON_PROPERTIES),
    CLUSCTL_GROUPSET_GET_RO_COMMON_PROPERTIES = CLUSCTL_GROUPSET_CODE(CLCTL_GET_RO_COMMON_PROPERTIES),
    CLUSCTL_GROUPSET_SET_COMMON_PROPERTIES = CLUSCTL_GROUPSET_CODE(CLCTL_SET_COMMON_PROPERTIES),
    CLUSCTL_GROUPSET_GET_GROUPS = CLUSCTL_GROUPSET_CODE(CLCTL_GROUPSET_GET_GROUPS),
    CLUSCTL_GROUPSET_GET_PROVIDER_GROUPS = CLUSCTL_GROUPSET_CODE(CLCTL_GROUPSET_GET_PROVIDER_GROUPS),
    CLUSCTL_GROUPSET_GET_PROVIDER_GROUPSETS = CLUSCTL_GROUPSET_CODE(CLCTL_GROUPSET_GET_PROVIDER_GROUPSETS),
    CLUSCTL_GROUP_GET_PROVIDER_GROUPS = CLUSCTL_GROUPSET_CODE(CLCTL_GROUP_GET_PROVIDER_GROUPS),
    CLUSCTL_GROUP_GET_PROVIDER_GROUPSETS = CLUSCTL_GROUPSET_CODE(CLCTL_GROUP_GET_PROVIDER_GROUPSETS),
    CLUSCTL_GROUPSET_GET_ID = CLUSCTL_GROUPSET_CODE(CLCTL_GET_ID),
} CLUSCTL_GROUPSET_CODES;

typedef enum CLUSTER_RESOURCE_CLASS {
    CLUS_RESCLASS_UNKNOWN = 0,
    CLUS_RESCLASS_STORAGE,
    CLUS_RESCLASS_NETWORK,
    CLUS_RESCLASS_USER = 32768
} CLUSTER_RESOURCE_CLASS;

typedef enum CLUS_RESSUBCLASS {
    CLUS_RESSUBCLASS_SHARED = 0x80000000
} CLUS_RESSUBCLASS;

typedef enum CLUS_RESSUBCLASS_STORAGE {
    CLUS_RESSUBCLASS_STORAGE_SHARED_BUS = 0x80000000,
    CLUS_RESSUBCLASS_STORAGE_DISK = 0x40000000,
    CLUS_RESSUBCLASS_STORAGE_REPLICATION = 0x10000000
} CLUS_RESSUBCLASS_STORAGE;

typedef enum CLUS_RESSUBCLASS_NETWORK {
    CLUS_RESSUBCLASS_NETWORK_INTERNET_PROTOCOL = 0x80000000
} CLUS_RESSUBCLASS_NETWORK;

typedef enum CLUS_CHARACTERISTICS {
    CLUS_CHAR_UNKNOWN = 0x00000000,
    CLUS_CHAR_QUORUM = 0x00000001,
    CLUS_CHAR_DELETE_REQUIRES_ALL_NODES = 0x00000002,
    CLUS_CHAR_LOCAL_QUORUM = 0x00000004,
    CLUS_CHAR_LOCAL_QUORUM_DEBUG = 0x00000008,
    CLUS_CHAR_REQUIRES_STATE_CHANGE_REASON = 0x00000010,
    CLUS_CHAR_BROADCAST_DELETE = 0x00000020,
    CLUS_CHAR_SINGLE_CLUSTER_INSTANCE = 0x00000040,
    CLUS_CHAR_SINGLE_GROUP_INSTANCE = 0x00000080,
    CLUS_CHAR_COEXIST_IN_SHARED_VOLUME_GROUP = 0x00000100,
    CLUS_CHAR_PLACEMENT_DATA = 0x00000200,
    CLUS_CHAR_MONITOR_DETACH = 0x00000400,
    CLUS_CHAR_MONITOR_REATTACH = 0x00000800,
    CLUS_CHAR_OPERATION_CONTEXT = 0x00001000,
    CLUS_CHAR_CLONES = 0x00002000,
    CLUS_CHAR_NOT_PREEMPTABLE = 0x00004000,
    CLUS_CHAR_NOTIFY_NEW_OWNER = 0x00008000,
    CLUS_CHAR_SUPPORTS_UNMONITORED_STATE = 0x00010000,
    CLUS_CHAR_INFRASTRUCTURE = 0x00020000
} CLUS_CHARACTERISTICS;

typedef enum CLUS_FLAGS {
    CLUS_FLAG_CORE = 0x00000001
} CLUS_FLAGS;

typedef union CLUSPROP_SYNTAX {
    DWORD dw;
    struct {
        WORD wFormat;
        WORD wType;
    } DUMMYSTRUCTNAME;
} CLUSPROP_SYNTAX, *PCLUSPROP_SYNTAX;

typedef struct CLUSPROP_VALUE {
    CLUSPROP_SYNTAX Syntax;
    DWORD cbLength;
} CLUSPROP_VALUE, *PCLUSPROP_VALUE;

#ifdef __cplusplus
typedef struct CLUSPROP_BINARY:public CLUSPROP_VALUE {
#else
typedef struct CLUSPROP_BINARY {
    CLUSPROP_VALUE;
#endif
    BYTE rgb[];
} CLUSPROP_BINARY, *PCLUSPROP_BINARY;

#ifdef __cplusplus
typedef struct CLUSPROP_WORD:public CLUSPROP_VALUE {
#else
typedef struct CLUSPROP_WORD {
    CLUSPROP_VALUE;
#endif
    WORD w;
} CLUSPROP_WORD, *PCLUSPROP_WORD;

#ifdef __cplusplus
typedef struct CLUSPROP_DWORD:public CLUSPROP_VALUE {
#else
typedef struct CLUSPROP_DWORD {
    CLUSPROP_VALUE;
#endif
    DWORD dw;
} CLUSPROP_DWORD, *PCLUSPROP_DWORD;

#ifdef __cplusplus
typedef struct CLUSPROP_LONG:public CLUSPROP_VALUE {
#else
typedef struct CLUSPROP_LONG {
    CLUSPROP_VALUE;
#endif
    LONG l;
} CLUSPROP_LONG, *PCLUSPROP_LONG;

#ifdef __cplusplus
typedef struct CLUSPROP_SZ:public CLUSPROP_VALUE {
#else
typedef struct CLUSPROP_SZ {
    CLUSPROP_VALUE;
#endif
    WCHAR sz[];
} CLUSPROP_SZ, *PCLUSPROP_SZ;

typedef CLUSPROP_SZ CLUSPROP_MULTI_SZ, *PCLUSPROP_MULTI_SZ;
typedef CLUSPROP_SZ CLUSPROP_PROPERTY_NAME, *PCLUSPROP_PROPERTY_NAME;

#ifdef __cplusplus
typedef struct CLUSPROP_ULARGE_INTEGER:public CLUSPROP_VALUE {
#else
typedef struct CLUSPROP_ULARGE_INTEGER {
    CLUSPROP_VALUE;
#endif
    ULARGE_INTEGER li;
} CLUSPROP_ULARGE_INTEGER;

typedef CLUSPROP_ULARGE_INTEGER UNALIGNED *PCLUSPROP_ULARGE_INTEGER;

#ifdef __cplusplus
typedef struct CLUSPROP_LARGE_INTEGER:public CLUSPROP_VALUE {
#else
typedef struct CLUSPROP_LARGE_INTEGER {
    CLUSPROP_VALUE;
#endif
    LARGE_INTEGER li;
} CLUSPROP_LARGE_INTEGER;

typedef CLUSPROP_LARGE_INTEGER UNALIGNED *PCLUSPROP_LARGE_INTEGER;

#ifdef __cplusplus
typedef struct CLUSPROP_SECURITY_DESCRIPTOR:public CLUSPROP_VALUE {
#else
typedef struct CLUSPROP_SECURITY_DESCRIPTOR {
    CLUSPROP_VALUE;
#endif
    union {
        SECURITY_DESCRIPTOR_RELATIVE sd;
        BYTE rgbSecurityDescriptor[];
    } DUMMYUNIONNAME;
} CLUSPROP_SECURITY_DESCRIPTOR, *PCLUSPROP_SECURITY_DESCRIPTOR;

#ifdef __cplusplus
typedef struct CLUSPROP_FILETIME:public CLUSPROP_VALUE {
#else
typedef struct CLUSPROP_FILETIME {
    CLUSPROP_VALUE;
#endif
    FILETIME ft;
} CLUSPROP_FILETIME, *PCLUSPROP_FILETIME;

typedef struct CLUS_RESOURCE_CLASS_INFO {
    union {
        struct {
            union {
                DWORD dw;
                CLUSTER_RESOURCE_CLASS rc;
            } DUMMYUNIONNAME;
            DWORD SubClass;
        } DUMMYSTRUCTNAME;
        ULARGE_INTEGER li;
    } DUMMYUNIONNAME;
} CLUS_RESOURCE_CLASS_INFO, *PCLUS_RESOURCE_CLASS_INFO;

#ifdef __cplusplus
typedef struct CLUSPROP_RESOURCE_CLASS:public CLUSPROP_VALUE {
#else
typedef struct CLUSPROP_RESOURCE_CLASS {
    CLUSPROP_VALUE;
#endif
    CLUSTER_RESOURCE_CLASS rc;
} CLUSPROP_RESOURCE_CLASS, *PCLUSPROP_RESOURCE_CLASS;

#ifdef __cplusplus
typedef struct CLUSPROP_RESOURCE_CLASS_INFO:public CLUSPROP_VALUE, public CLUS_RESOURCE_CLASS_INFO {
#else
typedef struct CLUSPROP_RESOURCE_CLASS_INFO {
    CLUSPROP_VALUE;
    CLUS_RESOURCE_CLASS_INFO;
#endif
} CLUSPROP_RESOURCE_CLASS_INFO, *PCLUSPROP_RESOURCE_CLASS_INFO;

typedef union CLUSPROP_REQUIRED_DEPENDENCY {
    CLUSPROP_VALUE Value;
    CLUSPROP_RESOURCE_CLASS ResClass;
    CLUSPROP_SZ ResTypeName;
} CLUSPROP_REQUIRED_DEPENDENCY, *PCLUSPROP_REQUIRED_DEPENDENCY;

typedef CLUSPROP_DWORD CLUSPROP_DISK_NUMBER, *PCLUSPROP_DISK_NUMBER;

typedef enum CLUSPROP_PIFLAGS {
    CLUSPROP_PIFLAG_STICKY = 0x00000001,
    CLUSPROP_PIFLAG_REMOVABLE = 0x00000002,
    CLUSPROP_PIFLAG_USABLE = 0x00000004,
    CLUSPROP_PIFLAG_DEFAULT_QUORUM = 0x00000008,
    CLUSPROP_PIFLAG_USABLE_FOR_CSV = 0x00000010,
    CLUSPROP_PIFLAG_ENCRYPTION_ENABLED = 0x00000020,
    CLUSPROP_PIFLAG_UNKNOWN = 0x80000000
} CLUSPROP_PIFLAGS;

typedef struct CLUS_FORCE_QUORUM_INFO {
    DWORD dwSize;
    DWORD dwNodeBitMask;
    DWORD dwMaxNumberofNodes;
    WCHAR multiszNodeList[1];
} CLUS_FORCE_QUORUM_INFO, *PCLUS_FORCE_QUORUM_INFO;

typedef struct CLUS_PARTITION_INFO {
    DWORD dwFlags;
    WCHAR szDeviceName[MAX_PATH];
    WCHAR szVolumeLabel[MAX_PATH];
    DWORD dwSerialNumber;
    DWORD rgdwMaximumComponentLength;
    DWORD dwFileSystemFlags;
    WCHAR szFileSystem[32];
} CLUS_PARTITION_INFO, *PCLUS_PARTITION_INFO;

typedef struct CLUS_PARTITION_INFO_EX {
    DWORD dwFlags;
    WCHAR szDeviceName[MAX_PATH];
    WCHAR szVolumeLabel[MAX_PATH];
    DWORD dwSerialNumber;
    DWORD rgdwMaximumComponentLength;
    DWORD dwFileSystemFlags;
    WCHAR szFileSystem[32];
    ULARGE_INTEGER TotalSizeInBytes;
    ULARGE_INTEGER FreeSizeInBytes;
    DWORD DeviceNumber;
    DWORD PartitionNumber;
    GUID VolumeGuid;
} CLUS_PARTITION_INFO_EX, *PCLUS_PARTITION_INFO_EX;

typedef struct CLUS_PARTITION_INFO_EX2 {
    GUID GptPartitionId;
    WCHAR szPartitionName[MAX_PATH];
    DWORD EncryptionFlags;
} CLUS_PARTITION_INFO_EX2, *PCLUS_PARTITION_INFO_EX2;

#define BitLockerEnabled     (0x00000001L)
#define BitLockerDecrypted   (0x00000004L)
#define BitlockerEncrypted   (0x00000008L)
#define BitLockerDecrypting  (0x00000010L)
#define BitlockerEncrypting  (0x00000020L)
#define BitLockerPaused      (0x00000040L)
#define BitLockerStopped     (0x00000080L)
#define BitLockerFlagsAll    (BitLockerEnabled|BitLockerDecrypted|BitlockerEncrypted|BitLockerDecrypting|BitlockerEncrypting|BitLockerPaused|BitLockerStopped)

typedef enum _CLUSTER_CSV_VOLUME_FAULT_STATE {
    VolumeStateNoFaults = 0x00000000,
    VolumeStateNoDirectIO = 0x00000001,
    VolumeStateNoAccess = 0x00000002,
    VolumeStateInMaintenance = 0x00000004,
    VolumeStateDismounted = 0x00000008
} CLUSTER_CSV_VOLUME_FAULT_STATE, *PCLUSTER_CSV_VOLUME_FAULT_STATE;

typedef enum _CLUSTER_SHARED_VOLUME_BACKUP_STATE {
    VolumeBackupNone = 0x00000000,
    VolumeBackupInProgress = 0x00000001
} CLUSTER_SHARED_VOLUME_BACKUP_STATE, *PCLUSTER_SHARED_VOLUME_BACKUP_STATE;

typedef struct _CLUS_CSV_VOLUME_INFO {
    ULARGE_INTEGER VolumeOffset;
    DWORD PartitionNumber;
    CLUSTER_CSV_VOLUME_FAULT_STATE FaultState;
    CLUSTER_SHARED_VOLUME_BACKUP_STATE BackupState;
    WCHAR szVolumeFriendlyName[MAX_PATH];
    WCHAR szVolumeName[50];
} CLUS_CSV_VOLUME_INFO, *PCLUS_CSV_VOLUME_INFO;

typedef struct _CLUS_CSV_VOLUME_NAME {
    LARGE_INTEGER VolumeOffset;
    WCHAR szVolumeName[MAX_PATH];
    WCHAR szRootPath[MAX_PATH+3];
}
CLUS_CSV_VOLUME_NAME, *PCLUS_CSV_VOLUME_NAME;

typedef enum _CLUSTER_SHARED_VOLUME_STATE {
    SharedVolumeStateUnavailable = 0,
    SharedVolumeStatePaused = 1,
    SharedVolumeStateActive = 2,
    SharedVolumeStateActiveRedirected = 3,
    SharedVolumeStateActiveVolumeRedirected = 4
} CLUSTER_SHARED_VOLUME_STATE, *PCLUSTER_SHARED_VOLUME_STATE;

typedef struct _CLUSTER_SHARED_VOLUME_STATE_INFO {
    WCHAR szVolumeName[MAX_PATH];
    WCHAR szNodeName[MAX_PATH];
    CLUSTER_SHARED_VOLUME_STATE VolumeState;
} CLUSTER_SHARED_VOLUME_STATE_INFO, *PCLUSTER_SHARED_VOLUME_STATE_INFO;

#define RedirectedIOReasonUserRequest  0x0000000000000001
#define RedirectedIOReasonUnsafeFileSystemFilter  0x0000000000000002
#define RedirectedIOReasonUnsafeVolumeFilter  0x0000000000000004
#define RedirectedIOReasonFileSystemTiering  0x0000000000000008
#define RedirectedIOReasonBitLockerInitializing  0x0000000000000010
#define RedirectedIOReasonReFs  0x0000000000000020
#define RedirectedIOReasonMax  0x8000000000000000

#define VolumeRedirectedIOReasonNoDiskConnectivity  0x0000000000000001
#define VolumeRedirectedIOReasonStorageSpaceNotAttached  0x0000000000000002
#define VolumeRedirectedIOReasonVolumeReplicationEnabled  0x0000000000000004
#define VolumeRedirectedIOReasonMax  0x8000000000000000

typedef struct _CLUSTER_SHARED_VOLUME_STATE_INFO_EX {
    WCHAR szVolumeName[MAX_PATH];
    WCHAR szNodeName[MAX_PATH];
    CLUSTER_SHARED_VOLUME_STATE VolumeState;
    WCHAR szVolumeFriendlyName[MAX_PATH];
    ULONGLONG RedirectedIOReason;
    ULONGLONG VolumeRedirectedIOReason;
} CLUSTER_SHARED_VOLUME_STATE_INFO_EX, *PCLUSTER_SHARED_VOLUME_STATE_INFO_EX;

typedef struct _CLUS_CHKDSK_INFO {
    DWORD PartitionNumber;
    DWORD ChkdskState;
    DWORD FileIdCount;
    ULONGLONG FileIdList[1];
} CLUS_CHKDSK_INFO, *PCLUS_CHKDSK_INFO;

typedef struct _CLUS_DISK_NUMBER_INFO {
    DWORD DiskNumber;
    DWORD BytesPerSector;
} CLUS_DISK_NUMBER_INFO, *PCLUS_DISK_NUMBER_INFO;

typedef struct _CLUS_SHARED_VOLUME_BACKUP_MODE {
    CLUSTER_SHARED_VOLUME_BACKUP_STATE BackupState;
    DWORD DelayTimerInSecs;
    WCHAR VolumeName[MAX_PATH];
} CLUS_SHARED_VOLUME_BACKUP_MODE, *PCLUS_SHARED_VOLUME_BACKUP_MODE;

#ifdef __cplusplus
typedef struct CLUSPROP_PARTITION_INFO:public CLUSPROP_VALUE, public CLUS_PARTITION_INFO {
#else
typedef struct CLUSPROP_PARTITION_INFO {
    CLUSPROP_VALUE;
    CLUS_PARTITION_INFO;
#endif
} CLUSPROP_PARTITION_INFO, *PCLUSPROP_PARTITION_INFO;

#ifdef __cplusplus
typedef struct CLUSPROP_PARTITION_INFO_EX:public CLUSPROP_VALUE, public CLUS_PARTITION_INFO_EX {
#else
typedef struct CLUSPROP_PARTITION_INFO_EX {
    CLUSPROP_VALUE;
    CLUS_PARTITION_INFO_EX;
#endif
} CLUSPROP_PARTITION_INFO_EX;

typedef CLUSPROP_PARTITION_INFO_EX UNALIGNED *PCLUSPROP_PARTITION_INFO_EX;

#ifdef __cplusplus
typedef struct CLUSPROP_PARTITION_INFO_EX2:public CLUSPROP_PARTITION_INFO_EX, public CLUS_PARTITION_INFO_EX2 {
#else
typedef struct CLUSPROP_PARTITION_INFO_EX2 {
    CLUSPROP_PARTITION_INFO_EX;
    CLUS_PARTITION_INFO_EX2;
#endif
} CLUSPROP_PARTITION_INFO_EX2;

typedef CLUSPROP_PARTITION_INFO_EX2 UNALIGNED *PCLUSPROP_PARTITION_INFO_EX2;

typedef struct CLUS_FTSET_INFO {
    DWORD dwRootSignature;
    DWORD dwFtType;
} CLUS_FTSET_INFO, *PCLUS_FTSET_INFO;

#ifdef __cplusplus
typedef struct CLUSPROP_FTSET_INFO:public CLUSPROP_VALUE, public CLUS_FTSET_INFO {
#else
typedef struct CLUSPROP_FTSET_INFO {
    CLUSPROP_VALUE;
    CLUS_FTSET_INFO;
#endif
} CLUSPROP_FTSET_INFO, *PCLUSPROP_FTSET_INFO;

typedef CLUSPROP_DWORD CLUSPROP_DISK_SIGNATURE, *PCLUSPROP_DISK_SIGNATURE;

typedef struct CLUS_SCSI_ADDRESS {
    union {
        struct {
            UCHAR PortNumber;
            UCHAR PathId;
            UCHAR TargetId;
            UCHAR Lun;
        } DUMMYSTRUCTNAME;
        DWORD dw;
    } DUMMYUNIONNAME;
} CLUS_SCSI_ADDRESS, *PCLUS_SCSI_ADDRESS;

#ifdef __cplusplus
typedef struct CLUSPROP_SCSI_ADDRESS:public CLUSPROP_VALUE, public CLUS_SCSI_ADDRESS {
#else
typedef struct CLUSPROP_SCSI_ADDRESS {
    CLUSPROP_VALUE;
    CLUS_SCSI_ADDRESS;
#endif
} CLUSPROP_SCSI_ADDRESS, *PCLUSPROP_SCSI_ADDRESS;

typedef struct CLUS_NETNAME_VS_TOKEN_INFO {
    DWORD ProcessID;
    DWORD DesiredAccess;
    BOOL  InheritHandle;
} CLUS_NETNAME_VS_TOKEN_INFO, *PCLUS_NETNAME_VS_TOKEN_INFO;

#define MAX_OBJECTID 64
#define MAX_CO_PASSWORD_LENGTH 16
#define GUID_PRESENT 0x1
#define CREATEDC_PRESENT 0x2

#define MAX_CREATINGDC_LENGTH 256

typedef struct CLUS_NETNAME_PWD_INFO {
    DWORD Flags;
    WCHAR Password[MAX_CO_PASSWORD_LENGTH];
    WCHAR CreatingDC[MAX_CREATINGDC_LENGTH+2];
    WCHAR ObjectGuid[MAX_OBJECTID];
} CLUS_NETNAME_PWD_INFO, *PCLUS_NETNAME_PWD_INFO;

typedef struct CLUS_DNN_LEADER_STATUS {
    BOOL IsOnline;
    BOOL IsFileServerPresent;
} CLUS_DNN_LEADER_STATUS, *PCLUS_DNN_LEADER_STATUS;

typedef struct CLUS_DNN_SODAFS_CLONE_STATUS {
    DWORD NodeId;
    CLUSTER_RESOURCE_STATE Status;
} CLUS_DNN_SODAFS_CLONE_STATUS, *PCLUS_DNN_SODAFS_CLONE_STATUS;

typedef struct CLUS_NETNAME_IP_INFO_ENTRY {
    DWORD NodeId;
    DWORD AddressSize;
    BYTE Address[ANYSIZE_ARRAY];
} CLUS_NETNAME_IP_INFO_ENTRY, *PCLUS_NETNAME_IP_INFO_ENTRY;

#define DNS_LENGTH 64
typedef struct CLUS_NETNAME_IP_INFO_FOR_MULTICHANNEL {
    WCHAR  szName[DNS_LENGTH];
    DWORD NumEntries;
    CLUS_NETNAME_IP_INFO_ENTRY IpInfo[ANYSIZE_ARRAY];
} CLUS_NETNAME_IP_INFO_FOR_MULTICHANNEL, *PCLUS_NETNAME_IP_INFO_FOR_MULTICHANNEL;

typedef struct CLUS_MAINTENANCE_MODE_INFO {
    BOOL  InMaintenance;
} CLUS_MAINTENANCE_MODE_INFO, *PCLUS_MAINTENANCE_MODE_INFO;

typedef struct CLUS_CSV_MAINTENANCE_MODE_INFO {
    BOOL InMaintenance;
    WCHAR VolumeName[MAX_PATH];
} CLUS_CSV_MAINTENANCE_MODE_INFO, *PCLUS_CSV_MAINTENANCE_MODE_INFO;

#define MAINTENANCE_MODE_V2_SIG  0xabbaf00f

typedef enum _MAINTENANCE_MODE_TYPE_ENUM {
    MaintenanceModeTypeDisableIsAliveCheck = 1,
    MaintenanceModeTypeOfflineResource = 2,
    MaintenanceModeTypeUnclusterResource = 3,
} MAINTENANCE_MODE_TYPE_ENUM, *PMAINTENANCE_MODE_TYPE_ENUM;

typedef struct _CLUS_MAINTENANCE_MODE_INFOEX {
    BOOL InMaintenance;
    MAINTENANCE_MODE_TYPE_ENUM MaintainenceModeType;
    CLUSTER_RESOURCE_STATE InternalState;
    DWORD Signature;
} CLUS_MAINTENANCE_MODE_INFOEX, *PCLUS_MAINTENANCE_MODE_INFOEX;

typedef struct _CLUS_STORAGE_SET_DRIVELETTER {
    DWORD PartitionNumber;
    DWORD DriveLetterMask;
} CLUS_STORAGE_SET_DRIVELETTER, *PCLUS_STORAGE_SET_DRIVELETTER;

typedef struct _CLUS_STORAGE_GET_AVAILABLE_DRIVELETTERS {
    DWORD AvailDrivelettersMask;
} CLUS_STORAGE_GET_AVAILABLE_DRIVELETTERS, *PCLUS_STORAGE_GET_AVAILABLE_DRIVELETTERS;

typedef struct _CLUS_STORAGE_REMAP_DRIVELETTER {
    DWORD CurrentDriveLetterMask;
    DWORD TargetDriveLetterMask;
} CLUS_STORAGE_REMAP_DRIVELETTER, *PCLUS_STORAGE_REMAP_DRIVELETTER;

typedef struct _CLUS_PROVIDER_STATE_CHANGE_INFO {
    DWORD dwSize;
    CLUSTER_RESOURCE_STATE resourceState;
    WCHAR szProviderId[1];
} CLUS_PROVIDER_STATE_CHANGE_INFO, *PCLUS_PROVIDER_STATE_CHANGE_INFO;

typedef struct CLUSPROP_LIST {
    DWORD nPropertyCount;
    CLUSPROP_PROPERTY_NAME PropertyName;
} CLUSPROP_LIST, *PCLUSPROP_LIST;

typedef enum CLUSPROP_IPADDR_ENABLENETBIOS {
    CLUSPROP_IPADDR_ENABLENETBIOS_DISABLED = 0,
    CLUSPROP_IPADDR_ENABLENETBIOS_ENABLED,
    CLUSPROP_IPADDR_ENABLENETBIOS_TRACK_NIC
} CLUSPROP_IPADDR_ENABLENETBIOS;

typedef enum _FILESHARE_CHANGE_ENUM {
    FILESHARE_CHANGE_NONE,
    FILESHARE_CHANGE_ADD,
    FILESHARE_CHANGE_DEL,
    FILESHARE_CHANGE_MODIFY
} FILESHARE_CHANGE_ENUM, *PFILESHARE_CHANGE_ENUM;

#define NNLEN  80
typedef struct _FILESHARE_CHANGE {
    FILESHARE_CHANGE_ENUM Change;
    WCHAR ShareName[NNLEN+4];
} FILESHARE_CHANGE, *PFILESHARE_CHANGE;

typedef struct _FILESHARE_CHANGE_LIST {
    DWORD NumEntries;
    FILESHARE_CHANGE ChangeEntry[0];
} FILESHARE_CHANGE_LIST, *PFILESHARE_CHANGE_LIST;

typedef struct _CLUSCTL_GROUP_GET_LAST_MOVE_TIME_OUTPUT {
    ULONGLONG GetTickCount64;
    SYSTEMTIME GetSystemTime;
    DWORD NodeId;
} CLUSCTL_GROUP_GET_LAST_MOVE_TIME_OUTPUT, *PCLUSCTL_GROUP_GET_LAST_MOVE_TIME_OUTPUT;

typedef union CLUSPROP_BUFFER_HELPER {
    BYTE *pb;
    WORD *pw;
    DWORD *pdw;
    LONG *pl;
    LPWSTR psz;
    PCLUSPROP_LIST pList;
    PCLUSPROP_SYNTAX pSyntax;
    PCLUSPROP_PROPERTY_NAME pName;
    PCLUSPROP_VALUE pValue;
    PCLUSPROP_BINARY pBinaryValue;
    PCLUSPROP_WORD pWordValue;
    PCLUSPROP_DWORD pDwordValue;
    PCLUSPROP_LONG pLongValue;
    PCLUSPROP_ULARGE_INTEGER pULargeIntegerValue;
    PCLUSPROP_LARGE_INTEGER pLargeIntegerValue;
    PCLUSPROP_SZ pStringValue;
    PCLUSPROP_MULTI_SZ pMultiSzValue;
    PCLUSPROP_SECURITY_DESCRIPTOR pSecurityDescriptor;
    PCLUSPROP_RESOURCE_CLASS pResourceClassValue;
    PCLUSPROP_RESOURCE_CLASS_INFO pResourceClassInfoValue;
    PCLUSPROP_DISK_SIGNATURE pDiskSignatureValue;
    PCLUSPROP_SCSI_ADDRESS pScsiAddressValue;
    PCLUSPROP_DISK_NUMBER pDiskNumberValue;
    PCLUSPROP_PARTITION_INFO pPartitionInfoValue;
    PCLUSPROP_REQUIRED_DEPENDENCY pRequiredDependencyValue;
    PCLUSPROP_PARTITION_INFO_EX pPartitionInfoValueEx;
    PCLUSPROP_FILETIME pFileTimeValue;
} CLUSPROP_BUFFER_HELPER, *PCLUSPROP_BUFFER_HELPER;

typedef enum CLUSTER_RESOURCE_ENUM {
    CLUSTER_RESOURCE_ENUM_DEPENDS = 0x00000001,
    CLUSTER_RESOURCE_ENUM_PROVIDES = 0x00000002,
    CLUSTER_RESOURCE_ENUM_NODES = 0x00000004,
    CLUSTER_RESOURCE_ENUM_ALL = (CLUSTER_RESOURCE_ENUM_DEPENDS|CLUSTER_RESOURCE_ENUM_PROVIDES|CLUSTER_RESOURCE_ENUM_NODES)
} CLUSTER_RESOURCE_ENUM;

typedef enum CLUSTER_RESOURCE_TYPE_ENUM {
    CLUSTER_RESOURCE_TYPE_ENUM_NODES = 0x00000001,
    CLUSTER_RESOURCE_TYPE_ENUM_RESOURCES = 0x00000002,
    CLUSTER_RESOURCE_TYPE_ENUM_ALL = CLUSTER_RESOURCE_TYPE_ENUM_NODES
} CLUSTER_RESOURCE_TYPE_ENUM;

typedef enum CLUSTER_NETWORK_ENUM {
    CLUSTER_NETWORK_ENUM_NETINTERFACES = 0x00000001,
    CLUSTER_NETWORK_ENUM_ALL = CLUSTER_NETWORK_ENUM_NETINTERFACES
} CLUSTER_NETWORK_ENUM;

typedef enum CLUSTER_NETWORK_STATE {
    ClusterNetworkStateUnknown = -1,
    ClusterNetworkUnavailable,
    ClusterNetworkDown,
    ClusterNetworkPartitioned,
    ClusterNetworkUp
} CLUSTER_NETWORK_STATE;

typedef enum CLUSTER_NETWORK_ROLE {
    ClusterNetworkRoleNone = 0,
    ClusterNetworkRoleInternalUse = 0x00000001,
    ClusterNetworkRoleClientAccess = 0x00000002,
    ClusterNetworkRoleInternalAndClient = 0x00000003
} CLUSTER_NETWORK_ROLE;

typedef enum CLUSTER_NETINTERFACE_STATE {
    ClusterNetInterfaceStateUnknown = -1,
    ClusterNetInterfaceUnavailable,
    ClusterNetInterfaceFailed,
    ClusterNetInterfaceUnreachable,
    ClusterNetInterfaceUp
} CLUSTER_NETINTERFACE_STATE;

#endif /* !_CLUSTER_API_TYPES_ */

typedef enum _CLUSTER_SETUP_PHASE {
    ClusterSetupPhaseInitialize = 1,
    ClusterSetupPhaseValidateNodeState = 100,
    ClusterSetupPhaseValidateNetft = 102,
    ClusterSetupPhaseValidateClusDisk = 103,
    ClusterSetupPhaseConfigureClusSvc = 104,
    ClusterSetupPhaseStartingClusSvc = 105,
    ClusterSetupPhaseQueryClusterNameAccount = 106,
    ClusterSetupPhaseValidateClusterNameAccount = 107,
    ClusterSetupPhaseCreateClusterAccount = 108,
    ClusterSetupPhaseConfigureClusterAccount = 109,
    ClusterSetupPhaseFormingCluster = 200,
    ClusterSetupPhaseAddClusterProperties = 201,
    ClusterSetupPhaseCreateResourceTypes = 202,
    ClusterSetupPhaseCreateGroups = 203,
    ClusterSetupPhaseCreateIPAddressResources = 204,
    ClusterSetupPhaseCreateNetworkName = 205,
    ClusterSetupPhaseClusterGroupOnline = 206,
    ClusterSetupPhaseGettingCurrentMembership = 300,
    ClusterSetupPhaseAddNodeToCluster = 301,
    ClusterSetupPhaseNodeUp = 302,
    ClusterSetupPhaseMoveGroup = 400,
    ClusterSetupPhaseDeleteGroup = 401,
    ClusterSetupPhaseCleanupCOs = 402,
    ClusterSetupPhaseOfflineGroup = 403,
    ClusterSetupPhaseEvictNode = 404,
    ClusterSetupPhaseCleanupNode = 405,
    ClusterSetupPhaseCoreGroupCleanup = 406,
    ClusterSetupPhaseFailureCleanup = 999
} CLUSTER_SETUP_PHASE;

typedef enum _CLUSTER_SETUP_PHASE_TYPE {
    ClusterSetupPhaseStart = 1,
    ClusterSetupPhaseContinue = 2,
    ClusterSetupPhaseEnd = 3,
    ClusterSetupPhaseReport = 4,
} CLUSTER_SETUP_PHASE_TYPE;

typedef enum _CLUSTER_SETUP_PHASE_SEVERITY {
    ClusterSetupPhaseInformational = 1,
    ClusterSetupPhaseWarning = 2,
    ClusterSetupPhaseFatal = 3,
} CLUSTER_SETUP_PHASE_SEVERITY;

#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8)
typedef enum CLUSTER_NODE_RESUME_FAILBACK_TYPE {  /* (c) Dick-is-a-moron */
    DoNotFailbackGroups = 0,
    FailbackGroupsImmediately,
    FailbackGroupsPerPolicy,
    ClusterNodeResumeFailbackTypeCount
} CLUSTER_NODE_RESUME_FAILBACK_TYPE;
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8 */

typedef enum PLACEMENT_OPTIONS {
    PLACEMENT_OPTIONS_MIN_VALUE = 0x00000000,
    PLACEMENT_OPTIONS_DEFAULT_PLACEMENT_OPTIONS = PLACEMENT_OPTIONS_MIN_VALUE,
    PLACEMENT_OPTIONS_DISABLE_CSV_VM_DEPENDENCY = 0x00000001,
    PLACEMENT_OPTIONS_CONSIDER_OFFLINE_VMS = 0x00000002,
    PLACEMENT_OPTIONS_DONT_USE_MEMORY = 0x00000004,
    PLACEMENT_OPTIONS_DONT_USE_CPU = 0x00000008,
    PLACEMENT_OPTIONS_ALL = (PLACEMENT_OPTIONS_DISABLE_CSV_VM_DEPENDENCY | PLACEMENT_OPTIONS_CONSIDER_OFFLINE_VMS |
                             PLACEMENT_OPTIONS_DONT_USE_MEMORY | PLACEMENT_OPTIONS_DONT_USE_CPU)
} PLACEMENT_OPTIONS;

#define SR_REPLICATED_PARTITION_DISALLOW_MULTINODE_IO  0x00000001

typedef struct _SR_RESOURCE_TYPE_REPLICATED_PARTITION_INFO {
    ULONGLONG PartitionOffset;
    ULONG Capabilities;
} SR_RESOURCE_TYPE_REPLICATED_PARTITION_INFO, *PSR_RESOURCE_TYPE_REPLICATED_PARTITION_INFO;

typedef struct _SR_RESOURCE_TYPE_REPLICATED_PARTITION_ARRAY {
    DWORD Count;
    SR_RESOURCE_TYPE_REPLICATED_PARTITION_INFO PartitionArray[1];
} SR_RESOURCE_TYPE_REPLICATED_PARTITION_ARRAY, *PSR_RESOURCE_TYPE_REPLICATED_PARTITION_ARRAY;

typedef enum _SR_REPLICATED_DISK_TYPE {
    SrReplicatedDiskTypeNone = 0,
    SrReplicatedDiskTypeSource = 1,
    SrReplicatedDiskTypeLogSource = 2,
    SrReplicatedDiskTypeDestination = 3,
    SrReplicatedDiskTypeLogDestination = 4,
    SrReplicatedDiskTypeNotInParthership = 5,
    SrReplicatedDiskTypeLogNotInParthership = 6,
    SrReplicatedDiskTypeOther
} SR_REPLICATED_DISK_TYPE, *PSR_REPLICATED_DISK_TYPE;

typedef enum _SR_DISK_REPLICATION_ELIGIBLE {
    SrDiskReplicationEligibleNone = 0,
    SrDiskReplicationEligibleYes = 1,
    SrDiskReplicationEligibleOffline = 2,
    SrDiskReplicationEligibleNotGpt = 3,
    SrDiskReplicationEligiblePartitionLayoutMismatch = 4,
    SrDiskReplicationEligibleInsufficientFreeSpace = 5,
    SrDiskReplicationEligibleNotInSameSite = 6,
    SrDiskReplicationEligibleInSameSite = 7,
    SrDiskReplicationEligibleFileSystemNotSupported = 8,
    SrDiskReplicationEligibleAlreadyInReplication = 9,
    SrDiskReplicationEligibleSameAsSpecifiedDisk = 10,
    SrDiskReplicationEligibleOther = 9999
} SR_DISK_REPLICATION_ELIGIBLE, *PSR_DISK_REPLICATION_ELIGIBLE;

typedef struct _SR_RESOURCE_TYPE_QUERY_ELIGIBLE_LOGDISKS {
    GUID DataDiskGuid;
    BOOLEAN IncludeOfflineDisks;
} SR_RESOURCE_TYPE_QUERY_ELIGIBLE_LOGDISKS, *PSR_RESOURCE_TYPE_QUERY_ELIGIBLE_LOGDISKS;

typedef struct _SR_RESOURCE_TYPE_ELIGIBLE_TARGET_DATADISKS {
    GUID SourceDataDiskGuid;
    GUID TargetReplicationGroupGuid;
    BOOLEAN SkipConnectivityCheck;
    BOOLEAN IncludeOfflineDisks;
} SR_RESOURCE_TYPE_QUERY_ELIGIBLE_TARGET_DATADISKS, *PSR_RESOURCE_TYPE_QUERY_ELIGIBLE_TARGET_DATADISKS;

typedef struct _SR_RESOURCE_TYPE_ELIGIBLE_SOURCE_DATADISKS {
    GUID DataDiskGuid;
    BOOLEAN IncludeAvailableStoargeDisks;
} SR_RESOURCE_TYPE_QUERY_ELIGIBLE_SOURCE_DATADISKS, *PSR_RESOURCE_TYPE_QUERY_ELIGIBLE_SOURCE_DATADISKS;

typedef struct _SR_RESOURCE_TYPE_DISK_INFO {
    SR_DISK_REPLICATION_ELIGIBLE Reason;
    GUID DiskGuid;
} SR_RESOURCE_TYPE_DISK_INFO, *PSR_RESOURCE_TYPE_DISK_INFO;

typedef struct _SR_RESOURCE_TYPE_ELIGIBLE_DISKS_RESULT {
    USHORT Count;
    SR_RESOURCE_TYPE_DISK_INFO DiskInfo[1];
} SR_RESOURCE_TYPE_ELIGIBLE_DISKS_RESULT, *PSR_RESOURCE_TYPE_ELIGIBLE_DISKS_RESULT;

typedef struct _SR_RESOURCE_TYPE_REPLICATED_DISK {
    SR_REPLICATED_DISK_TYPE Type;
    GUID ClusterDiskResourceGuid;
    GUID ReplicationGroupId;
    WCHAR ReplicationGroupName[MAX_PATH];
} SR_RESOURCE_TYPE_REPLICATED_DISK, *PSR_RESOURCE_TYPE_REPLICATED_DISK;

typedef struct _SR_RESOURCE_TYPE_REPLICATED_DISKS_RESULT {
    USHORT Count;
    SR_RESOURCE_TYPE_REPLICATED_DISK ReplicatedDisks[1];
} SR_RESOURCE_TYPE_REPLICATED_DISKS_RESULT, *PSR_RESOURCE_TYPE_REPLICATED_DISKS_RESULT;

typedef struct _CLUSCTL_RESOURCE_TYPE_STORAGE_GET_AVAILABLE_DISKS_EX2_INPUT {
    DWORD dwFlags;
    GUID guidPoolFilter;
} CLUSCTL_RESOURCE_TYPE_STORAGE_GET_AVAILABLE_DISKS_EX2_INPUT, *PCLUSCTL_RESOURCE_TYPE_STORAGE_GET_AVAILABLE_DISKS_EX2_INPUT;

typedef DWORD (WINAPI *PCLUSAPI_GET_NODE_CLUSTER_STATE)(LPCWSTR,LPDWORD);
typedef HCLUSTER (WINAPI *PCLUSAPI_OPEN_CLUSTER)(LPCWSTR);
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2)
typedef HCLUSTER (WINAPI *PCLUSAPI_OPEN_CLUSTER_EX)(LPCWSTR,DWORD,LPDWORD);
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2 */
typedef BOOL (WINAPI *PCLUSAPI_CLOSE_CLUSTER)(HCLUSTER);
typedef DWORD (WINAPI *PCLUSAPI_SetClusterName)(HCLUSTER,LPCWSTR);
typedef DWORD (WINAPI *PCLUSAPI_GET_CLUSTER_INFORMATION)(HCLUSTER,LPWSTR,LPDWORD,LPCLUSTERVERSIONINFO);
typedef DWORD (WINAPI *PCLUSAPI_GET_CLUSTER_QUORUM_RESOURCE)(HCLUSTER,LPWSTR,LPDWORD,LPWSTR,LPDWORD,LPDWORD);
typedef DWORD (WINAPI *PCLUSAPI_SET_CLUSTER_QUORUM_RESOURCE)(HRESOURCE,LPCWSTR,DWORD);
typedef DWORD (WINAPI *PCLUSAPI_BACKUP_CLUSTER_DATABASE)(HCLUSTER,LPCWSTR);
typedef DWORD (WINAPI *PCLUSAPI_RESTORE_CLUSTER_DATABASE)(LPCWSTR,BOOL,LPCWSTR);
typedef DWORD (WINAPI *PCLUSAPI_SET_CLUSTER_NETWORK_PRIORITY_ORDER)(HCLUSTER,DWORD,HNETWORK[]);
typedef DWORD (WINAPI *PCLUSAPI_SET_CLUSTER_SERVICE_ACCOUNT_PASSWORD)(LPCWSTR,LPCWSTR,DWORD,PCLUSTER_SET_PASSWORD_STATUS,LPDWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_CONTROL)(HCLUSTER,HNODE,DWORD,LPVOID,DWORD,LPVOID,DWORD,LPDWORD);
typedef BOOL (WINAPI *PCLUSTER_UPGRADE_PROGRESS_CALLBACK)(PVOID,CLUSTER_UPGRADE_PHASE);
typedef DWORD (WINAPI * PCLUSAPI_CLUSTER_UPGRADE)(HCLUSTER,BOOL,PCLUSTER_UPGRADE_PROGRESS_CALLBACK,PVOID);
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8)
typedef HCHANGE (WINAPI *PCLUSAPI_CREATE_CLUSTER_NOTIFY_PORT_V2)(HCHANGE,HCLUSTER,NOTIFY_FILTER_AND_TYPE*,DWORD,DWORD_PTR);
typedef DWORD (WINAPI *PCLUSAPI_REGISTER_CLUSTER_NOTIFY_V2)(HCHANGE,NOTIFY_FILTER_AND_TYPE,HANDLE,DWORD_PTR);
typedef DWORD (WINAPI *PCLUSAPI_GET_NOTIFY_EVENT_HANDLE_V2)(HCHANGE,LPHANDLE);
typedef DWORD (WINAPI *PCLUSAPI_GET_CLUSTER_NOTIFY_V2)(HCHANGE,DWORD_PTR*,PNOTIFY_FILTER_AND_TYPE,BYTE*,LPDWORD,LPWSTR,LPDWORD,LPWSTR,LPDWORD,LPWSTR,LPDWORD,LPWSTR,LPDWORD,DWORD /*etc.*/);
#endif // (CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8)
typedef HCHANGE (WINAPI *PCLUSAPI_CREATE_CLUSTER_NOTIFY_PORT)(HCHANGE,HCLUSTER,DWORD,DWORD_PTR);
typedef DWORD (WINAPI *PCLUSAPI_REGISTER_CLUSTER_NOTIFY)(HCHANGE,DWORD,HANDLE,DWORD_PTR);
typedef DWORD (WINAPI *PCLUSAPI_GET_CLUSTER_NOTIFY)(HCHANGE,DWORD_PTR*,LPDWORD,LPWSTR,LPDWORD,DWORD);
typedef BOOL (WINAPI *PCLUSAPI_CLOSE_CLUSTER_NOTIFY_PORT)(HCHANGE);
typedef HCLUSENUM (WINAPI *PCLUSAPI_CLUSTER_OPEN_ENUM)(HCLUSTER,DWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_GET_ENUM_COUNT)(HCLUSENUM);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_ENUM)(HCLUSENUM,DWORD,LPDWORD,LPWSTR,LPDWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_CLOSE_ENUM)(HCLUSENUM);
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2)
typedef HCLUSENUMEX (WINAPI *PCLUSAPI_CLUSTER_OPEN_ENUM_EX)(HCLUSTER,DWORD,PVOID);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_GET_ENUM_COUNT_EX)(HCLUSENUMEX);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_ENUM_EX)(HCLUSENUMEX,DWORD,PCLUSTER_ENUM_ITEM,LPDWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_CLOSE_ENUM_EX)(HCLUSENUMEX);
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2 */
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_WINTHRESHOLD)
typedef HGROUPSET (WINAPI *PCLUSAPI_CREATE_CLUSTER_GROUP_GROUPSET)(HCLUSTER,LPCWSTR);
typedef HGROUPSET (WINAPI *PCLUSAPI_OPEN_CLUSTER_GROUP_GROUPSET)(HCLUSTER,LPCWSTR);
typedef BOOL (WINAPI *PCLUSAPI_CLOSE_CLUSTER_GROUP_GROUPSET)(HGROUPSET);
typedef DWORD (WINAPI *PCLUSAPI_DELETE_CLUSTER_GROUP_GROUPSET)(HGROUPSET);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_ADD_GROUP_TO_GROUP_GROUPSET)(HGROUPSET,HGROUP);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_REMOVE_GROUP_FROM_GROUP_GROUPSET)(HGROUPSET,HGROUP);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_GROUP_GROUPSET_CONTROL)(HGROUPSET,HNODE,DWORD,LPVOID,DWORD,LPVOID,DWORD,LPDWORD);
typedef DWORD (WINAPI *PCLUSAPI_ADD_CLUSTER_GROUP_DEPENDENCY)(HGROUP,HGROUP);
typedef DWORD (WINAPI *PCLUSAPI_SET_GROUP_DEPENDENCY_EXPRESSION)(HGROUP,LPCWSTR);
typedef DWORD (WINAPI *PCLUSAPI_REMOVE_CLUSTER_GROUP_DEPENDENCY)(HGROUP,HGROUP);
typedef DWORD (WINAPI *PCLUSAPI_ADD_CLUSTER_GROUP_GROUPSET_DEPENDENCY)(HGROUPSET,HGROUPSET);
typedef DWORD (WINAPI *PCLUSAPI_SET_CLUSTER_GROUP_GROUPSET_DEPENDENCY_EXPRESSION)(HGROUPSET,LPCWSTR);
typedef DWORD (WINAPI *PCLUSAPI_REMOVE_CLUSTER_GROUP_GROUPSET_DEPENDENCY)(HGROUPSET,HGROUPSET);
typedef DWORD (WINAPI *PCLUSAPI_ADD_CLUSTER_GROUP_TO_GROUP_GROUPSET_DEPENDENCY)(HGROUP,HGROUPSET);
typedef DWORD (WINAPI *PCLUSAPI_REMOVE_CLUSTER_GROUP_TO_GROUP_GROUPSET_DEPENDENCY)(HGROUP,HGROUPSET);
typedef HCLUSTER (WINAPI *PCLUSAPI_GET_CLUSTER_FROM_GROUP_GROUPSET)(HGROUPSET);
typedef DWORD (WINAPI *PCLUSAPI_ADD_CROSS_CLUSTER_GROUP_TO_GROUP_DEPENDENCY)(HGROUP,LPCWSTR,LPCWSTR);
typedef DWORD (WINAPI *PCLUSAPI_ADD_CROSS_CLUSTER_GROUP_TO_GROUP_GROUPSET_DEPENDENCY)(HGROUP,LPCWSTR,LPCWSTR);
typedef DWORD (WINAPI *PCLUSAPI_ADD_CROSS_CLUSTER_GROUP_GROUPSET_TO_GROUP_GROUPSET_DEPENDENCY)(HGROUPSET,LPCWSTR,LPCWSTR);
typedef DWORD (WINAPI *PCLUSAPI_REMOVE_CROSS_CLUSTER_GROUP_TO_GROUP_DEPENDENCY)(HGROUP,LPCWSTR,LPCWSTR);
typedef DWORD (WINAPI *PCLUSAPI_REMOVE_CROSS_CLUSTER_GROUP_TO_GROUP_GROUPSET_DEPENDENCY)(HGROUP,LPCWSTR,LPCWSTR);
typedef DWORD (WINAPI *PCLUSAPI_REMOVE_CROSS_CLUSTER_GROUP_GROUPSET_TO_GROUP_GROUPSET_DEPENDENCY)(HGROUPSET,LPCWSTR,LPCWSTR);
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_WINTHRESHOLD */
typedef HNODE (WINAPI *PCLUSAPI_OPEN_CLUSTER_NODE)(HCLUSTER,LPCWSTR);
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2)
typedef HNODE (WINAPI *PCLUSAPI_OPEN_CLUSTER_NODE_EX)(HCLUSTER,LPCWSTR,DWORD,LPDWORD);
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2 */
typedef BOOL (WINAPI *PCLUSAPI_CLOSE_CLUSTER_NODE)(HNODE);
typedef CLUSTER_NODE_STATE (WINAPI *PCLUSAPI_GET_CLUSTER_NODE_STATE)(HNODE);
typedef DWORD (WINAPI *PCLUSAPI_GET_CLUSTER_NODE_ID)(HNODE,LPWSTR,LPDWORD);

typedef HCLUSTER (WINAPI *PCLUSAPI_GET_CLUSTER_FROM_NODE)(HNODE);
typedef DWORD (WINAPI *PCLUSAPI_PAUSE_CLUSTER_NODE)(HNODE);
typedef DWORD (WINAPI *PCLUSAPI_RESUME_CLUSTER_NODE)(HNODE);
typedef DWORD (WINAPI *PCLUSAPI_EVICT_CLUSTER_NODE)(HNODE);
typedef HNODEENUM (WINAPI *PCLUSAPI_CLUSTER_NODE_OPEN_ENUM)(HNODE,DWORD);
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2)
typedef HNODEENUMEX (WINAPI *PCLUSAPI_CLUSTER_NODE_OPEN_ENUM_EX)(HNODE,DWORD,PVOID);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_NODE_GET_ENUM_COUNT_EX)(HNODEENUMEX);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_NODE_ENUM_EX)(HNODEENUMEX,DWORD,PCLUSTER_ENUM_ITEM,LPDWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_NODE_CLOSE_ENUM_EX)(HNODEENUMEX);
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2 */
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_NODE_GET_ENUM_COUNT)(HNODEENUM);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_NODE_CLOSE_ENUM)(HNODEENUM);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_NODE_ENUM)(HNODEENUM,DWORD,LPDWORD,LPWSTR,LPDWORD);
typedef DWORD (WINAPI *PCLUSAPI_EVICT_CLUSTER_NODE_EX)(HNODE,DWORD,HRESULT*);

typedef HKEY (WINAPI *PCLUSAPI_GET_CLUSTER_RESOURCE_TYPE_KEY)(HCLUSTER, LPCWSTR, REGSAM);
typedef HGROUP (WINAPI *PCLUSAPI_CREATE_CLUSTER_GROUP)(HCLUSTER, LPCWSTR);
typedef HGROUP (WINAPI *PCLUSAPI_OPEN_CLUSTER_GROUP)(HCLUSTER, LPCWSTR);
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2)
typedef HGROUP (WINAPI *PCLUSAPI_OPEN_CLUSTER_GROUP_EX)(HCLUSTER,LPCWSTR,DWORD,LPDWORD);
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2 */
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8)
typedef DWORD (WINAPI *PCLUSAPI_PAUSE_CLUSTER_NODE_EX)(HNODE,BOOL,DWORD,HNODE);
typedef DWORD (WINAPI *PCLUSAPI_RESUME_CLUSTER_NODE_EX)(HNODE,CLUSTER_NODE_RESUME_FAILBACK_TYPE,DWORD);
typedef HGROUP (WINAPI *PCLUSAPI_CREATE_CLUSTER_GROUPEX)(HCLUSTER,LPCWSTR,PCLUSTER_CREATE_GROUP_INFO);
typedef HGROUPENUMEX (WINAPI *PCLUSAPI_CLUSTER_GROUP_OPEN_ENUM_EX)(HCLUSTER,LPCWSTR,DWORD,LPCWSTR,DWORD,DWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_GROUP_GET_ENUM_COUNT_EX)(HGROUPENUMEX);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_GROUP_ENUM_EX)(HGROUPENUMEX,DWORD,PCLUSTER_GROUP_ENUM_ITEM,LPDWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_GROUP_CLOSE_ENUM_EX)(HGROUPENUMEX);
typedef HRESENUMEX (WINAPI *PCLUSAPI_CLUSTER_RESOURCE_OPEN_ENUM_EX)(HCLUSTER,LPCWSTR,DWORD,LPCWSTR,DWORD,DWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_RESOURCE_GET_ENUM_COUNT_EX)(HRESENUMEX);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_RESOURCE_ENUM_EX)(HRESENUMEX,DWORD,PCLUSTER_RESOURCE_ENUM_ITEM,LPDWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_RESOURCE_CLOSE_ENUM_EX)(HRESENUMEX);
typedef DWORD (WINAPI *PCLUSAPI_RESTART_CLUSTER_RESOURCE)(HRESOURCE,DWORD);
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8 */
typedef BOOL (WINAPI *PCLUSAPI_CLOSE_CLUSTER_GROUP)(HGROUP);
typedef HCLUSTER (WINAPI *PCLUSAPI_GET_CLUSTER_FROM_GROUP)(HGROUP);
typedef CLUSTER_GROUP_STATE (WINAPI *PCLUSAPI_GET_CLUSTER_GROUP_STATE)(HGROUP, LPWSTR, LPDWORD);
typedef DWORD (WINAPI *PCLUSAPI_SET_CLUSTER_GROUP_NAME)(HGROUP, LPCWSTR);
typedef DWORD (WINAPI *PCLUSAPI_SET_CLUSTER_GROUP_NODE_LIST)(HGROUP, DWORD, HNODE[]);
typedef DWORD (WINAPI *PCLUSAPI_ONLINE_CLUSTER_GROUP)(HGROUP, HNODE);
typedef DWORD (WINAPI *PCLUSAPI_MOVE_CLUSTER_GROUP)(HGROUP, HNODE);
typedef DWORD (WINAPI *PCLUSAPI_OFFLINE_CLUSTER_GROUP)(HGROUP);
typedef DWORD (WINAPI *PCLUSAPI_DELETE_CLUSTER_GROUP)(HGROUP);
typedef DWORD (WINAPI *PCLUSAPI_DESTROY_CLUSTER_GROUP)(HGROUP);
typedef HGROUPENUM (WINAPI *PCLUSAPI_CLUSTER_GROUP_OPEN_ENUM)(HGROUP, DWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_GROUP_GET_ENUM_COUNT)(HGROUPENUM);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_GROUP_ENUM)(HGROUPENUM, DWORD, LPDWORD, LPWSTR, LPDWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_GROUP_CLOSE_ENUM)(HGROUPENUM);
typedef HRESOURCE (WINAPI *PCLUSAPI_CREATE_CLUSTER_RESOURCE)(HGROUP, LPCWSTR, LPCWSTR, DWORD);
typedef HRESOURCE (WINAPI *PCLUSAPI_OPEN_CLUSTER_RESOURCE)(HCLUSTER, LPCWSTR);
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2)
typedef HRESOURCE (WINAPI *PCLUSAPI_OPEN_CLUSTER_RESOURCE_EX)(HCLUSTER,LPCWSTR,DWORD,LPDWORD);
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2 */
typedef BOOL (WINAPI *PCLUSAPI_CLOSE_CLUSTER_RESOURCE)(HRESOURCE);
typedef HCLUSTER (WINAPI *PCLUSAPI_GET_CLUSTER_FROM_RESOURCE)(HRESOURCE);
typedef DWORD (WINAPI *PCLUSAPI_DELETE_CLUSTER_RESOURCE)(HRESOURCE);
typedef CLUSTER_RESOURCE_STATE (WINAPI *PCLUSAPI_GET_CLUSTER_RESOURCE_STATE)(HRESOURCE, LPWSTR, LPDWORD, LPWSTR, LPDWORD);
typedef DWORD (WINAPI *PCLUSAPI_SET_CLUSTER_RESOURCE_NAME)(HRESOURCE, LPCWSTR);
typedef DWORD (WINAPI *PCLUSAPI_FAIL_CLUSTER_RESOURCE)(HRESOURCE);
typedef DWORD (WINAPI *PCLUSAPI_ONLINE_CLUSTER_RESOURCE)(HRESOURCE);
typedef DWORD (WINAPI *PCLUSAPI_OFFLINE_CLUSTER_RESOURCE)(HRESOURCE);
typedef DWORD (WINAPI *PCLUSAPI_CHANGE_CLUSTER_RESOURCE_GROUP)(HRESOURCE, HGROUP);
typedef DWORD (WINAPI *PCLUSAPI_ADD_CLUSTER_RESOURCE_NODE)(HRESOURCE, HNODE);
typedef DWORD (WINAPI *PCLUSAPI_REMOVE_CLUSTER_RESOURCE_NODE)(HRESOURCE, HNODE);
typedef DWORD (WINAPI *PCLUSAPI_ADD_CLUSTER_RESOURCE_DEPENDENCY)(HRESOURCE, HRESOURCE);
typedef DWORD (WINAPI *PCLUSAPI_REMOVE_CLUSTER_RESOURCE_DEPENDENCY)(HRESOURCE, HRESOURCE);
typedef DWORD (WINAPI *PCLUSAPI_SET_CLUSTER_RESOURCE_DEPENDENCY_EXPRESSION)(HRESOURCE, LPCWSTR);
typedef DWORD (WINAPI *PCLUSAPI_GET_CLUSTER_RESOURCE_DEPENDENCY_EXPRESSION)(HRESOURCE, LPWSTR, LPDWORD);
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2)
typedef DWORD (WINAPI *PCLUSAPI_ADD_RESOURCE_TO_CLUSTER_SHARED_VOLUMES)(HRESOURCE);
typedef DWORD (WINAPI *PCLUSAPI_REMOVE_RESOURCE_FROM_CLUSTER_SHARED_VOLUMES)(HRESOURCE);
typedef DWORD (WINAPI *PCLUSAPI_IS_FILE_ON_CLUSTER_SHARED_VOLUME)(LPCWSTR,PBOOL);
typedef DWORD (WINAPI *PCLUSAPI_SHARED_VOLUME_SET_SNAPSHOT_STATE)(GUID,LPCWSTR,CLUSTER_SHARED_VOLUME_SNAPSHOT_STATE);
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2 */
typedef BOOL (WINAPI *PCLUSAPI_CAN_RESOURCE_BE_DEPENDENT)(HRESOURCE, HRESOURCE);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_RESOURCE_CONTROL)(HRESOURCE, HNODE, DWORD, LPVOID, DWORD, LPVOID, DWORD, LPDWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_RESOURCE_TYPE_CONTROL)(HCLUSTER, LPCWSTR, HNODE, DWORD, LPVOID, DWORD, LPVOID, DWORD, LPDWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_GROUP_CONTROL)(HGROUP, HNODE, DWORD, LPVOID, DWORD, LPVOID, DWORD, LPDWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_NODE_CONTROL)(HNODE, HNODE, DWORD, LPVOID, DWORD, LPVOID, DWORD, LPDWORD);
typedef BOOL (WINAPI *PCLUSAPI_GET_CLUSTER_RESOURCE_NETWORK_NAME)(HRESOURCE, LPWSTR, LPDWORD);
typedef HRESENUM (WINAPI *PCLUSAPI_CLUSTER_RESOURCE_OPEN_ENUM)(HRESOURCE, DWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_RESOURCE_GET_ENUM_COUNT)(HRESENUM);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_RESOURCE_ENUM)(HRESENUM, DWORD, LPDWORD, LPWSTR, LPDWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_RESOURCE_CLOSE_ENUM)(HRESENUM);
typedef DWORD (WINAPI *PCLUSAPI_CREATE_CLUSTER_RESOURCE_TYPE)(HCLUSTER, LPCWSTR, LPCWSTR, LPCWSTR, DWORD, DWORD);
typedef DWORD (WINAPI *PCLUSAPI_DELETE_CLUSTER_RESOURCE_TYPE)(HCLUSTER, LPCWSTR);
typedef HRESTYPEENUM (WINAPI *PCLUSAPI_CLUSTER_RESOURCE_TYPE_OPEN_ENUM)(HCLUSTER, LPCWSTR, DWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_RESOURCE_TYPE_GET_ENUM_COUNT)(HRESTYPEENUM);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_RESOURCE_TYPE_ENUM)(HRESTYPEENUM, DWORD, LPDWORD, LPWSTR, LPDWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_RESOURCE_TYPE_CLOSE_ENUM)(HRESTYPEENUM);
typedef HNETWORK (WINAPI *PCLUSAPI_OPEN_CLUSTER_NETWORK)(HCLUSTER, LPCWSTR);
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2)
typedef HNETWORK (WINAPI *PCLUSAPI_OPEN_CLUSTER_NETWORK_EX)(HCLUSTER,LPCWSTR,DWORD,LPDWORD);
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2 */
typedef BOOL (WINAPI *PCLUSAPI_CLOSE_CLUSTER_NETWORK)(HNETWORK);
typedef HCLUSTER (WINAPI *PCLUSAPI_GET_CLUSTER_FROM_NETWORK)(HNETWORK);
typedef HNETWORKENUM (WINAPI *PCLUSAPI_CLUSTER_NETWORK_OPEN_ENUM)(HNETWORK, DWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_NETWORK_GET_ENUM_COUNT)(HNETWORKENUM);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_NETWORK_ENUM)(HNETWORKENUM, DWORD, LPDWORD, LPWSTR, LPDWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_NETWORK_CLOSE_ENUM)(HNETWORKENUM);
typedef CLUSTER_NETWORK_STATE (WINAPI *PCLUSAPI_GET_CLUSTER_NETWORK_STATE)(HNETWORK);
typedef DWORD (WINAPI *PCLUSAPI_SET_CLUSTER_NETWORK_NAME)(HNETWORK, LPCWSTR);
typedef DWORD (WINAPI *PCLUSAPI_GET_CLUSTER_NETWORK_ID)(HNETWORK, LPWSTR, LPDWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_NETWORK_CONTROL)(HNETWORK, HNODE, DWORD, LPVOID, DWORD, LPVOID, DWORD, LPDWORD);
typedef HNETINTERFACE (WINAPI *PCLUSAPI_OPEN_CLUSTER_NET_INTERFACE)(HCLUSTER, LPCWSTR);
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2)
typedef HNETINTERFACE (WINAPI *PCLUSAPI_OPEN_CLUSTER_NETINTERFACE_EX)(HCLUSTER,LPCWSTR,DWORD,LPDWORD);
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2 */
typedef DWORD (WINAPI *PCLUSAPI_GET_CLUSTER_NET_INTERFACE)(HCLUSTER, LPCWSTR, LPCWSTR, LPWSTR, LPDWORD);
typedef BOOL (WINAPI *PCLUSAPI_CLOSE_CLUSTER_NET_INTERFACE)(HNETINTERFACE);
typedef HCLUSTER (WINAPI *PCLUSAPI_GET_CLUSTER_FROM_NET_INTERFACE)(HNETINTERFACE);
typedef CLUSTER_NETINTERFACE_STATE (WINAPI *PCLUSAPI_GET_CLUSTER_NET_INTERFACE_STATE)(HNETINTERFACE);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_NET_INTERFACE_CONTROL)(HNETINTERFACE, HNODE, DWORD, LPVOID, DWORD, LPVOID, DWORD, LPDWORD);
typedef HKEY (WINAPI *PCLUSAPI_GET_CLUSTER_KEY)(HCLUSTER, REGSAM);
typedef HKEY (WINAPI *PCLUSAPI_GET_CLUSTER_GROUP_KEY)(HGROUP, REGSAM);
typedef HKEY (WINAPI *PCLUSAPI_GET_CLUSTER_RESOURCE_KEY)(HRESOURCE, REGSAM);
typedef HKEY (WINAPI *PCLUSAPI_GET_CLUSTER_NODE_KEY)(HNODE, REGSAM);
typedef HKEY (WINAPI *PCLUSAPI_GET_CLUSTER_NETWORK_KEY)(HNETWORK, REGSAM);
typedef HKEY (WINAPI *PCLUSAPI_GET_CLUSTER_NET_INTERFACE_KEY)(HNETINTERFACE, REGSAM);
typedef LONG (WINAPI *PCLUSAPI_CLUSTER_REG_CREATE_KEY)(HKEY, LPCWSTR, DWORD, REGSAM, LPSECURITY_ATTRIBUTES, PHKEY, LPDWORD);
typedef LONG (WINAPI *PCLUSAPI_CLUSTER_REG_OPEN_KEY)(HKEY, LPCWSTR, REGSAM, PHKEY);
typedef LONG (WINAPI *PCLUSAPI_CLUSTER_REG_DELETE_KEY)(HKEY, LPCWSTR);
typedef LONG (WINAPI *PCLUSAPI_CLUSTER_REG_CLOSE_KEY)(HKEY);
typedef LONG (WINAPI *PCLUSAPI_CLUSTER_REG_ENUM_KEY)(HKEY, DWORD, LPWSTR, LPDWORD, PFILETIME);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_REG_SET_VALUE)(HKEY, LPCWSTR, DWORD, CONST BYTE *, DWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_REG_DELETE_VALUE)(HKEY, LPCWSTR);
typedef LONG (WINAPI *PCLUSAPI_CLUSTER_REG_QUERY_VALUE)(HKEY, LPCWSTR, LPDWORD, LPBYTE, LPDWORD);
typedef DWORD (WINAPI *PCLUSAPI_CLUSTER_REG_ENUM_VALUE)(HKEY, DWORD, LPWSTR, LPDWORD, LPDWORD, LPBYTE, LPDWORD);
typedef LONG (WINAPI *PCLUSAPI_CLUSTER_REG_QUERY_INFO_KEY)(HKEY, LPDWORD, LPDWORD, LPDWORD, LPDWORD, LPDWORD, LPDWORD, PFILETIME);
typedef LONG (WINAPI *PCLUSAPI_CLUSTER_REG_GET_KEY_SECURITY)(HKEY, SECURITY_INFORMATION, PSECURITY_DESCRIPTOR, LPDWORD);
typedef LONG (WINAPI *PCLUSAPI_CLUSTER_REG_SET_KEY_SECURITY)(HKEY, SECURITY_INFORMATION, PSECURITY_DESCRIPTOR);
typedef LONG (WINAPI * PCLUSAPI_CLUSTER_REG_SYNC_DATABASE)(HCLUSTER,DWORD);
typedef LONG (WINAPI *PCLUSAPI_CLUSTER_REG_CREATE_BATCH)(HKEY, HREGBATCH *);
typedef LONG (WINAPI *PCLUSTER_REG_BATCH_ADD_COMMAND)(HREGBATCH, CLUSTER_REG_COMMAND, LPWSTR, DWORD, VOID CONST *, DWORD);
typedef LONG (WINAPI *PCLUSTER_REG_CLOSE_BATCH)(HREGBATCH, BOOL, INT *);
typedef LONG (WINAPI *PCLUSTER_REG_BATCH_READ_COMMAND)(HREGBATCHNOTIFICATION, CLUSTER_BATCH_COMMAND *);
typedef LONG (WINAPI *PCLUSTER_REG_BATCH_CLOSE_NOTIFICATION)(HREGBATCHNOTIFICATION);
typedef LONG (WINAPI *PCLUSTER_REG_CREATE_BATCH_NOTIFY_PORT)(HKEY hKey, HREGBATCHPORT *);
typedef LONG (WINAPI *PCLUSTER_REG_CLOSE_BATCH_NOTIFY_PORT)(HREGBATCHPORT);
typedef LONG (WINAPI *PCLUSTER_REG_GET_BATCH_NOTIFICATION)(HREGBATCHPORT, HREGBATCHNOTIFICATION *);
typedef LONG (WINAPI *PCLUSTER_REG_CREATE_READ_BATCH)(HKEY,HREGREADBATCH*);
typedef LONG (WINAPI *PCLUSTER_REG_READ_BATCH_ADD_COMMAND)(HREGREADBATCH,LPCWSTR,LPCWSTR);
typedef LONG (WINAPI *PCLUSTER_REG_CLOSE_READ_BATCH)(HREGREADBATCH,HREGREADBATCHREPLY*);
typedef LONG (WINAPI *PCLUSTER_REG_CLOSE_READ_BATCH_EX)(HREGREADBATCH,DWORD,HREGREADBATCHREPLY*);
typedef LONG (WINAPI *PCLUSTER_REG_READ_BATCH_REPLY_NEXT_COMMAND)(HREGREADBATCHREPLY,CLUSTER_READ_BATCH_COMMAND*);
typedef LONG (WINAPI *PCLUSTER_REG_CLOSE_READ_BATCH_REPLY)(HREGREADBATCHREPLY);
typedef DWORD (WINAPI *PCLUSTER_SET_ACCOUNT_ACCESS)(HCLUSTER,LPCWSTR,DWORD,DWORD);
typedef BOOL (WINAPI *PCLUSTER_SETUP_PROGRESS_CALLBACK)(PVOID, CLUSTER_SETUP_PHASE, CLUSTER_SETUP_PHASE_TYPE, CLUSTER_SETUP_PHASE_SEVERITY, DWORD, PCWSTR, DWORD);
typedef HCLUSTER (WINAPI *PCLUSAPI_CREATE_CLUSTER)(PCREATE_CLUSTER_CONFIG, PCLUSTER_SETUP_PROGRESS_CALLBACK, PVOID);
typedef HCLUSTER (WINAPI * PCLUSAPI_CREATE_CLUSTER_CNOLESS)(PCREATE_CLUSTER_CONFIG,PCLUSTER_SETUP_PROGRESS_CALLBACK,PVOID);
typedef DWORD (WINAPI *PCLUSAPI_CREATE_CLUSTER_NAME_ACCOUNT)(HCLUSTER,PCREATE_CLUSTER_NAME_ACCOUNT,PCLUSTER_SETUP_PROGRESS_CALLBACK,PVOID);
typedef HNODE (WINAPI *PCLUSAPI_ADD_CLUSTER_NODE)(HCLUSTER, PCWSTR, PCLUSTER_SETUP_PROGRESS_CALLBACK, PVOID);
typedef DWORD (WINAPI *PCLUSAPI_DESTROY_CLUSTER)(HCLUSTER, PCLUSTER_SETUP_PROGRESS_CALLBACK, PVOID, BOOL);

DWORD WINAPI GetNodeClusterState(LPCWSTR,DWORD*);
HCLUSTER WINAPI OpenCluster(LPCWSTR);
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2)
HCLUSTER WINAPI OpenClusterEx(LPCWSTR,DWORD,DWORD*);
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2 */
BOOL WINAPI CloseCluster(HCLUSTER);
DWORD WINAPI SetClusterName(HCLUSTER,LPCWSTR);
DWORD WINAPI GetClusterInformation(HCLUSTER,LPWSTR,LPDWORD,LPCLUSTERVERSIONINFO);
DWORD WINAPI GetClusterQuorumResource(HCLUSTER,LPWSTR,LPDWORD,LPWSTR,LPDWORD,LPDWORD);
DWORD WINAPI SetClusterQuorumResource(HRESOURCE,LPCWSTR,DWORD);
DWORD WINAPI BackupClusterDatabase(HCLUSTER,LPCWSTR);
DWORD WINAPI RestoreClusterDatabase(LPCWSTR,BOOL,LPCWSTR);
DWORD WINAPI SetClusterNetworkPriorityOrder(HCLUSTER,DWORD,HNETWORK[]);
DWORD WINAPI SetClusterServiceAccountPassword(LPCWSTR,LPCWSTR,DWORD,PCLUSTER_SET_PASSWORD_STATUS,LPDWORD);
DWORD WINAPI ClusterControl(HCLUSTER,HNODE,DWORD,LPVOID,DWORD,LPVOID,DWORD,LPDWORD);
DWORD WINAPI ClusterUpgradeFunctionalLevel(HCLUSTER,BOOL,PCLUSTER_UPGRADE_PROGRESS_CALLBACK,PVOID);
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8)
HCHANGE WINAPI CreateClusterNotifyPortV2(HCHANGE,HCLUSTER,NOTIFY_FILTER_AND_TYPE*,DWORD,DWORD_PTR);
DWORD WINAPI RegisterClusterNotifyV2(HCHANGE,NOTIFY_FILTER_AND_TYPE,HANDLE,DWORD_PTR);
DWORD WINAPI GetNotifyEventHandle(HCHANGE,LPHANDLE);
DWORD WINAPI GetClusterNotifyV2(HCHANGE,DWORD_PTR*,PNOTIFY_FILTER_AND_TYPE,BYTE*,LPDWORD,LPWSTR,LPDWORD,LPWSTR,LPDWORD,LPWSTR,LPDWORD,LPWSTR,LPDWORD,DWORD);
#endif // (CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8)
HCHANGE WINAPI CreateClusterNotifyPort(HCHANGE,HCLUSTER,DWORD,DWORD_PTR);
DWORD WINAPI RegisterClusterNotify(HCHANGE,DWORD,HANDLE,DWORD_PTR);
DWORD WINAPI GetClusterNotify(HCHANGE,DWORD_PTR*,LPDWORD,LPWSTR,LPDWORD,DWORD);
BOOL WINAPI CloseClusterNotifyPort(HCHANGE);
HCLUSENUM WINAPI ClusterOpenEnum(HCLUSTER,DWORD);
DWORD WINAPI ClusterGetEnumCount(HCLUSENUM);
DWORD WINAPI ClusterEnum(HCLUSENUM,DWORD,LPDWORD,LPWSTR,LPDWORD);
DWORD WINAPI ClusterCloseEnum(HCLUSENUM);
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2)
HCLUSENUMEX WINAPI ClusterOpenEnumEx(HCLUSTER,DWORD,PVOID);
DWORD WINAPI ClusterGetEnumCountEx(HCLUSENUMEX);
DWORD WINAPI ClusterEnumEx(HCLUSENUMEX,DWORD,PCLUSTER_ENUM_ITEM,LPDWORD);
DWORD WINAPI ClusterCloseEnumEx(HCLUSENUMEX);
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2 */
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_WINTHRESHOLD)
HGROUPSET WINAPI CreateClusterGroupSet(HCLUSTER,LPCWSTR);
HGROUPSET WINAPI OpenClusterGroupSet(HCLUSTER,LPCWSTR);
BOOL WINAPI CloseClusterGroupSet(HGROUPSET);
DWORD WINAPI DeleteClusterGroupSet(HGROUPSET);
DWORD WINAPI ClusterAddGroupToGroupSet(HGROUPSET,HGROUP);
DWORD WINAPI ClusterRemoveGroupFromGroupSet(HGROUP);
DWORD WINAPI ClusterGroupSetControl(HGROUPSET,HNODE,DWORD,LPVOID,DWORD,LPVOID,DWORD,LPDWORD);
DWORD WINAPI AddClusterGroupDependency(HGROUP,HGROUP);
DWORD WINAPI SetGroupDependencyExpression(HGROUP,LPCWSTR);
DWORD WINAPI RemoveClusterGroupDependency(HGROUP,HGROUP);
DWORD WINAPI AddClusterGroupSetDependency(HGROUPSET,HGROUPSET);
DWORD WINAPI SetClusterGroupSetDependencyExpression(HGROUPSET,LPCWSTR);
DWORD WINAPI RemoveClusterGroupSetDependency(HGROUPSET,HGROUPSET);
DWORD WINAPI AddClusterGroupToGroupSetDependency(HGROUP,HGROUPSET);
DWORD WINAPI RemoveClusterGroupToGroupSetDependency(HGROUP,HGROUPSET);
HGROUPSETENUM WINAPI ClusterGroupSetOpenEnum(HCLUSTER);
DWORD WINAPI ClusterGroupSetGetEnumCount(HGROUPSETENUM);
DWORD WINAPI ClusterGroupSetEnum(HGROUPSETENUM,DWORD,LPWSTR,LPDWORD);
DWORD WINAPI ClusterGroupSetCloseEnum(HGROUPSETENUM);
HCLUSTER WINAPI GetClusterFromGroupSet(HGROUPSET);
DWORD WINAPI AddCrossClusterGroupToGroupDependency(HGROUP,LPCWSTR,LPCWSTR);
DWORD WINAPI AddCrossClusterGroupToGroupSetDependency(HGROUP,LPCWSTR,LPCWSTR);
DWORD WINAPI AddCrossClusterGroupSetToGroupSetDependency(HGROUPSET,LPCWSTR,LPCWSTR);
DWORD WINAPI RemoveCrossClusterGroupToGroupDependency(HGROUP,LPCWSTR,LPCWSTR);
DWORD WINAPI RemoveCrossClusterGroupToGroupSetDependency(HGROUP,LPCWSTR,LPCWSTR);
DWORD WINAPI RemoveCrossClusterGroupSetToGroupSetDependency(HGROUPSET,LPCWSTR,LPCWSTR);
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_WINTHRESHOLD */
HNODE WINAPI OpenClusterNode(HCLUSTER,LPCWSTR);
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2)
HNODE WINAPI OpenClusterNodeEx(HCLUSTER,LPCWSTR,DWORD,DWORD*);
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2 */
BOOL WINAPI CloseClusterNode(HNODE);
CLUSTER_NODE_STATE WINAPI GetClusterNodeState(HNODE);
DWORD WINAPI GetClusterNodeId(HNODE,LPWSTR,LPDWORD);
#define GetCurrentClusterNodeId(nodeid,name)  GetClusterNodeId(NULL,(nodeid),(name))
HCLUSTER WINAPI GetClusterFromNode(HNODE);
DWORD WINAPI PauseClusterNode(HNODE);
DWORD WINAPI ResumeClusterNode(HNODE);
DWORD WINAPI EvictClusterNode(HNODE);
HNODEENUM WINAPI ClusterNodeOpenEnum(HNODE,DWORD);
DWORD WINAPI ClusterNetInterfaceEnum(HNETINTERFACEENUM,DWORD,LPWSTR,LPDWORD);
DWORD WINAPI ClusterNetInterfaceCloseEnum(HNETINTERFACEENUM);
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2)
HNODEENUMEX WINAPI ClusterNodeOpenEnumEx(HNODE,DWORD,PVOID);
DWORD WINAPI ClusterNodeGetEnumCountEx(HNODEENUMEX);
DWORD WINAPI ClusterNodeEnumEx(HNODEENUMEX,DWORD,PCLUSTER_ENUM_ITEM,LPDWORD);
DWORD WINAPI ClusterNodeCloseEnumEx(HNODEENUMEX);
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2 */
DWORD WINAPI ClusterNodeGetEnumCount(HNODEENUM);
DWORD WINAPI ClusterNodeCloseEnum(HNODEENUM);
DWORD WINAPI ClusterNodeEnum(HNODEENUM,DWORD,LPDWORD,LPWSTR,LPDWORD);
DWORD WINAPI EvictClusterNodeEx(HNODE,DWORD,HRESULT*);

/* muppilainen */
HKEY WINAPI GetClusterResourceTypeKey(HCLUSTER, LPCWSTR, REGSAM);
HGROUP WINAPI CreateClusterGroup(HCLUSTER, LPCWSTR);
HGROUP WINAPI OpenClusterGroup(HCLUSTER, LPCWSTR);
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2)
HGROUP WINAPI OpenClusterGroupEx(HCLUSTER,LPCWSTR,DWORD,DWORD*);
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2 */
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8)
DWORD WINAPI PauseClusterNodeEx(HNODE,BOOL,DWORD,HNODE);
DWORD WINAPI ResumeClusterNodeEx(HNODE,CLUSTER_NODE_RESUME_FAILBACK_TYPE,DWORD);
HGROUP WINAPI CreateClusterGroupEx(HCLUSTER,LPCWSTR,PCLUSTER_CREATE_GROUP_INFO);
HGROUPENUMEX WINAPI ClusterGroupOpenEnumEx(HCLUSTER,LPCWSTR,DWORD,LPCWSTR,DWORD,DWORD);
DWORD WINAPI ClusterGroupGetEnumCountEx(HGROUPENUMEX);
DWORD WINAPI ClusterGroupEnumEx(HGROUPENUMEX,DWORD,PCLUSTER_GROUP_ENUM_ITEM,LPDWORD);
DWORD WINAPI ClusterGroupCloseEnumEx(HGROUPENUMEX);
HRESENUMEX WINAPI ClusterResourceOpenEnumEx(HCLUSTER,LPCWSTR,DWORD,LPCWSTR,DWORD,DWORD);
DWORD WINAPI ClusterResourceGetEnumCountEx(HRESENUMEX);
DWORD WINAPI ClusterResourceEnumEx(HRESENUMEX,DWORD,PCLUSTER_RESOURCE_ENUM_ITEM,LPDWORD);
DWORD WINAPI ClusterResourceCloseEnumEx(HRESENUMEX);
DWORD WINAPI OnlineClusterGroupEx(HGROUP,HNODE,DWORD,PBYTE,DWORD);
DWORD WINAPI OfflineClusterGroupEx(HGROUP,DWORD,PBYTE,DWORD);
DWORD WINAPI OnlineClusterResourceEx(HRESOURCE,DWORD,PBYTE,DWORD);
DWORD WINAPI OfflineClusterResourceEx(HRESOURCE,DWORD,PBYTE,DWORD);
DWORD WINAPI MoveClusterGroupEx(HGROUP,HNODE,DWORD,PBYTE,DWORD);
DWORD WINAPI CancelClusterGroupOperation(HGROUP,DWORD);
DWORD WINAPI RestartClusterResource(HRESOURCE,DWORD);
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_WINDOWS8 */
BOOL WINAPI CloseClusterGroup(HGROUP);
HCLUSTER WINAPI GetClusterFromGroup(HGROUP);
CLUSTER_GROUP_STATE WINAPI GetClusterGroupState(HGROUP, LPWSTR, LPDWORD);
DWORD WINAPI SetClusterGroupName(HGROUP, LPCWSTR);
DWORD WINAPI SetClusterGroupNodeList(HGROUP, DWORD, HNODE[]);
DWORD WINAPI OnlineClusterGroup(HGROUP, HNODE);
DWORD WINAPI MoveClusterGroup(HGROUP, HNODE);
DWORD WINAPI OfflineClusterGroup(HGROUP);
DWORD WINAPI DeleteClusterGroup(HGROUP);
DWORD WINAPI DestroyClusterGroup(HGROUP);
HGROUPENUM WINAPI ClusterGroupOpenEnum(HGROUP, DWORD);
DWORD WINAPI ClusterGroupGetEnumCount(HGROUPENUM);
DWORD WINAPI ClusterGroupEnum(HGROUPENUM, DWORD, LPDWORD, LPWSTR, LPDWORD);
DWORD WINAPI ClusterGroupCloseEnum(HGROUPENUM);
HRESOURCE WINAPI CreateClusterResource(HGROUP, LPCWSTR, LPCWSTR, DWORD);
HRESOURCE WINAPI OpenClusterResource(HCLUSTER, LPCWSTR);
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2)
HRESOURCE WINAPI OpenClusterResourceEx(HCLUSTER,LPCWSTR,DWORD,DWORD*);
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2 */
BOOL WINAPI CloseClusterResource(HRESOURCE);
HCLUSTER WINAPI GetClusterFromResource(HRESOURCE);
DWORD WINAPI DeleteClusterResource(HRESOURCE);
CLUSTER_RESOURCE_STATE WINAPI GetClusterResourceState(HRESOURCE, LPWSTR, LPDWORD, LPWSTR, LPDWORD);
DWORD WINAPI SetClusterResourceName(HRESOURCE, LPCWSTR);
DWORD WINAPI FailClusterResource(HRESOURCE);
DWORD WINAPI OnlineClusterResource(HRESOURCE);
DWORD WINAPI OfflineClusterResource(HRESOURCE);
DWORD WINAPI ChangeClusterResourceGroup(HRESOURCE, HGROUP);
DWORD WINAPI AddClusterResourceNode(HRESOURCE, HNODE);
DWORD WINAPI RemoveClusterResourceNode(HRESOURCE, HNODE);
DWORD WINAPI AddClusterResourceDependency(HRESOURCE, HRESOURCE);
DWORD WINAPI RemoveClusterResourceDependency(HRESOURCE, HRESOURCE);
DWORD WINAPI SetClusterResourceDependencyExpression(HRESOURCE, LPCWSTR);
DWORD WINAPI GetClusterResourceDependencyExpression(HRESOURCE, LPWSTR, LPDWORD);
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2)
DWORD WINAPI AddResourceToClusterSharedVolumes(HRESOURCE);
DWORD WINAPI RemoveResourceFromClusterSharedVolumes(HRESOURCE);
DWORD WINAPI IsFileOnClusterSharedVolume(LPCWSTR,PBOOL);
DWORD WINAPI ClusterSharedVolumeSetSnapshotState(GUID,LPCWSTR,CLUSTER_SHARED_VOLUME_SNAPSHOT_STATE);
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2 */
BOOL WINAPI CanResourceBeDependent(HRESOURCE, HRESOURCE);
DWORD WINAPI ClusterResourceControl(HRESOURCE, HNODE, DWORD, LPVOID, DWORD, LPVOID, DWORD, LPDWORD);
DWORD WINAPI ClusterResourceControlAsUser(HRESOURCE,HNODE,DWORD,LPVOID,DWORD,LPVOID,DWORD,LPDWORD);
DWORD WINAPI ClusterResourceTypeControl(HCLUSTER, LPCWSTR, HNODE, DWORD, LPVOID, DWORD, LPVOID, DWORD, LPDWORD);
DWORD WINAPI ClusterResourceTypeControlAsUser(HCLUSTER,LPCWSTR,HNODE,DWORD,LPVOID,DWORD,LPVOID,DWORD,LPDWORD);
DWORD WINAPI ClusterGroupControl(HGROUP, HNODE, DWORD, LPVOID, DWORD, LPVOID, DWORD, LPDWORD);
DWORD WINAPI ClusterNodeControl(HNODE, HNODE, DWORD, LPVOID, DWORD, LPVOID, DWORD, LPDWORD);
BOOL WINAPI GetClusterResourceNetworkName(HRESOURCE, LPWSTR, LPDWORD);
HRESENUM WINAPI ClusterResourceOpenEnum(HRESOURCE, DWORD);
DWORD WINAPI ClusterResourceGetEnumCount(HRESENUM);
DWORD WINAPI ClusterResourceEnum(HRESENUM, DWORD, LPDWORD, LPWSTR, LPDWORD);
DWORD WINAPI ClusterResourceCloseEnum(HRESENUM);
DWORD WINAPI CreateClusterResourceType(HCLUSTER, LPCWSTR, LPCWSTR, LPCWSTR, DWORD, DWORD);
DWORD WINAPI DeleteClusterResourceType(HCLUSTER, LPCWSTR);
HRESTYPEENUM WINAPI ClusterResourceTypeOpenEnum(HCLUSTER, LPCWSTR, DWORD);
DWORD WINAPI ClusterResourceTypeGetEnumCount(HRESTYPEENUM);
DWORD WINAPI ClusterResourceTypeEnum(HRESTYPEENUM, DWORD, LPDWORD, LPWSTR, LPDWORD);
DWORD WINAPI ClusterResourceTypeCloseEnum(HRESTYPEENUM);
HNETWORK WINAPI OpenClusterNetwork(HCLUSTER, LPCWSTR);
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2)
HNETWORK WINAPI OpenClusterNetworkEx(HCLUSTER,LPCWSTR,DWORD,DWORD*);
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2 */
BOOL WINAPI CloseClusterNetwork(HNETWORK);
HCLUSTER WINAPI GetClusterFromNetwork(HNETWORK);
HNETWORKENUM WINAPI ClusterNetworkOpenEnum(HNETWORK, DWORD);
DWORD WINAPI ClusterNetworkGetEnumCount(HNETWORKENUM);
DWORD WINAPI ClusterNetworkEnum(HNETWORKENUM, DWORD, LPDWORD, LPWSTR, LPDWORD);
DWORD WINAPI ClusterNetworkCloseEnum(HNETWORKENUM);
CLUSTER_NETWORK_STATE WINAPI GetClusterNetworkState(HNETWORK);
DWORD WINAPI SetClusterNetworkName(HNETWORK, LPCWSTR);
DWORD WINAPI GetClusterNetworkId(HNETWORK, LPWSTR, LPDWORD);
DWORD WINAPI ClusterNetworkControl(HNETWORK, HNODE, DWORD, LPVOID, DWORD, LPVOID, DWORD, LPDWORD);
HNETINTERFACE WINAPI OpenClusterNetInterface(HCLUSTER, LPCWSTR);
#if (CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2)
HNETINTERFACE WINAPI OpenClusterNetInterfaceEx(HCLUSTER,LPCWSTR,DWORD,DWORD*);
#endif /* CLUSAPI_VERSION >= CLUSAPI_VERSION_SERVER2008R2 */
DWORD WINAPI GetClusterNetInterface(HCLUSTER, LPCWSTR, LPCWSTR, LPWSTR, LPDWORD);
BOOL WINAPI CloseClusterNetInterface(HNETINTERFACE);
HCLUSTER WINAPI GetClusterFromNetInterface(HNETINTERFACE);
CLUSTER_NETINTERFACE_STATE WINAPI GetClusterNetInterfaceState(HNETINTERFACE);
DWORD WINAPI ClusterNetInterfaceControl(HNETINTERFACE, HNODE, DWORD, LPVOID, DWORD, LPVOID, DWORD, LPDWORD);
HKEY WINAPI GetClusterKey(HCLUSTER, REGSAM);
HKEY WINAPI GetClusterGroupKey(HGROUP, REGSAM);
HKEY WINAPI GetClusterResourceKey(HRESOURCE, REGSAM);
HKEY WINAPI GetClusterNodeKey(HNODE, REGSAM);
HKEY WINAPI GetClusterNetworkKey(HNETWORK, REGSAM);
HKEY WINAPI GetClusterNetInterfaceKey(HNETINTERFACE, REGSAM);
LONG WINAPI ClusterRegCreateKey(HKEY, LPCWSTR, DWORD, REGSAM, LPSECURITY_ATTRIBUTES, PHKEY, LPDWORD);
LONG WINAPI ClusterRegOpenKey(HKEY, LPCWSTR, REGSAM, PHKEY);
LONG WINAPI ClusterRegDeleteKey(HKEY, LPCWSTR);
LONG WINAPI ClusterRegCloseKey(HKEY);
LONG WINAPI ClusterRegEnumKey(HKEY, DWORD, LPWSTR, LPDWORD, PFILETIME);
DWORD WINAPI ClusterRegSetValue(HKEY, LPCWSTR, DWORD, CONST BYTE *, DWORD);
DWORD WINAPI ClusterRegDeleteValue(HKEY, LPCWSTR);
LONG WINAPI ClusterRegQueryValue(HKEY, LPCWSTR, LPDWORD, LPBYTE, LPDWORD);
DWORD WINAPI ClusterRegEnumValue(HKEY, DWORD, LPWSTR, LPDWORD, LPDWORD, LPBYTE, LPDWORD);
LONG WINAPI ClusterRegQueryInfoKey(HKEY, LPDWORD, LPDWORD, LPDWORD, LPDWORD, LPDWORD, LPDWORD, PFILETIME);
LONG WINAPI ClusterRegGetKeySecurity(HKEY, SECURITY_INFORMATION, PSECURITY_DESCRIPTOR, LPDWORD);
LONG WINAPI ClusterRegSetKeySecurity(HKEY, SECURITY_INFORMATION, PSECURITY_DESCRIPTOR);
LONG WINAPI ClusterRegSyncDatabase(HCLUSTER,DWORD);
LONG WINAPI ClusterRegCreateBatch(HKEY, HREGBATCH *);
LONG WINAPI ClusterRegBatchAddCommand(HREGBATCH, CLUSTER_REG_COMMAND, LPCWSTR, DWORD, VOID CONST *, DWORD);
LONG WINAPI ClusterRegCloseBatch(HREGBATCH, BOOL, INT *);
LONG WINAPI ClusterRegCloseBatchEx(HREGBATCH,DWORD,INT*);
LONG WINAPI ClusterRegBatchReadCommand(HREGBATCHNOTIFICATION, CLUSTER_BATCH_COMMAND *);
LONG WINAPI ClusterRegBatchCloseNotification(HREGBATCHNOTIFICATION);
LONG WINAPI ClusterRegCreateBatchNotifyPort(HKEY hKey, HREGBATCHPORT *);
LONG WINAPI ClusterRegCloseBatchNotifyPort(HREGBATCHPORT);
LONG WINAPI ClusterRegGetBatchNotification(HREGBATCHPORT, HREGBATCHNOTIFICATION *);
LONG WINAPI ClusterRegCreateReadBatch(HKEY,HREGREADBATCH*);
LONG WINAPI ClusterRegReadBatchAddCommand(HREGREADBATCH,LPCWSTR,LPCWSTR);
LONG WINAPI ClusterRegCloseReadBatch(HREGREADBATCH,HREGREADBATCHREPLY*);
LONG WINAPI ClusterRegCloseReadBatchEx(HREGREADBATCH,DWORD,HREGREADBATCHREPLY*);
LONG WINAPI ClusterRegReadBatchReplyNextCommand(HREGREADBATCHREPLY,CLUSTER_READ_BATCH_COMMAND*);
LONG WINAPI ClusterRegCloseReadBatchReply(HREGREADBATCHREPLY);
DWORD WINAPI ClusterSetAccountAccess(HCLUSTER,LPCWSTR,DWORD,DWORD);
HCLUSTER WINAPI CreateCluster(PCREATE_CLUSTER_CONFIG, PCLUSTER_SETUP_PROGRESS_CALLBACK, PVOID);
HCLUSTER WINAPI CreateClusterCNOless(PCREATE_CLUSTER_CONFIG,PCLUSTER_SETUP_PROGRESS_CALLBACK,PVOID);
DWORD WINAPI CreateClusterNameAccount(HCLUSTER,PCREATE_CLUSTER_NAME_ACCOUNT,PCLUSTER_SETUP_PROGRESS_CALLBACK,PVOID);
DWORD WINAPI RegisterClusterResourceTypeNotifyV2(HCHANGE,HCLUSTER,LONGLONG,LPCWSTR,DWORD_PTR);
HNODE WINAPI AddClusterNode(HCLUSTER, PCWSTR, PCLUSTER_SETUP_PROGRESS_CALLBACK, PVOID);
DWORD WINAPI DestroyCluster(HCLUSTER, PCLUSTER_SETUP_PROGRESS_CALLBACK, PVOID, BOOL);

#define CLUS_RESTYPE_NAME_GENAPP  L"Generic Application"
#define CLUS_RESTYPE_NAME_GENSVC  L"Generic Service"
#define CLUS_RESTYPE_NAME_GENSCRIPT  L"Generic Script"
#define CLUS_RESTYPE_NAME_IPADDR  L"IP Address"
#define CLUS_RESTYPE_NAME_NETNAME  L"Network Name"
#define CLUS_RESTYPE_NAME_DNN  L"Distributed Network Name"
#define CLUS_RESTYPE_NAME_FILESHR  L"File Share"
#define CLUS_RESTYPE_NAME_PRTSPLR  L"Print Spooler"
#define CLUS_RESTYPE_NAME_IPV6_NATIVE  L"IPv6 Address"
#define CLUS_RESTYPE_NAME_IPV6_TUNNEL  L"IPv6 Tunnel Address"
#define CLUS_RESTYPE_NAME_VSSTASK  L"Volume Shadow Copy Service Task"
#define CLUS_RESTYPE_NAME_WINS  L"WINS Service"
#define CLUS_RESTYPE_NAME_DHCP  L"DHCP Service"
#define CLUS_RESTYPE_NAME_MSMQ  L"Microsoft Message Queue Server"
#define CLUS_RESTYPE_NAME_NEW_MSMQ  L"MSMQ"
#define CLUS_RESTYPE_NAME_MSMQ_TRIGGER  L"MSMQTriggers"
#define CLUS_RESTYPE_NAME_MSDTC  L"Distributed Transaction Coordinator"
#define CLUS_RESTYPE_NAME_NFS  L"NFS Share"
#define CLUS_RESTYPE_NAME_NETWORK_FILE_SYSTEM   L"Network File System"
#define CLUS_RESTYPE_NAME_ISNS  L"Microsoft iSNS"
#define CLUS_RESTYPE_NAME_HARDDISK  L"Physical Disk"
#define CLUS_RESTYPE_NAME_PHYS_DISK  CLUS_RESTYPE_NAME_HARDDISK
#define CLUS_RESTYPE_NAME_FSWITNESS  L"File Share Witness"
#define CLUS_RESTYPE_NAME_FILESERVER  L"File Server"
#define CLUS_RESTYPE_NAME_SODAFILESERVER  L"Scale Out File Server"
#define CLUS_RESTYPE_NAME_DFS  L"Distributed File System"
#define CLUS_RESTYPE_NAME_DFSR  L"DFS Replicated Folder"
#define CLUS_RESTYPE_NAME_VM  L"Virtual Machine"
#define CLUS_RESTYPE_NAME_VM_CONFIG  L"Virtual Machine Configuration"
#define CLUS_RESTYPE_NAME_ISCSITARGET  L"iSCSI Target Server"
#define CLUS_RESTYPE_NAME_STORAGE_POOL  L"Storage Pool"
#define CLUS_RESTYPE_NAME_TASKSCHEDULER  L"Task Scheduler"
#define CLUS_RESTYPE_NAME_VMREPLICA_BROKER  L"Virtual Machine Replication Broker"
#define CLUS_RESTYPE_NAME_VMREPLICA_COORDINATOR  L"Virtual Machine Replication Coordinator"
#define CLUS_RESTYPE_NAME_NFS_V2  L"Network File System"
#define CLUS_RESTYPE_NAME_CAU  L"ClusterAwareUpdatingResource"
#define CLUS_RESTYPE_NAME_NV_PROVIDER_ADDRESS  L"Provider Address"
#define CLUS_RESTYPE_NAME_NAT  L"Nat"
#define CLUS_RESTYPE_NAME_STORAGE_POLICIES  L"Storage Policies"
#define CLUS_RESTYPE_NAME_STORQOS  L"Storage QoS Policy Manager"
#define CLUS_RESTYPE_NAME_HEALTH_SERVICE  L"Health Service"
#define CLUS_RESTYPE_NAME_VM_WMI  L"Virtual Machine Cluster WMI"

#define CLUS_RESTYPE_NAME_VIRTUAL_IPV4  L"Disjoint IPv4 Address"
#define CLUS_RESTYPE_NAME_VIRTUAL_IPV6  L"Disjoint IPv6 Address"
#define CLUS_RESTYPE_NAME_CLOUD_WITNESS  L"Cloud Witness"
#define CLUS_RESTYPE_NAME_STORAGE_REPLICA  L"Storage Replica"
#define CLUS_RESTYPE_NAME_CROSS_CLUSTER  L"Cross Cluster Dependency Orchestrator"

#define CLUSREG_NAME_CLUS_DESC  L"Description"
#define CLUSREG_NAME_CLUS_SD  L"Security Descriptor"
#define CLUSREG_NAME_CLUS_DEFAULT_NETWORK_ROLE  L"DefaultNetworkRole"
#define CLUSREG_NAME_QUORUM_ARBITRATION_TIMEOUT  L"QuorumArbitrationTimeMax"
#define CLUSTER_HANG_TIMEOUT_KEYNAME  L"ClusSvcHangTimeout"
#define CLUSTER_HANG_RECOVERY_ACTION_KEYNAME  L"HangRecoveryAction"
#define CLUSTER_CSA_VSS_STATE  L"BackupInProgress"
#define CLUSTER_REQUEST_REPLY_TIMEOUT  L"RequestReplyTimeout"
#define CLUSTER_WITNESS_FAILED_RESTART_INTERVAL  L"WitnessRestartInterval"
#define CLUSTER_WITNESS_DATABASE_WRITE_TIMEOUT  L"WitnessDatabaseWriteTimeout"

#define CLUSTER_ADD_EVICT_DELAY  L"AddEvictDelay"
#define CLUSREG_NAME_FIXQUORUM  L"FixQuorum"
#define CLUSREG_NAME_PREVENTQUORUM  L"PreventQuorum"
#define CLUSREG_NAME_IGNORE_PERSISTENT_STATE  L"IgnorePersistentStateOnStartup"
#define CLUSTER_SHARED_VOLUMES_ROOT  L"SharedVolumesRoot"
#define ENABLE_CLUSTER_SHARED_VOLUMES  L"EnableSharedVolumes"
#define CLUSTER_SHARED_VOLUME_VSS_WRITER_OPERATION_TIMEOUT  L"SharedVolumeVssWriterOperationTimeout"
#define USE_CLIENT_ACCESS_NETWORKS_FOR_CSV  L"UseClientAccessNetworksForSharedVolumes"
#define CLUSTER_CSV_COMPATIBLE_FILTERS  L"SharedVolumeCompatibleFilters"
#define CLUSTER_CSV_INCOMPATIBLE_FILTERS  L"SharedVolumeIncompatibleFilters"
#define CLUSTER_GROUP_WAIT_DELAY  L"ClusterGroupWaitDelay"
#define MINIMUM_PREEMPTOR_PRIORITY  L"MinimumPreemptorPriority"
#define MINIMUM_NEVER_PREEMPT_PRIORITY  L"MinimumNeverPreemptPriority"
#define CLUSTER_ENFORCED_ANTIAFFINITY  L"ClusterEnforcedAntiaffinity"
#define CLUSREG_NAME_SHUTDOWN_TIMEOUT_MINUTES  L"ShutdownTimeoutInMinutes"
#define CLUSREG_NAME_CSV_MDS_SD  L"SharedVolumeSecurityDescriptor"
#define CLUSREG_NAME_FAILOVER_MOVE_MIGRATION_TYPE  L"FailoverMoveMigrationType"
#define CLUSREG_NAME_CSV_BLOCK_CACHE  L"BlockCacheSize"
#define CLUSREG_NAME_ROUTE_HISTORY_LENGTH  L"RouteHistoryLength"
#define CLUSREG_NAME_LAST_RECENT_EVENTS_RESET_TIME  L"RecentEventsResetTime"
#define CLUSREG_NAME_DRAIN_ON_SHUTDOWN  L"DrainOnShutdown"
#define CLUSREG_NAME_NETFT_IPSEC_ENABLED  L"NetftIPSecEnabled"
#define CLUSREG_NAME_WITNESS_DYNAMIC_WEIGHT  L"WitnessDynamicWeight"
#define CLUSREG_NAME_MESSAGE_BUFFER_LENGTH  L"MessageBufferLength"
#define CLUSREG_NAME_DATABASE_READ_WRITE_MODE  L"DatabaseReadWriteMode"
#define CLUSREG_NAME_FUNCTIONAL_LEVEL  L"ClusterFunctionalLevel"
#define CLUSREG_NAME_UPGRADE_VERSION  L"ClusterUpgradeVersion"
#define CLUSREG_NAME_RESILIENCY_LEVEL  L"ResiliencyLevel"
#define CLUSREG_NAME_RESILIENCY_DEFAULT_SECONDS  L"ResiliencyDefaultPeriod"
#define CLUSREG_NAME_QUARANTINE_THRESHOLD  L"QuarantineThreshold"
#define CLUSREG_NAME_QUARANTINE_DURATION  L"QuarantineDuration"
#define CLUSTER_S2D_ENABLED  L"S2DEnabled"
#define CLUSTER_S2D_BUS_TYPES  L"S2DBusTypes"
#define CLUSTER_S2D_CACHE_PAGE_SIZE_KBYTES  L"S2DCachePageSizeKBytes"
#define CLUSTER_S2D_OPTIMIZATIONS  L"S2DOptimizations"
#define CLUSTER_S2D_IO_LATENCY_THRESHOLD  L"S2DIOLatencyThreshold"
#define CLUSTER_S2D_CACHE_DESIRED_STATE  L"S2DCacheDesiredState"
#define CLUSTER_S2D_CACHE_METADATA_RESERVE  L"S2DCacheMetadataReserveBytes"
#define CLUSTER_S2D_CACHE_FLASH_RESERVE_PERCENT  L"S2DCacheFlashReservePercent"
#define CLUSTER_S2D_CACHE_BEHAVIOR_FLAGS  L"S2DCacheBehavior"
#define CLUSTER_NAME_PREFERRED_SITE  L"PreferredSite"
#define CLUSTER_NAME_AUTO_BALANCER_MODE  L"AutoBalancerMode"
#define CLUSTER_NAME_AUTO_BALANCER_LEVEL  L"AutoBalancerLevel"
#define CLUSREG_NAME_GROUP_DEPENDENCY_TIMEOUT  L"GroupDependencyTimeout"
#define CLUSREG_NAME_PLACEMENT_OPTIONS  L"PlacementOptions"

#define CLUSREG_NAME_SAME_SUBNET_DELAY  L"SameSubnetDelay"
#define CLUSREG_NAME_CROSS_SUBNET_DELAY  L"CrossSubnetDelay"
#define CLUSREG_NAME_CROSS_SITE_DELAY  L"CrossSiteDelay"
#define CLUSREG_NAME_SAME_SUBNET_THRESHOLD  L"SameSubnetThreshold"
#define CLUSREG_NAME_CROSS_SUBNET_THRESHOLD  L"CrossSubnetThreshold"
#define CLUSREG_NAME_CROSS_SITE_THRESHOLD  L"CrossSiteThreshold"
#define CLUSREG_NAME_PLUMB_ALL_CROSS_SUBNET_ROUTES  L"PlumbAllCrossSubnetRoutes"

#define CLUSREG_NAME_NODE_NAME  L"NodeName"
#define CLUSREG_NAME_NODE_HIGHEST_VERSION  L"NodeHighestVersion"
#define CLUSREG_NAME_NODE_LOWEST_VERSION  L"NodeLowestVersion"
#define CLUSREG_NAME_NODE_DESC  L"Description"
#define CLUSREG_NAME_NODE_MAJOR_VERSION  L"MajorVersion"
#define CLUSREG_NAME_NODE_MINOR_VERSION  L"MinorVersion"
#define CLUSREG_NAME_NODE_BUILD_NUMBER  L"BuildNumber"
#define CLUSREG_NAME_NODE_CSDVERSION  L"CSDVersion"
#define CLUSREG_NAME_NODE_WEIGHT  L"NodeWeight"
#define CLUSREG_NAME_NODE_DYNAMIC_WEIGHT  L"DynamicWeight"
#define CLUSREG_NAME_NODE_IS_PRIMARY  L"IsPrimary"
#define CLUSREG_NAME_NODE_DRAIN_STATUS  L"NodeDrainStatus"
#define CLUSREG_NAME_NODE_DRAIN_TARGET  L"NodeDrainTarget"
#define CLUSREG_NAME_NODE_NEEDS_PQ  L"NeedsPreventQuorum"
#define CLUSREG_NAME_NODE_FDID  L"FaultDomainId"
#define CLUSREG_NAME_NODE_STATUS_INFO  L"StatusInformation"
#define CLUSREG_NAME_NODE_FAULT_DOMAIN  L"FaultDomain"
#define CLUSREG_NAME_NODE_MODEL  L"Model"
#define CLUSREG_NAME_NODE_SERIALNUMBER  L"SerialNumber"
#define CLUSREG_NAME_NODE_MANUFACTURER  L"Manufacturer"

#define CLUSREG_NAME_GRP_NAME  L"Name"
#define CLUSREG_NAME_GRP_TYPE  L"GroupType"
#define CLUSREG_NAME_GRP_DESC  L"Description"
#define CLUSREG_NAME_GRP_PERSISTENT_STATE  L"PersistentState"
#define CLUSREG_NAME_GRP_FAILBACK_TYPE  L"AutoFailbackType"
#define CLUSREG_NAME_GRP_FAILBACK_WIN_START  L"FailbackWindowStart"
#define CLUSREG_NAME_GRP_FAILBACK_WIN_END  L"FailbackWindowEnd"
#define CLUSREG_NAME_GRP_FAILOVER_THRESHOLD  L"FailoverThreshold"
#define CLUSREG_NAME_GRP_FAILOVER_PERIOD  L"FailoverPeriod"
#define CLUSREG_NAME_GRP_PRIORITY  L"Priority"
#define CLUSREG_NAME_GRP_DEFAULT_OWNER  L"DefaultOwner"
#define CLUSREG_NAME_GRP_STATUS_INFORMATION  L"StatusInformation"
#define CLUSREG_NAME_GRP_ANTI_AFFINITY_CLASS_NAME  L"AntiAffinityClassNames"
#define CLUSREG_NAME_GRP_START_DELAY  L"GroupStartDelay"
#define CLUSREG_NAME_GRP_CCF_EPOCH  L"CCFEpoch"
#define CLUSREG_NAME_GRP_RESILIENCY_PERIOD  L"ResiliencyPeriod"
#define CLUSREG_NAME_GRP_PREFERRED_SITE  L"PreferredSite"
#define CLUSREG_NAME_GRP_COLD_START_SETTING  L"ColdStartSetting"

#define CLUSREG_NAME_RES_NAME  L"Name"
#define CLUSREG_NAME_RES_TYPE  L"Type"
#define CLUSREG_NAME_RES_DESC  L"Description"
#define CLUSREG_NAME_RES_SEPARATE_MONITOR  L"SeparateMonitor"
#define CLUSREG_NAME_RES_PERSISTENT_STATE  L"PersistentState"
#define CLUSREG_NAME_RES_LOOKS_ALIVE  L"LooksAlivePollInterval"
#define CLUSREG_NAME_RES_IS_ALIVE  L"IsAlivePollInterval"
#define CLUSREG_NAME_RES_RESTART_ACTION  L"RestartAction"
#define CLUSREG_NAME_RES_RESTART_THRESHOLD  L"RestartThreshold"
#define CLUSREG_NAME_RES_RESTART_PERIOD  L"RestartPeriod"
#define CLUSREG_NAME_RES_RESTART_DELAY  L"RestartDelay"
#define CLUSREG_NAME_RES_RETRY_PERIOD_ON_FAILURE  L"RetryPeriodOnFailure"
#define CLUSREG_NAME_RES_EMBEDDED_FAILURE_ACTION  L"EmbeddedFailureAction"
#define CLUSREG_NAME_RES_PENDING_TIMEOUT  L"PendingTimeout"
#define CLUSREG_NAME_RES_DEADLOCK_TIMEOUT  L"DeadlockTimeout"
#define CLUSREG_NAME_RES_MONITOR_PID  L"MonitorProcessId"
#define CLUSREG_NAME_RES_STATUS_INFORMATION  L"StatusInformation"
#define CLUSREG_NAME_RES_LAST_OPERATION_STATUS_CODE  L"LastOperationStatusCode"
#define CLUSREG_NAME_RES_DATA1  L"ResourceSpecificData1"
#define CLUSREG_NAME_RES_DATA2  L"ResourceSpecificData2"
#define CLUSREG_NAME_RES_STATUS  L"ResourceSpecificStatus"

#define CLUSREG_NAME_RESTYPE_NAME  L"Name"
#define CLUSREG_NAME_RESTYPE_DESC  L"Description"
#define CLUSREG_NAME_RESTYPE_DLL_NAME  L"DllName"
#define CLUSREG_NAME_RESTYPE_ADMIN_EXTENSIONS  L"AdminExtensions"
#define CLUSREG_NAME_RESTYPE_LOOKS_ALIVE  CLUSREG_NAME_RES_LOOKS_ALIVE
#define CLUSREG_NAME_RESTYPE_IS_ALIVE  CLUSREG_NAME_RES_IS_ALIVE
#define CLUSREG_NAME_RESTYPE_PENDING_TIMEOUT  CLUSREG_NAME_RES_PENDING_TIMEOUT
#define CLUSREG_NAME_RESTYPE_DEADLOCK_TIMEOUT  CLUSREG_NAME_RES_DEADLOCK_TIMEOUT
#define CLUSREG_NAME_RESTYPE_DUMP_POLICY  L"DumpPolicy"
#define CLUSREG_NAME_RESTYPE_DUMP_LOG_QUERY  L"DumpLogQuery"
#define CLUSREG_NAME_RESTYPE_DUMP_SERVICES  L"DumpServices"
#define CLUSREG_NAME_RESTYPE_ENABLED_EVENT_LOGS  L"EnabledEventLogs"
#define CLUSREG_NAME_RESTYPE_MAX_MONITORS  L"MaximumMonitors"

#define CLUSREG_NAME_NET_NAME  L"Name"
#define CLUSREG_NAME_NET_IPV6_ADDRESSES  L"IPv6Addresses"
#define CLUSREG_NAME_NET_IPV6_PREFIXLENGTHS L"IPv6PrefixLengths"
#define CLUSREG_NAME_NET_IPV4_ADDRESSES  L"IPv4Addresses"
#define CLUSREG_NAME_NET_IPV4_PREFIXLENGTHS L"IPv4PrefixLengths"
#define CLUSREG_NAME_NET_ADDRESS  L"Address"
#define CLUSREG_NAME_NET_ADDRESS_MASK  L"AddressMask"
#define CLUSREG_NAME_NET_DESC  L"Description"
#define CLUSREG_NAME_NET_ROLE  L"Role"
#define CLUSREG_NAME_NET_SPEED  L"LinkSpeed"
#define CLUSREG_NAME_NET_RDMA_CAPABLE  L"RdmaCapable"
#define CLUSREG_NAME_NET_RSS_CAPABLE  L"RssCapable"
#define CLUSREG_NAME_NET_METRIC  L"Metric"
#define CLUSREG_NAME_NET_AUTOMETRIC  L"AutoMetric"

#define CLUSREG_NAME_NETIFACE_NAME  L"Name"
#define CLUSREG_NAME_NETIFACE_NODE  L"Node"
#define CLUSREG_NAME_NETIFACE_NETWORK  L"Network"
#define CLUSREG_NAME_NETIFACE_ADAPTER_NAME  L"Adapter"
#define CLUSREG_NAME_NETIFACE_ADAPTER_ID  L"AdapterId"
#define CLUSREG_NAME_NETIFACE_DHCP_ENABLED  L"DhcpEnabled"
#define CLUSREG_NAME_NETIFACE_IPV6_ADDRESSES  L"IPv6Addresses"
#define CLUSREG_NAME_NETIFACE_IPV4_ADDRESSES  L"IPv4Addresses"
#define CLUSREG_NAME_NETIFACE_ADDRESS  L"Address"
#define CLUSREG_NAME_NETIFACE_DESC  L"Description"

#define CLUSREG_NAME_GROUPSET_NAME  L"Name"
#define CLUSREG_NAME_GROUPSET_STARTUP_SETTING  L"StartupSetting"
#define CLUSREG_NAME_GROUPSET_STARTUP_COUNT  L"StartupCount"
#define CLUSREG_NAME_GROUPSET_STARTUP_DELAY  L"StartupDelay"
#define CLUSREG_NAME_GROUPSET_IS_GLOBAL  L"IsGlobal"
#define CLUSREG_NAME_GROUPSET_STATUS_INFORMATION  L"StatusInformation"

#define CLUSREG_NAME_START_MEMORY  L"StartMemory"
#define CLUSREG_NAME_VIRTUAL_NUMA_COUNT  L"VirtualNumaCount"

#define CLUSREG_NAME_PHYSDISK_DISKIDTYPE  L"DiskIdType"
#define CLUSREG_NAME_PHYSDISK_DISKSIGNATURE  L"DiskSignature"
#define CLUSREG_NAME_PHYSDISK_DISKIDGUID  L"DiskIdGuid"
#define CLUSREG_NAME_PHYSDISK_DISKRUNCHKDSK  L"DiskRunChkDsk"
#define CLUSREG_NAME_PHYSDISK_DISKUNIQUEIDS  L"DiskUniqueIds"
#define CLUSREG_NAME_PHYSDISK_DISKVOLUMEINFO  L"DiskVolumeInfo"
#define CLUSREG_NAME_PHYSDISK_DISKARBTYPE  L"DiskArbType"
#define CLUSREG_NAME_PHYSDISK_DISKARBINTERVAL  L"DiskArbInterval"
#define CLUSREG_NAME_PHYSDISK_DISKPATH  L"DiskPath"
#define CLUSREG_NAME_PHYSDISK_DISKRELOAD  L"DiskReload"
#define CLUSREG_NAME_PHYSDISK_MAINTMODE  L"MaintenanceMode"
#define CLUSREG_NAME_PHYSDISK_DISKIODELAY  L"MaxIoLatency"
#define CLUSREG_NAME_PHYSDISK_MIGRATEFIXUP  L"MigrateDriveLetters"
#define CLUSREG_NAME_PHYSDISK_CSVWRITETHROUGH  L"CsvEnforceWriteThrough"
#define CLUSREG_NAME_PHYSDISK_CSVBLOCKCACHE  L"EnableBlockCache"
#define CLUSREG_NAME_PHYSDISK_FASTONLINEARBITRATE  L"FastOnlineArbitrate"
#define CLUSREG_NAME_PHYSDISK_SPACEIDGUID  L"VirtualDiskId"
#define CLUSREG_NAME_STORAGESPACE_POOLIDGUID  L"PoolId"
#define CLUSREG_NAME_PHYSDISK_CSVSNAPSHOTDIFFAREASIZE  L"SnapshotDiffSize"
#define CLUSREG_NAME_PHYSDISK_CSVSNAPSHOTAGELIMIT  L"SnapshotAgeLimit"
#define CLUSREG_NAME_PHYSDISK_DISKGUID  L"DiskGuid"
#define CLUSREG_NAME_PHYSDISK_VOLSNAPACTIVATETIMEOUT  L"VolsnapActivateTimeout"

#define CLUSREG_NAME_STORAGESPACE_NAME  L"VirtualDiskName"
#define CLUSREG_NAME_STORAGESPACE_DESCRIPTION  L"VirtualDiskDescription"
#define CLUSREG_NAME_STORAGESPACE_HEALTH  L"VirtualDiskHealth"
#define CLUSREG_NAME_STORAGESPACE_STATE  L"VirtualDiskState"
#define CLUSREG_NAME_STORAGESPACE_PROVISIONING  L"VirtualDiskProvisioning"
#define CLUSREG_NAME_STORAGESPACE_RESILIENCYTYPE  L"VirtualDiskResiliencyType"
#define CLUSREG_NAME_STORAGESPACE_RESILIENCYCOLUMNS  L"VirtualDiskResiliencyColumns"
#define CLUSREG_NAME_STORAGESPACE_RESILIENCYINTERLEAVE  L"VirtualDiskResiliencyInterleave"

#define CLUSREG_NAME_GENAPP_COMMAND_LINE  L"CommandLine"
#define CLUSREG_NAME_GENAPP_CURRENT_DIRECTORY  L"CurrentDirectory"
#define CLUSREG_NAME_GENAPP_USE_NETWORK_NAME  L"UseNetworkName"

#define CLUSREG_NAME_GENSCRIPT_SCRIPT_FILEPATH  L"ScriptFilepath"

#define CLUSREG_NAME_GENSVC_SERVICE_NAME  L"ServiceName"
#define CLUSREG_NAME_GENSVC_STARTUP_PARAMS  L"StartupParameters"
#define CLUSREG_NAME_GENSVC_USE_NETWORK_NAME  L"UseNetworkName"

#define CLUSREG_NAME_IPADDR_NETWORK  L"Network"
#define CLUSREG_NAME_IPADDR_ADDRESS  L"Address"
#define CLUSREG_NAME_IPADDR_SUBNET_MASK  L"SubnetMask"
#define CLUSREG_NAME_IPADDR_ENABLE_NETBIOS  L"EnableNetBIOS"
#define CLUSREG_NAME_IPADDR_OVERRIDE_ADDRMATCH  L"OverrideAddressMatch"
#define CLUSREG_NAME_IPADDR_ENABLE_DHCP  L"EnableDhcp"
#define CLUSREG_NAME_IPADDR_LEASE_OBTAINED_TIME  L"LeaseObtainedTime"
#define CLUSREG_NAME_IPADDR_LEASE_TERMINATES_TIME  L"LeaseExpiresTime"
#define CLUSREG_NAME_IPADDR_T1  L"T1"
#define CLUSREG_NAME_IPADDR_T2  L"T2"
#define CLUSREG_NAME_IPADDR_DHCP_SERVER  L"DhcpServer"
#define CLUSREG_NAME_IPADDR_DHCP_ADDRESS  L"DhcpAddress"
#define CLUSREG_NAME_IPADDR_DHCP_SUBNET_MASK  L"DhcpSubnetMask"
#define CLUSREG_NAME_IPADDR_SHARED_NETNAME  L"SharedNetname"
#define CLUSREG_NAME_IPADDR_PROBE_PORT  L"ProbePort"
#define CLUSREG_NAME_IPADDR_PROBE_FAILURE_THRESHOLD  L"ProbeFailureThreshold"

#define CLUSREG_NAME_IPV6_NATIVE_NETWORK  L"Network"
#define CLUSREG_NAME_IPV6_NATIVE_ADDRESS  L"Address"
#define CLUSREG_NAME_IPV6_NATIVE_PREFIX_LENGTH  L"PrefixLength"

#define CLUSREG_NAME_IPV6_TUNNEL_ADDRESS  L"Address"
#define CLUSREG_NAME_IPV6_TUNNEL_TUNNELTYPE  L"TunnelType"

#define CLUSREG_NAME_NETNAME_NAME  L"Name"
#define CLUSREG_NAME_NETNAME_CREATING_DC  L"CreatingDC"
#define CLUSREG_NAME_NETNAME_OBJECT_ID  L"ObjectGUID"
#define CLUSREG_NAME_NETNAME_DNS_NAME  L"DnsName"
#define CLUSREG_NAME_NETNAME_REMAP_PIPE_NAMES  L"RemapPipeNames"
#define CLUSREG_NAME_NETNAME_RESOURCE_DATA  L"ResourceData"
#define CLUSREG_NAME_NETNAME_STATUS_NETBIOS  L"StatusNetBIOS"
#define CLUSREG_NAME_NETNAME_STATUS_DNS  L"StatusDNS"
#define CLUSREG_NAME_NETNAME_STATUS_KERBEROS  L"StatusKerberos"
#define CLUSREG_NAME_NETNAME_VCO_CONTAINER  L"VcoContainer"
#define CLUSREG_NAME_NETNAME_LAST_DNS_UPDATE  L"LastDNSUpdateTime"
#define CLUSREG_NAME_NETNAME_CONTAINERGUID  L"CryptoContainerGUID"
#define CLUSREG_NAME_NETNAME_HOST_TTL  L"HostRecordTTL"
#define CLUSREG_NAME_NETNAME_PUBLISH_PTR  L"PublishPTRRecords"
#define CLUSREG_NAME_NETNAME_REMOVEVCO_ONDELETE  L"DeleteVcoOnResCleanup"
#define CLUSREG_NAME_NETNAME_REGISTER_ALL_IP  L"RegisterAllProvidersIP"
#define CLUSREG_KEYNAME_OBJECTGUIDS  L"ObjectGUIDs"
#define CLUSREG_NAME_NETNAME_EXCLUDE_NETWORKS  L"ExcludeNetworks"
#define CLUSREG_NAME_NETNAME_ALIASES  L"Aliases"
#define CLUSREG_NAME_NETNAME_IN_USE_NETWORKS  L"InUseNetworks"
#define CLUSREG_NAME_NETNAME_DNS_SUFFIX  L"DnsSuffix"
#define CLUSREG_NAME_NETNAME_AD_AWARE  L"ADAware"

#define CLUSREG_NAME_PRTSPOOL_DEFAULT_SPOOL_DIR  L"DefaultSpoolDirectory"
#define CLUSREG_NAME_PRTSPOOL_TIMEOUT  L"JobCompletionTimeout"

#define CLUSREG_NAME_FILESHR_SERVER_NAME  L"ServerName"
#define CLUSREG_NAME_FILESHR_SHARE_NAME  L"ShareName"
#define CLUSREG_NAME_FILESHR_PATH  L"Path"
#define CLUSREG_NAME_FILESHR_REMARK  L"Remark"
#define CLUSREG_NAME_FILESHR_MAX_USERS  L"MaxUsers"
#define CLUSREG_NAME_FILESHR_SD  L"Security Descriptor"
#define CLUSREG_NAME_FILESHR_SHARE_SUBDIRS  L"ShareSubDirs"
#define CLUSREG_NAME_FILESHR_HIDE_SUBDIR_SHARES  L"HideSubDirShares"
#define CLUSREG_NAME_FILESHR_IS_DFS_ROOT  L"IsDfsRoot"
#define CLUSREG_NAME_FILESHR_SHARE_FLAGS  L"ShareFlags"
#define CLUSREG_NAME_FILESHR_CA_TIMEOUT  L"CATimeout"

#define CLUSREG_NAME_DHCP_DATABASE_PATH  L"DatabasePath"
#define CLUSREG_NAME_DHCP_BACKUP_PATH  L"BackupPath"
#define CLUSREG_NAME_LOG_FILE_PATH  L"LogFilePath"

#define CLUSREG_NAME_WINS_DATABASE_PATH  L"DatabasePath"
#define CLUSREG_NAME_WINS_BACKUP_PATH  L"BackupPath"

#define CLUSREG_NAME_VSSTASK_CURRENTDIRECTORY  L"CurrentDirectory"
#define CLUSREG_NAME_VSSTASK_APPNAME  L"ApplicationName"
#define CLUSREG_NAME_VSSTASK_APPPARAMS  L"ApplicationParams"
#define CLUSREG_NAME_VSSTASK_TRIGGERARRAY  L"TriggerArray"

#define CLUSREG_NAME_FSWITNESS_SHARE_PATH  L"SharePath"
#define CLUSREG_NAME_FSWITNESS_ARB_DELAY  L"ArbitrationDelay"
#define CLUSREG_NAME_FSWITNESS_IMPERSONATE_CNO  L"ImpersonateCNO"

#define CLUSREG_NAME_STORAGESPACE_POOLNAME  L"Name"
#define CLUSREG_NAME_STORAGESPACE_POOLDESC  L"Description"
#define CLUSREG_NAME_STORAGESPACE_POOLDRIVEIDS  L"DriveIds"
#define CLUSREG_NAME_STORAGESPACE_POOLHEALTH  L"Health"
#define CLUSREG_NAME_STORAGESPACE_POOLSTATE  L"State"
#define CLUSREG_NAME_STORAGESPACE_POOLTOTALCAPACITY  L"TotalCapacity"
#define CLUSREG_NAME_STORAGESPACE_POOLCONSUMEDCAPACITY  L"ConsumedCapacity"

#define CLUSREG_NAME_STORAGESPACE_POOLARBITRATE  L"Arbitrate"
#define CLUSREG_NAME_STORAGESPACE_POOLREEVALTIMEOUT  L"ReEvaluatePlacementTimeout"
#define CLUSREG_NAME_STORAGESPACE_POOLQUORUMSHARE  L"PoolQuorumShare"
#define CLUSREG_NAME_STORAGESPACE_POOLQUORUMUSERACCOUNT  L"PoolQuorumUserAccount"

#define CLUSREG_NAME_SOFS_SMBASYMMETRYMODE  L"SmbAsymmetryMode"

#define CLUSREG_NAME_VIP_PREFIX_LENGTH  L"PrefixLength"
#define CLUSREG_NAME_VIP_ADAPTER_NAME  L"AdapterName"
#define CLUSREG_NAME_VIP_ADDRESS  L"Address"
#define CLUSREG_NAME_VIP_VSID  L"VSID"
#define CLUSREG_NAME_VIP_RDID  L"RDID"

#define CLUSREG_NAME_CLOUDWITNESS_PRIMARY_TOKEN  L"PrimaryToken"
#define CLUSREG_NAME_CLOUDWITNESS_PRIMARY_KEY  L"PrimaryKey"
#define CLUSREG_NAME_CLOUDWITNESS_ACCOUNT_NAME  L"AccountName"
#define CLUSREG_NAME_CLOUDWITNESS_ENDPOINT_INFO  L"EndpointInfo"
#define CLOUD_WITNESS_CONTAINER_NAME  L"msft-cloud-witness"

#define CLUS_NAME_RES_TYPE_SOURCE_RESID  L"SourceResourceId"
#define CLUS_NAME_RES_TYPE_TARGET_RESID  L"TargetResourceId"
#define CLUS_NAME_RES_TYPE_SOURCE_VOLUMES  L"SourceVolumes"
#define CLUS_NAME_RES_TYPE_TARGET_VOLUMES  L"TargetVolumes"
#define CLUS_NAME_RES_TYPE_DATA_RESID  L"DataResourceId"
#define CLUS_NAME_RES_TYPE_LOG_RESID  L"LogResourceId"
#define CLUS_NAME_RES_TYPE_LOG_VOLUME  L"LogVolume"
#define CLUS_NAME_RES_TYPE_REPLICATION_GROUPID  L"ReplicationGroupId"
#define CLUS_NAME_RES_TYPE_CLUSTER_GROUPID  L"ClusterGroupId"
#define CLUS_NAME_RES_TYPE_REPLICATION_GROUP_TYPE  L"ReplicationClusterGroupType"
#define CLUS_NAME_RES_TYPE_MINIMUM_LOG_SIZE  L"MinimumLogSizeInBytes"
#define CLUS_NAME_RES_TYPE_UNIT_LOG_SIZE_CHANGE  L"UnitOfLogSizeChangeInBytes"
#define CLUS_NAME_RES_TYPE_LOG_MULTIPLE  L"LogSizeMultiple"

#define CLUSCTL_RESOURCE_TYPE_STORAGE_GET_AVAILABLE_DISKS_EX2_FLAG_ADD_VOLUME_INFO  0x00000001
#define CLUSCTL_RESOURCE_TYPE_STORAGE_GET_AVAILABLE_DISKS_EX2_FLAG_FILTER_BY_POOL  0x00000002
#define CLUSCTL_RESOURCE_TYPE_STORAGE_GET_AVAILABLE_DISKS_EX2_FLAG_INCLUDE_NON_SHARED_DISKS  0x00000004


#ifdef __cplusplus
}
#endif

#ifndef _CLUSTER_API_TYPES_
#define _CLUSTER_API_TYPES_
#endif /* _CLUSTER_API_TYPES_ */

#endif /* _CLUSAPI_H */
